@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -34,7 +34,7 @@ int CBuffer::AddLine(const CString& sPre, const CString& sPost, bool bIncNick) {
 		return 0;
 	}
 
-	if (size() >= m_uLineCount) {
+	while (size() >= m_uLineCount) {
 		erase(begin());
 	}
 
@@ -74,3 +74,12 @@ bool CBuffer::GetNextLine(const CString& sTarget, CString& sRet) {
 	erase(begin());
 	return true;
 }
+
+void CBuffer::SetLineCount(unsigned int u) {
+	m_uLineCount = u;
+
+	// We may need to shrink the buffer if the allowed size got smaller
+	while (size() > m_uLineCount) {
+		erase(begin());
+	}
+}
