@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -34,13 +34,8 @@ CChan::~CChan() {
 }
 
 void CChan::Reset() {
-	m_bWhoDone = false;
 	m_bIsOn = false;
-	m_suUserPerms.clear();
 	m_musModes.clear();
-	m_muuPermCount.clear();
-	m_uLimit = 0;
-	m_uClientRequests = 0;
 	m_sTopic = "";
 	m_sTopicOwner = "";
 	m_ulTopicDate = 0;
@@ -55,16 +50,18 @@ bool CChan::WriteConfig(CFile& File) {
 		return false;
 	}
 
-	File.Write("\t<Chan " + GetName() + ">\n");
+	File.Write("\t<Chan " + GetName().FirstLine() + ">\n");
 
 	if (m_pUser->GetBufferCount() != GetBufferCount())
-		File.Write("\t\tBuffer     = " + CString(GetBufferCount()) + "\n");
+		m_pUser->PrintLine(File, "\tBuffer", CString(GetBufferCount()));
 	if (m_pUser->KeepBuffer() != KeepBuffer())
-		File.Write("\t\tKeepBuffer = " + CString(KeepBuffer()) + "\n");
+		m_pUser->PrintLine(File, "\tKeepBuffer", CString(KeepBuffer()));
 	if (IsDetached())
-		File.Write("\t\tDetached   = true\n");
-	if (!GetKey().empty()) { File.Write("\t\tKey        = " + GetKey() + "\n"); }
-	if (!GetDefaultModes().empty()) { File.Write("\t\tModes      = " + GetDefaultModes() + "\n"); }
+		m_pUser->PrintLine(File, "\tDetached", "true");
+	if (!GetKey().empty())
+		m_pUser->PrintLine(File, "\tKey", GetKey());
+	if (!GetDefaultModes().empty())
+		m_pUser->PrintLine(File, "\tModes", GetDefaultModes());
 
 	File.Write("\t</Chan>\n");
 	return true;
@@ -98,8 +95,6 @@ void CChan::Cycle() const {
 
 void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 	if (!bForce && (!IsOn() || !IsDetached())) {
-		IncClientRequests();
-
 		m_pUser->PutIRC("JOIN " + GetName() + " " + ((sKey.empty()) ? GetKey() : sKey));
 		return;
 	}
@@ -203,32 +198,9 @@ CString CChan::GetModeForNames() const {
 
 void CChan::SetModes(const CString& sModes) {
 	m_musModes.clear();
-	m_uLimit = 0;
 	ModeChange(sModes);
 }
 
-void CChan::IncClientRequests() {
-	m_uClientRequests++;
-}
-
-bool CChan::DecClientRequests() {
-	if (!m_uClientRequests) {
-		return false;
-	}
-
-	m_uClientRequests--;
-	return true;
-}
-
-bool CChan::Who() {
-	if (m_bWhoDone) {
-		return false;
-	}
-
-	m_pUser->PutIRC("WHO " + GetName());
-	return true;
-}
-
 void CChan::OnWho(const CString& sNick, const CString& sIdent, const CString& sHost) {
 	CNick* pNick = FindNick(sNick);
 
@@ -266,17 +238,13 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 
 				if (uPerm) {
 					if (bAdd) {
-						if (pNick->AddPerm(uPerm)) {
-							IncPermCount(uPerm);
-						}
+						pNick->AddPerm(uPerm);
 
 						if (pNick->GetNick().Equals(m_pUser->GetCurNick())) {
 							AddPerm(uPerm);
 						}
 					} else {
-						if (pNick->RemPerm(uPerm)) {
-							DecPermCount(uPerm);
-						}
+						pNick->RemPerm(uPerm);
 
 						if (pNick->GetNick().Equals(m_pUser->GetCurNick())) {
 							RemPerm(uPerm);
@@ -406,26 +374,13 @@ void CChan::ClearNicks() {
 
 int CChan::AddNicks(const CString& sNicks) {
 	int iRet = 0;
-	CString sCurNick;
+	VCString vsNicks;
+	VCString::iterator it;
 
-	for (unsigned int a = 0; a < sNicks.size(); a++) {
-		switch (sNicks[a]) {
-			case ' ':
-				if (AddNick(sCurNick)) {
-					iRet++;
-				}
+	sNicks.Split(" ", vsNicks, false);
 
-				sCurNick = "";
-
-				break;
-			default:
-				sCurNick += sNicks[a];
-				break;
-		}
-	}
-
-	if (!sCurNick.empty()) {
-		if (AddNick(sCurNick)) {
+	for (it = vsNicks.begin(); it != vsNicks.end(); it++) {
+		if (AddNick(*it)) {
 			iRet++;
 		}
 	}
@@ -446,9 +401,13 @@ bool CChan::AddNick(const CString& sNick) {
 	}
 
 	sTmp = p;
-	sIdent = sTmp.Token(1, true, "!").Token(0, false, "@");
-	sHost = sTmp.Token(1, true, "@");
-	sTmp = sTmp.Token(0, false, "!");
+
+	// The UHNames extension gets us nick!ident@host instead of just plain nick
+	sIdent = sTmp.Token(1, true, "!");
+	sHost  = sIdent.Token(1, true, "@");
+	sIdent = sIdent.Token(0, false, "@");
+	// Get the nick
+	sTmp   = sTmp.Token(0, false, "!");
 
 	CNick* pNick = FindNick(sTmp);
 	if (!pNick) {
@@ -462,9 +421,7 @@ bool CChan::AddNick(const CString& sNick) {
 		pNick->SetHost(sHost);
 
 	for (CString::size_type i = 0; i < sPrefix.length(); i++) {
-		if (pNick->AddPerm(sPrefix[i])) {
-			IncPermCount(sPrefix[i]);
-		}
+		pNick->AddPerm(sPrefix[i]);
 	}
 
 	if (pNick->GetNick().Equals(m_pUser->GetCurNick())) {
@@ -478,31 +435,19 @@ bool CChan::AddNick(const CString& sNick) {
 	return true;
 }
 
-unsigned int CChan::GetPermCount(unsigned char uPerm) const {
-	map<unsigned char, unsigned int>::const_iterator it = m_muuPermCount.find(uPerm);
-	return (it == m_muuPermCount.end()) ? 0 : it->second;
-}
+map<char, unsigned int> CChan::GetPermCounts() const {
+	map<char, unsigned int> mRet;
 
-void CChan::DecPermCount(unsigned char uPerm) {
-	map<unsigned char, unsigned int>::iterator it = m_muuPermCount.find(uPerm);
+	map<CString,CNick*>::const_iterator it;
+	for (it = m_msNicks.begin(); it != m_msNicks.end(); it++) {
+		CString sPerms = it->second->GetPermStr();
 
-	if (it == m_muuPermCount.end()) {
-		m_muuPermCount[uPerm] = 0;
-	} else {
-		if (it->second) {
-			m_muuPermCount[uPerm]--;
+		for (unsigned int p = 0; p < sPerms.size(); p++) {
+			mRet[sPerms[p]]++;
 		}
 	}
-}
 
-void CChan::IncPermCount(unsigned char uPerm) {
-	map<unsigned char, unsigned int>::iterator it = m_muuPermCount.find(uPerm);
-
-	if (it == m_muuPermCount.end()) {
-		m_muuPermCount[uPerm] = 1;
-	} else {
-		m_muuPermCount[uPerm]++;
-	}
+	return mRet;
 }
 
 bool CChan::RemNick(const CString& sNick) {
@@ -514,12 +459,6 @@ bool CChan::RemNick(const CString& sNick) {
 		return false;
 	}
 
-	const set<unsigned char>& suPerms = it->second->GetChanPerms();
-
-	for (it2 = suPerms.begin(); it2 != suPerms.end(); it2++) {
-		DecPermCount(*it2);
-	}
-
 	delete it->second;
 	m_msNicks.erase(it);
 
@@ -571,17 +510,29 @@ void CChan::SendBuffer(CClient* pClient) {
 		const vector<CString>& vsBuffer = GetBuffer();
 
 		if (vsBuffer.size()) {
-			m_pUser->PutUser(":***!znc@znc.in PRIVMSG " + GetName() + " :Buffer Playback...", pClient);
+			bool bSkipStatusMsg = false;
+			MODULECALL(OnChanBufferStarting(*this, *pClient), m_pUser, NULL, bSkipStatusMsg = true);
+
+			if (!bSkipStatusMsg) {
+				m_pUser->PutUser(":***!znc@znc.in PRIVMSG " + GetName() + " :Buffer Playback...", pClient);
+			}
 
 			for (unsigned int a = 0; a < vsBuffer.size(); a++) {
-				m_pUser->PutUser(vsBuffer[a], pClient);
+				CString sLine(vsBuffer[a]);
+				MODULECALL(OnChanBufferPlayLine(*this, *pClient, sLine), m_pUser, NULL, continue);
+				m_pUser->PutUser(sLine, pClient);
 			}
 
 			if (!KeepBuffer()) {
 				ClearBuffer();
 			}
 
-			m_pUser->PutUser(":***!znc@znc.in PRIVMSG " + GetName() + " :Playback Complete.", pClient);
+			bSkipStatusMsg = false;
+			MODULECALL(OnChanBufferEnding(*this, *pClient), m_pUser, NULL, bSkipStatusMsg = true);
+
+			if (!bSkipStatusMsg) {
+				m_pUser->PutUser(":***!znc@znc.in PRIVMSG " + GetName() + " :Playback Complete.", pClient);
+			}
 		}
 	}
 }
