@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -63,15 +63,16 @@ CClient::~CClient() {
 void CClient::ReadLine(const CString& sData) {
 	CString sLine = sData;
 
-	while ((sLine.Right(1) == "\r") || (sLine.Right(1) == "\n")) {
-		sLine.RightChomp();
-	}
+	sLine.TrimRight("\n\r");
 
-	DEBUG_ONLY(cout << "(" << ((m_pUser) ? m_pUser->GetUserName() : CString("")) << ") CLI -> ZNC [" << sLine << "]" << endl);
+	DEBUG("(" << ((m_pUser) ? m_pUser->GetUserName() : GetRemoteIP()) << ") CLI -> ZNC [" << sLine << "]");
 
 #ifdef _MODULES
 	if (IsAttached()) {
 		MODULECALL(OnUserRaw(sLine), m_pUser, this, return);
+	} else {
+		if (CZNC::Get().GetModules().OnUnknownUserRaw(this, sLine))
+			return;
 	}
 #endif
 
@@ -87,6 +88,8 @@ void CClient::ReadLine(const CString& sData) {
 		if (!IsAttached()) {
 			m_bGotPass = true;
 			m_sPass = sLine.Token(1);
+			if (m_sPass.Left(1) == ":")
+				m_sPass.LeftChomp();
 
 			if (m_sPass.find(":") != CString::npos) {
 				m_sUser = m_sPass.Token(0, false, ":");
@@ -281,18 +284,21 @@ void CClient::ReadLine(const CString& sData) {
 		if (m_pUser->IsChan(sTarget)) {
 			CChan *pChan = m_pUser->FindChan(sTarget);
 
-			if (pChan && sModes.empty()) {
+			// If we are on that channel and already received a
+			// /mode reply from the server, we can answer this
+			// request ourself.
+			if (pChan && pChan->IsOn() && sModes.empty() && !pChan->GetModeString().empty()) {
 				PutClient(":" + m_pUser->GetIRCServer() + " 324 " + GetNick() + " " + sTarget + " " + pChan->GetModeString());
 				if (pChan->GetCreationDate() > 0) {
 					PutClient(":" + m_pUser->GetIRCServer() + " 329 " + GetNick() + " " + sTarget + " " + CString(pChan->GetCreationDate()));
 				}
-			return;
+				return;
 			}
 		}
 	} else if (sCommand.Equals("QUIT")) {
 		m_pUser->UserDisconnected(this);
 
-		Close();	// Treat a client quit as a detach
+		Close(Csock::CLT_AFTERWRITE);	// Treat a client quit as a detach
 		return;		// Don't forward this msg.  We don't want the client getting us disconnected.
 	} else if (sCommand.Equals("PROTOCTL")) {
 		unsigned int i = 1;
@@ -385,9 +391,9 @@ void CClient::ReadLine(const CString& sData) {
 			if (sCTCP.Equals("DCC ", false, 4) && m_pUser->BounceDCCs()) {
 				CString sType = sCTCP.Token(1);
 				CString sFile = sCTCP.Token(2);
-				unsigned long uLongIP = strtoul(sCTCP.Token(3).c_str(), NULL, 10);
-				unsigned short uPort = strtoul(sCTCP.Token(4).c_str(), NULL, 10);
-				unsigned long uFileSize = strtoul(sCTCP.Token(5).c_str(), NULL, 10);
+				unsigned long uLongIP = sCTCP.Token(3).ToULong();
+				unsigned short uPort = sCTCP.Token(4).ToUShort();
+				unsigned long uFileSize = sCTCP.Token(5).ToULong();
 				CString sIP = (m_pIRCSock) ? m_pIRCSock->GetLocalIP() : GetLocalIP();
 
 				if (!m_pUser->UseClientIP()) {
@@ -415,9 +421,14 @@ void CClient::ReadLine(const CString& sData) {
 								return;
 							}
 
-							CString sLocalFile = sPath + "/" + sFile;
+							CString sAbsolutePath = CDir::CheckPathPrefix(sPath, sFile);
 
-							m_pUser->GetFile(GetNick(), CUtils::GetIP(uLongIP), uPort, sLocalFile, uFileSize);
+							if (sAbsolutePath.empty()) {
+								PutStatus("Illegal path.");
+								return;
+							}
+
+							m_pUser->GetFile(GetNick(), CUtils::GetIP(uLongIP), uPort, sAbsolutePath, uFileSize);
 						} else {
 							MODULECALL(OnDCCUserSend(CString(m_pUser->GetStatusPrefix() + sTarget), uLongIP, uPort, sFile, uFileSize), m_pUser, this, return);
 						}
@@ -429,8 +440,8 @@ void CClient::ReadLine(const CString& sData) {
 					}
 				} else if (sType.Equals("RESUME")) {
 					// PRIVMSG user :DCC RESUME "znc.o" 58810 151552
-					unsigned short uResumePort = atoi(sCTCP.Token(3).c_str());
-					unsigned long uResumeSize = strtoul(sCTCP.Token(4).c_str(), NULL, 10);
+					unsigned short uResumePort = sCTCP.Token(3).ToUShort();
+					unsigned long uResumeSize = sCTCP.Token(4).ToULong();
 
 					// Need to lookup the connection by port, filter the port, and forward to the user
 					CString sStatusPrefix = m_pUser->GetStatusPrefix();
@@ -456,7 +467,7 @@ void CClient::ReadLine(const CString& sData) {
 							CDCCBounce* pSock = (CDCCBounce*) Manager[a];
 
 							if (pSock && pSock->GetSockName().Equals("DCC::", false, 5)) {
-								if (pSock->GetUserPort() == atoi(sCTCP.Token(3).c_str())) {
+								if (pSock->GetUserPort() == sCTCP.Token(3).ToUShort()) {
 									PutIRC("PRIVMSG " + sTarget + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetLocalPort()) + " " + sCTCP.Token(4) + "\001");
 								}
 							}
@@ -511,7 +522,6 @@ void CClient::ReadLine(const CString& sData) {
 
 		if (sTarget.TrimPrefix(m_pUser->GetStatusPrefix())) {
 			if (sTarget.Equals("status")) {
-				MODULECALL(OnStatusCommand(sMsg), m_pUser, this, return);
 				UserCommand(sMsg);
 			} else {
 #ifdef _MODULES
@@ -614,17 +624,37 @@ void CClient::AuthUser() {
 }
 
 CClientAuth::CClientAuth(CClient* pClient, const CString& sUsername, const CString& sPassword)
-		: CAuthBase(sUsername, sPassword, pClient->GetRemoteIP()) {
+		: CAuthBase(sUsername, sPassword, pClient) {
 	m_pClient = pClient;
 }
 
-void CClientAuth::RefuseLogin(const CString& sReason) {
+void CClientAuth::RefusedLogin(const CString& sReason) {
 	if (m_pClient) {
 		m_pClient->RefuseLogin(sReason);
 	}
+}
+
+CString CAuthBase::GetRemoteIP() const {
+	if (m_pSock)
+		return m_pSock->GetRemoteIP();
+	return "";
+}
+
+void CAuthBase::RefuseLogin(const CString& sReason) {
+	CUser* pUser = CZNC::Get().GetUser(GetUsername());
+
+	// If the username is valid, notify that user that someone tried to
+	// login. Use sReason because there are other reasons than "wrong
+	// password" for a login to be rejected (e.g. fail2ban).
+	if (pUser) {
+		pUser->PutStatus("A client from [" + GetRemoteIP() + "] attempted "
+				"to login as you, but was rejected [" + sReason + "].");
+	}
+
 #ifdef _MODULES
 	CZNC::Get().GetModules().OnFailedLogin(GetUsername(), GetRemoteIP());
 #endif
+	RefusedLogin(sReason);
 }
 
 void CClient::RefuseLogin(const CString& sReason) {
@@ -635,10 +665,10 @@ void CClient::RefuseLogin(const CString& sReason) {
 
 	PutStatus("Bad username and/or password.");
 	PutClient(":irc.znc.in 464 " + GetNick() + " :" + sReason);
-	Close();
+	Close(Csock::CLT_AFTERWRITE);
 }
 
-void CClientAuth::AcceptLogin(CUser& User) {
+void CClientAuth::AcceptedLogin(CUser& User) {
 	if (m_pClient) {
 		m_pClient->AcceptLogin(User);
 	}
@@ -678,15 +708,16 @@ void CClient::LoginTimeout() {
 }
 
 void CClient::Connected() {
-	DEBUG_ONLY(cout << GetSockName() << " == Connected();" << endl);
-	SetTimeout(900);	// Now that we are connected, let nature take its course
+	DEBUG(GetSockName() << " == Connected();");
+	SetTimeout(240, TMO_READ);	// Now that we are connected, let nature take its course
 }
 
 void CClient::ConnectionRefused() {
-	DEBUG_ONLY(cout << GetSockName() << " == ConnectionRefused()" << endl);
+	DEBUG(GetSockName() << " == ConnectionRefused()");
 }
 
 void CClient::Disconnected() {
+	DEBUG(GetSockName() << " == Disconnected()");
 	if (m_pUser) {
 		m_pUser->UserDisconnected(this);
 	}
@@ -697,7 +728,7 @@ void CClient::Disconnected() {
 }
 
 void CClient::ReachedMaxBuffer() {
-	DEBUG_ONLY(cout << GetSockName() << " == ReachedMaxBuffer()" << endl);
+	DEBUG(GetSockName() << " == ReachedMaxBuffer()");
 	if (IsAttached()) {
 		PutClient("ERROR :Closing link [Too long raw line]");
 	}
@@ -711,7 +742,7 @@ void CClient::IRCConnected(CIRCSock* pIRCSock) {
 void CClient::BouncedOff() {
 	PutStatusNotice("You are being disconnected because another user just authenticated as you.");
 	m_pIRCSock = NULL;
-	Close();
+	Close(Csock::CLT_AFTERWRITE);
 }
 
 void CClient::IRCDisconnected() {
@@ -725,7 +756,7 @@ void CClient::PutIRC(const CString& sLine) {
 }
 
 void CClient::PutClient(const CString& sLine) {
-	DEBUG_ONLY(cout << "(" << ((m_pUser) ? m_pUser->GetUserName() : CString("")) << ") ZNC -> CLI [" << sLine << "]" << endl);
+	DEBUG("(" << ((m_pUser) ? m_pUser->GetUserName() : GetRemoteIP()) << ") ZNC -> CLI [" << sLine << "]");
 	Write(sLine + "\r\n");
 }
 
@@ -750,7 +781,7 @@ void CClient::PutModNotice(const CString& sModule, const CString& sLine) {
 		return;
 	}
 
-	DEBUG_ONLY(cout << "(" << m_pUser->GetUserName() << ") ZNC -> CLI [:" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in NOTICE " << GetNick() << " :" << sLine << "]" << endl);
+	DEBUG("(" << m_pUser->GetUserName() << ") ZNC -> CLI [:" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in NOTICE " << GetNick() << " :" << sLine << "]");
 	Write(":" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in NOTICE " + GetNick() + " :" + sLine + "\r\n");
 }
 
@@ -759,7 +790,7 @@ void CClient::PutModule(const CString& sModule, const CString& sLine) {
 		return;
 	}
 
-	DEBUG_ONLY(cout << "(" << m_pUser->GetUserName() << ") ZNC -> CLI [:" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in PRIVMSG " << GetNick() << " :" << sLine << "]" << endl);
+	DEBUG("(" << m_pUser->GetUserName() << ") ZNC -> CLI [:" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in PRIVMSG " << GetNick() << " :" << sLine << "]");
 	Write(":" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in PRIVMSG " + GetNick() + " :" + sLine + "\r\n");
 }
 
