@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,7 +9,7 @@
 #ifndef _CLIENT_H
 #define _CLIENT_H
 
-#include "Csocket.h"
+#include "Socket.h"
 #include "Utils.h"
 #include "main.h"
 
@@ -23,30 +23,35 @@ class CClientTimeout;
 
 class CAuthBase {
 public:
-	CAuthBase(const CString& sUsername, const CString& sPassword, const CString& sRemoteIP) {
-		SetLoginInfo(sUsername, sPassword, sRemoteIP);
+	CAuthBase(const CString& sUsername, const CString& sPassword, Csock *pSock) {
+		SetLoginInfo(sUsername, sPassword, pSock);
 	}
 
 	virtual ~CAuthBase() {}
 
 	virtual void SetLoginInfo(const CString& sUsername, const CString& sPassword,
-			const CString& sRemoteIP) {
+			Csock *pSock) {
 		m_sUsername = sUsername;
 		m_sPassword = sPassword;
-		m_sRemoteIP = sRemoteIP;
+		m_pSock = pSock;
 	}
 
-	virtual void AcceptLogin(CUser& User) = 0;
-	virtual void RefuseLogin(const CString& sReason) = 0;
+	void AcceptLogin(CUser& User) { AcceptedLogin(User); }
+	void RefuseLogin(const CString& sReason);
 
-	virtual const CString& GetUsername() const { return m_sUsername; }
-	virtual const CString& GetPassword() const { return m_sPassword; }
-	virtual const CString& GetRemoteIP() const { return m_sRemoteIP; }
+	const CString& GetUsername() const { return m_sUsername; }
+	const CString& GetPassword() const { return m_sPassword; }
+	Csock *GetSocket() const { return m_pSock; }
+	CString GetRemoteIP() const;
+
+protected:
+	virtual void AcceptedLogin(CUser& User) = 0;
+	virtual void RefusedLogin(const CString& sReason) = 0;
 
 private:
 	CString		m_sUsername;
 	CString		m_sPassword;
-	CString		m_sRemoteIP;
+	Csock*		m_pSock;
 };
 
 
@@ -56,16 +61,16 @@ public:
 	virtual ~CClientAuth() {}
 
 	void SetClient(CClient* pClient) { m_pClient = pClient; }
-	void AcceptLogin(CUser& User);
-	void RefuseLogin(const CString& sReason);
+	void AcceptedLogin(CUser& User);
+	void RefusedLogin(const CString& sReason);
 private:
 protected:
 	CClient*	m_pClient;
 };
 
-class CClient : public Csock {
+class CClient : public CZNCSock {
 public:
-	CClient(const CString& sHostname, unsigned short uPort, int iTimeout = 60) : Csock(sHostname, uPort, iTimeout) {
+	CClient(const CString& sHostname, unsigned short uPort, int iTimeout = 60) : CZNCSock(sHostname, uPort, iTimeout) {
 		m_pUser = NULL;
 		m_pTimeout = NULL;
 		m_pIRCSock = NULL;
@@ -80,6 +85,8 @@ public:
 		SetMaxBufferThreshold(1024);
 
 		StartLoginTimeout();
+
+		SetNick("unknown-nick");
 	}
 
 	virtual ~CClient();
@@ -94,7 +101,7 @@ public:
 	bool HasNamesx() const { return m_bNamesx; }
 	bool HasUHNames() const { return m_bUHNames; }
 
-	void UserCommand(const CString& sCommand);
+	void UserCommand(CString& sCommand);
 	void StatusCTCP(const CString& sCommand);
 	void IRCConnected(CIRCSock* pIRCSock);
 	void IRCDisconnected();
