@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -15,7 +15,7 @@
 #include "User.h"
 #include "znc.h"
 
-void CClient::UserCommand(const CString& sLine) {
+void CClient::UserCommand(CString& sLine) {
 	if (!m_pUser) {
 		return;
 	}
@@ -24,6 +24,8 @@ void CClient::UserCommand(const CString& sLine) {
 		return;
 	}
 
+	MODULECALL(OnStatusCommand(sLine), m_pUser, this, return);
+
 	CString sCommand = sLine.Token(0);
 
 	if (sCommand.Equals("HELP")) {
@@ -210,8 +212,6 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 
 		CZNC::Get().Broadcast(sMessage);
-		usleep(100000);	// Sleep for 10ms to attempt to allow the previous Broadcast() to go through to all users
-
 		throw CException(CException::EX_Shutdown);
 	} else if (m_pUser->IsAdmin() && sCommand.Equals("RESTART")) {
 		CString sMessage = sLine.Token(1, true);
@@ -305,10 +305,10 @@ void CClient::UserCommand(const CString& sLine) {
 			Table.SetCell("Modes", pChan->GetModeString());
 			Table.SetCell("Users", CString(pChan->GetNickCount()));
 
+			map<char, unsigned int> mPerms = pChan->GetPermCounts();
 			for (unsigned int b = 0; b < sPerms.size(); b++) {
-				CString sPerm;
-				sPerm += sPerms[b];
-				Table.SetCell(sPerm, CString(pChan->GetPermCount(sPerms[b])));
+				char cPerm = sPerms[b];
+				Table.SetCell(CString(cPerm), CString(mPerms[cPerm]));
 			}
 		}
 
@@ -321,15 +321,11 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		if (m_pUser->FindServer(sServer)) {
-			PutStatus("That server already exists");
-			return;
-		}
-
 		if (m_pUser->AddServer(sLine.Token(1, true))) {
 			PutStatus("Server added");
 		} else {
 			PutStatus("Unable to add that server");
+			PutStatus("Perhaps the server is already added or openssl is disabled?");
 		}
 	} else if (sCommand.Equals("REMSERVER") || sCommand.Equals("DELSERVER")) {
 		CString sServer = sLine.Token(1);
@@ -354,6 +350,7 @@ void CClient::UserCommand(const CString& sLine) {
 	} else if (sCommand.Equals("LISTSERVERS")) {
 		if (m_pUser->HasServers()) {
 			const vector<CServer*>& vServers = m_pUser->GetServers();
+			CServer* pCurServ = m_pUser->GetCurrentServer();
 			CTable Table;
 			Table.AddColumn("Host");
 			Table.AddColumn("Port");
@@ -363,7 +360,7 @@ void CClient::UserCommand(const CString& sLine) {
 			for (unsigned int a = 0; a < vServers.size(); a++) {
 				CServer* pServer = vServers[a];
 				Table.AddRow();
-				Table.SetCell("Host", pServer->GetName());
+				Table.SetCell("Host", pServer->GetName() + (pServer == pCurServ ? "*" : ""));
 				Table.SetCell("Port", CString(pServer->GetPort()));
 				Table.SetCell("SSL", (pServer->IsSSL()) ? "SSL" : "");
 				Table.SetCell("Pass", pServer->GetPass());
@@ -400,9 +397,9 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		sAbsolutePath = CDir::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
+		sAbsolutePath = CDir::CheckPathPrefix(sAllowedPath, sFile);
 
-		if (sAbsolutePath.Left(sAllowedPath.length()) != sAllowedPath) {
+		if (sAbsolutePath.empty()) {
 			PutStatus("Illegal path.");
 			return;
 		}
@@ -418,9 +415,9 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		sAbsolutePath = CDir::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
+		sAbsolutePath = CDir::CheckPathPrefix(sAllowedPath, sFile);
 
-		if (sAbsolutePath.Left(sAllowedPath.length()) != sAllowedPath) {
+		if (sAbsolutePath.empty()) {
 			PutStatus("Illegal path.");
 			return;
 		}
@@ -529,12 +526,12 @@ void CClient::UserCommand(const CString& sLine) {
 				PutStatus("Global modules:");
 				CTable GTable;
 				GTable.AddColumn("Name");
-				GTable.AddColumn("Description");
+				GTable.AddColumn("Arguments");
 
 				for (unsigned int b = 0; b < GModules.size(); b++) {
 					GTable.AddRow();
 					GTable.SetCell("Name", GModules[b]->GetModName());
-					GTable.SetCell("Description", GModules[b]->GetDescription().Ellipsize(128));
+					GTable.SetCell("Arguments", GModules[b]->GetArgs());
 				}
 
 				PutStatus(GTable);
@@ -544,17 +541,17 @@ void CClient::UserCommand(const CString& sLine) {
 		CModules& Modules = m_pUser->GetModules();
 
 		if (!Modules.size()) {
-			PutStatus("You have no modules loaded.");
+			PutStatus("Your user has no modules loaded.");
 		} else {
 			PutStatus("User modules:");
 			CTable Table;
 			Table.AddColumn("Name");
-			Table.AddColumn("Description");
+			Table.AddColumn("Arguments");
 
 			for (unsigned int b = 0; b < Modules.size(); b++) {
 				Table.AddRow();
 				Table.SetCell("Name", Modules[b]->GetModName());
-				Table.SetCell("Description", Modules[b]->GetDescription().Ellipsize(128));
+				Table.SetCell("Arguments", Modules[b]->GetArgs());
 			}
 
 			PutStatus(Table);
@@ -622,30 +619,31 @@ void CClient::UserCommand(const CString& sLine) {
 	} else if (sCommand.Equals("LOADMOD") || sCommand.Equals("LOADMODULE")) {
 		CString sMod;
 		CString sArgs;
-		bool bGlobal = false;
 
-		if (sLine.Token(1).Equals("-global")) {
-			sMod = sLine.Token(2);
-
-			if (!m_pUser->IsAdmin()) {
-				PutStatus("Unable to load global module [" + sMod + "] Access Denied.");
-				return;
-			}
-
-			sArgs = sLine.Token(3, true);
-			bGlobal = true;
-		} else {
-			sMod = sLine.Token(1);
-			sArgs = sLine.Token(2, true);
-		}
+		sMod = sLine.Token(1);
+		sArgs = sLine.Token(2, true);
 
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Unable to load [" + sMod + "] Access Denied.");
 			return;
 		}
-#ifdef _MODULES
+
 		if (sMod.empty()) {
-			PutStatus("Usage: LoadMod [-global] <module> [args]");
+			PutStatus("Usage: LoadMod <module> [args]");
+			return;
+		}
+
+#ifdef _MODULES
+		CModInfo ModInfo;
+		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod)) {
+			PutStatus("Unable to find modinfo [" + sMod + "]");
+			return;
+		}
+
+		bool bGlobal = ModInfo.IsGlobal();
+
+		if (bGlobal && !m_pUser->IsAdmin()) {
+			PutStatus("Unable to load global module [" + sMod + "] Access Denied.");
 			return;
 		}
 
@@ -657,8 +655,9 @@ void CClient::UserCommand(const CString& sLine) {
 		} else {
 			b = m_pUser->GetModules().LoadModule(sMod, sArgs, m_pUser, sModRet);
 		}
+
 		if (!b) {
-			PutStatus("Unable to load module [" + sMod + "] [" + sModRet + "]");
+			PutStatus(sModRet);
 			return;
 		}
 
@@ -669,27 +668,28 @@ void CClient::UserCommand(const CString& sLine) {
 		return;
 	} else if (sCommand.Equals("UNLOADMOD") || sCommand.Equals("UNLOADMODULE")) {
 		CString sMod;
-		bool bGlobal = false;
-
-		if (sLine.Token(1).Equals("-global")) {
-			sMod = sLine.Token(2);
-
-			if (!m_pUser->IsAdmin()) {
-				PutStatus("Unable to unload global module [" + sMod + "] Access Denied.");
-				return;
-			}
-
-			bGlobal = true;
-		} else
-			sMod = sLine.Token(1);
+		sMod = sLine.Token(1);
 
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Unable to unload [" + sMod + "] Access Denied.");
 			return;
 		}
 #ifdef _MODULES
+		CModInfo ModInfo;
+		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod)) {
+			PutStatus("Unable to find modinfo for [" + sMod + "]");
+			return;
+		}
+
+		bool bGlobal = ModInfo.IsGlobal();
+
+		if (bGlobal && !m_pUser->IsAdmin()) {
+			PutStatus("Unable to unload global module [" + sMod + "] Access Denied.");
+			return;
+		}
+
 		if (sMod.empty()) {
-			PutStatus("Usage: UnloadMod [-global] <module>");
+			PutStatus("Usage: UnloadMod <module>");
 			return;
 		}
 
@@ -709,30 +709,30 @@ void CClient::UserCommand(const CString& sLine) {
 	} else if (sCommand.Equals("RELOADMOD") || sCommand.Equals("RELOADMODULE")) {
 		CString sMod;
 		CString sArgs;
-		bool bGlobal = false;
 
-		if (sLine.Token(1).Equals("-global")) {
-			sMod = sLine.Token(2);
-
-			if (!m_pUser->IsAdmin()) {
-				PutStatus("Unable to reload global module [" + sMod + "] Access Denied.");
-				return;
-			}
-
-			sArgs = sLine.Token(3, true);
-			bGlobal = true;
-		} else {
-			sMod = sLine.Token(1);
-			sArgs = sLine.Token(2, true);
-		}
+		sMod = sLine.Token(1);
+		sArgs = sLine.Token(2, true);
 
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Unable to reload [" + sMod + "] Access Denied.");
 			return;
 		}
 #ifdef _MODULES
+		CModInfo ModInfo;
+		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod)) {
+			PutStatus("Unable to find modinfo for [" + sMod + "]");
+			return;
+		}
+
+		bool bGlobal = ModInfo.IsGlobal();
+
+		if (bGlobal && !m_pUser->IsAdmin()) {
+			PutStatus("Unable to reload global module [" + sMod + "] Access Denied.");
+			return;
+		}
+
 		if (sMod.empty()) {
-			PutStatus("Usage: ReloadMod [-global] <module> [args]");
+			PutStatus("Usage: ReloadMod <module> [args]");
 			return;
 		}
 
@@ -749,6 +749,25 @@ void CClient::UserCommand(const CString& sLine) {
 		PutStatus("Unable to unload [" + sMod + "] Modules are not enabled.");
 #endif
 		return;
+	} else if ((sCommand.Equals("UPDATEMOD") || sCommand.Equals("UPDATEMODULE")) && m_pUser->IsAdmin() ) {
+#ifndef _MODULES
+		PutStatus("Modules are not enabled.");
+#else
+		CString sMod = sLine.Token(1);
+
+		if (m_pUser->DenyLoadMod() || !m_pUser->IsAdmin()) {
+			PutStatus("Unable to reload [" + sMod + "] Access Denied.");
+			return;
+		}
+
+		PutStatus("Reloading [" + sMod + "] on all users...");
+		if (CUser::UpdateModule(sMod)) {
+			PutStatus("Done");
+		} else {
+			PutStatus("Done, but there were errors, some users no longer have ["
+					+ sMod + "] loaded");
+		}
+#endif
 	} else if (sCommand.Equals("ADDVHOST") && m_pUser->IsAdmin()) {
 		CString sVHost = sLine.Token(1);
 
@@ -899,12 +918,8 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		if (!pChan->IsOn()) {
-			PutStatus("You are not on [" + sChan + "] [trying]");
-			return;
-		}
 
-		unsigned int uLineCount = strtoul(sLine.Token(2).c_str(), NULL, 10);
+		unsigned int uLineCount = sLine.Token(2).ToUInt();
 
 		if (uLineCount > 500) {
 			PutStatus("Max linecount is 500.");
@@ -915,44 +930,44 @@ void CClient::UserCommand(const CString& sLine) {
 
 		PutStatus("BufferCount for [" + sChan + "] set to [" + CString(pChan->GetBufferCount()) + "]");
 	} else if (m_pUser->IsAdmin() && sCommand.Equals("TRAFFIC")) {
-		CZNC::Get().UpdateTrafficStats();
-		const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
+		CZNC::TrafficStatsPair Users, ZNC, Total;
+		CZNC::TrafficStatsMap traffic = CZNC::Get().GetTrafficStats(Users, ZNC, Total);
+		CZNC::TrafficStatsMap::const_iterator it;
+
 		CTable Table;
 		Table.AddColumn("Username");
 		Table.AddColumn("In");
 		Table.AddColumn("Out");
 		Table.AddColumn("Total");
-		unsigned long long users_total_in = 0;
-		unsigned long long users_total_out = 0;
-		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+
+		for (it = traffic.begin(); it != traffic.end(); it++) {
 			Table.AddRow();
 			Table.SetCell("Username", it->first);
-			Table.SetCell("In", CString::ToByteStr(it->second->BytesRead()));
-			Table.SetCell("Out", CString::ToByteStr(it->second->BytesWritten()));
-			Table.SetCell("Total", CString::ToByteStr(it->second->BytesRead() + it->second->BytesWritten()));
-			users_total_in += it->second->BytesRead();
-			users_total_out += it->second->BytesWritten();
+			Table.SetCell("In", CString::ToByteStr(it->second.first));
+			Table.SetCell("Out", CString::ToByteStr(it->second.second));
+			Table.SetCell("Total", CString::ToByteStr(it->second.first + it->second.second));
 		}
+
 		Table.AddRow();
 		Table.SetCell("Username", "<Users>");
-		Table.SetCell("In", CString::ToByteStr(users_total_in));
-		Table.SetCell("Out", CString::ToByteStr(users_total_out));
-		Table.SetCell("Total", CString::ToByteStr(users_total_in + users_total_out));
+		Table.SetCell("In", CString::ToByteStr(Users.first));
+		Table.SetCell("Out", CString::ToByteStr(Users.second));
+		Table.SetCell("Total", CString::ToByteStr(Users.first + Users.second));
 
 		Table.AddRow();
 		Table.SetCell("Username", "<ZNC>");
-		Table.SetCell("In", CString::ToByteStr(CZNC::Get().BytesRead()));
-		Table.SetCell("Out", CString::ToByteStr(CZNC::Get().BytesWritten()));
-		Table.SetCell("Total", CString::ToByteStr(CZNC::Get().BytesRead() + CZNC::Get().BytesWritten()));
+		Table.SetCell("In", CString::ToByteStr(ZNC.first));
+		Table.SetCell("Out", CString::ToByteStr(ZNC.second));
+		Table.SetCell("Total", CString::ToByteStr(ZNC.first + ZNC.second));
 
 		Table.AddRow();
 		Table.SetCell("Username", "<Total>");
-		Table.SetCell("In", CString::ToByteStr(users_total_in + CZNC::Get().BytesRead()));
-		Table.SetCell("Out", CString::ToByteStr(users_total_out + CZNC::Get().BytesWritten()));
-		Table.SetCell("Total", CString::ToByteStr(users_total_in + CZNC::Get().BytesRead() + users_total_out + CZNC::Get().BytesWritten()));
+		Table.SetCell("In", CString::ToByteStr(Total.first));
+		Table.SetCell("Out", CString::ToByteStr(Total.second));
+		Table.SetCell("Total", CString::ToByteStr(Total.first + Total.second));
 
 		PutStatus(Table);
-	} else if (m_pUser->IsAdmin() && sCommand.Equals("UPTIME")) {
+	} else if (sCommand.Equals("UPTIME")) {
 		PutStatus("Running for " + CZNC::Get().GetUptime());
 	} else {
 		PutStatus("Unknown command [" + sCommand + "] try 'Help'");
@@ -1106,6 +1121,11 @@ void CClient::HelpUser() {
 	Table.SetCell("Arguments", "<file>");
 	Table.SetCell("Description", "Send a shell file to yourself");
 
+	Table.AddRow();
+	Table.SetCell("Command", "Uptime");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Show how long ZNC is already running");
+
 	if (!m_pUser->DenyLoadMod()) {
 		Table.AddRow();
 		Table.SetCell("Command", "LoadMod");
@@ -1121,6 +1141,13 @@ void CClient::HelpUser() {
 		Table.SetCell("Command", "ReloadMod");
 		Table.SetCell("Arguments", "<module>");
 		Table.SetCell("Description", "Reload a module");
+
+		if (m_pUser->IsAdmin()) {
+			Table.AddRow();
+			Table.SetCell("Command", "UpdateMod");
+			Table.SetCell("Arguments", "<module>");
+			Table.SetCell("Description", "Reload a module on all users");
+		}
 	}
 
 	Table.AddRow();
@@ -1169,11 +1196,6 @@ void CClient::HelpUser() {
 		Table.SetCell("Arguments", "");
 		Table.SetCell("Description", "Show basic traffic stats for all znc users");
 
-		Table.AddRow();
-		Table.SetCell("Command", "Uptime");
-		Table.SetCell("Arguments", "");
-		Table.SetCell("Description", "Show how long ZNC is already running");
-
 		Table.AddRow();
 		Table.SetCell("Command", "Broadcast");
 		Table.SetCell("Arguments", "[message]");
