@@ -1,6 +1,6 @@
 /** @file
 *
-*    Copyright (c) 1999-2007 Jim Hull <imaginos@imaginos.net>
+*    Copyright (c) 1999-2009 Jim Hull <imaginos@imaginos.net>
 *    All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without modification,
@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.89 $
+* $Revision: 1.99 $
 */
 
 #include "Csocket.h"
@@ -193,7 +193,7 @@ static int __GetHostByName( const CS_STRING & sHostName, struct in_addr *paddr,
 	if ( iReturn == 0 )
 		memcpy( &paddr->s_addr, hent->h_addr_list[0], sizeof( paddr->s_addr ) );
 
-	return( iReturn );
+	return( iReturn == TRY_AGAIN ? EAGAIN : iReturn );
 }
 #endif /* !HAVE_IPV6 */
 
@@ -311,6 +311,31 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 	return( ETIMEDOUT );
 }
 
+bool InitCsocket()
+{
+#ifdef _WIN32
+	WSADATA wsaData;
+	int iResult = WSAStartup( MAKEWORD( 2, 2 ), &wsaData );
+	if( iResult != NO_ERROR )
+		return( false );
+#endif /* _WIN32 */
+#ifdef HAVE_LIBSSL
+	if( !InitSSL() )
+		return( false );
+#endif /* HAVE_LIBSSL */
+	return( true );
+}
+
+void ShutdownCsocket()
+{
+#ifdef HAVE_LIBSSL
+	ERR_free_strings();
+#endif /* HAVE_LIBSSL */
+#ifdef _WIN32
+	WSACleanup();
+#endif /* _WIN32 */
+}
+
 #ifdef HAVE_LIBSSL
 bool InitSSL( ECompType eCompressionType )
 {
@@ -372,7 +397,7 @@ void SSLErrors( const char *filename, u_int iLineNum )
 
 void __Perror( const CS_STRING & s, const char *pszFile, unsigned int iLineNo )
 {
-#if defined(__sun) || defined(_WIN32) || __NetBSD_Version__ < 4000000000
+#if defined(__sun) || defined(_WIN32) || (defined(__NetBSD_Version__) && __NetBSD_Version__ < 4000000000)
 	std::cerr << s << "(" << pszFile << ":" << iLineNo << "): " << strerror( GetSockError() ) << endl;
 #else
 	char buff[512];
@@ -537,6 +562,7 @@ void Csock::Dereference()
 #endif /* HAVE_LIBSSL */
 
 	m_vcCrons.clear();
+	Close( CLT_DEREFERENCE );
 }
 
 void Csock::Copy( const Csock & cCopy )
@@ -557,7 +583,6 @@ void Csock::Copy( const Csock & cCopy )
 	m_bFullsslAccept	= cCopy.m_bFullsslAccept;
 	m_bsslEstablished	= cCopy.m_bsslEstablished;
 	m_bEnableReadLine	= cCopy.m_bEnableReadLine;
-	m_bRequireClientCert = cCopy.m_bRequireClientCert;
 	m_bPauseRead		= cCopy.m_bPauseRead;
 	m_shostname		= cCopy.m_shostname;
 	m_sbuffer		= cCopy.m_sbuffer;
@@ -587,6 +612,7 @@ void Csock::Copy( const Csock & cCopy )
 	m_bSkipConnect		= cCopy.m_bSkipConnect;
 
 #ifdef HAVE_LIBSSL
+	m_iRequireClientCertFlags = cCopy.m_iRequireClientCertFlags;
 	m_sSSLBuffer	= cCopy.m_sSSLBuffer;
 
 	FREE_SSL();
@@ -832,7 +858,7 @@ bool Csock::Listen( u_short iPort, int iMaxConns, const CS_STRING & sBindHost, u
 	m_sBindHost = sBindHost;
 	if ( !sBindHost.empty() )
 	{
-		if( GetAddrInfo( sBindHost, this, m_address ) != 0 )
+		if( GetAddrInfo( sBindHost, m_address ) != 0 )
 			return( false );
 	}
 
@@ -916,7 +942,7 @@ int Csock::Accept( CS_STRING & sHost, u_short & iRPort )
 
 		if ( !ConnectionFrom( sHost, iRPort ) )
 		{
-			close( iSock );
+			CS_CLOSE( iSock );
 			iSock = -1;
 		}
 
@@ -1004,6 +1030,8 @@ bool Csock::SSLClientSetup()
 	if ( !m_ssl_ctx )
 		return( false );
 
+	SSL_CTX_set_default_verify_paths( m_ssl_ctx );
+
 	if ( !m_sPemFile.empty() )
 	{	// are we sending a client cerificate ?
 		SSL_CTX_set_default_passwd_cb( m_ssl_ctx, PemPassCB );
@@ -1092,6 +1120,7 @@ bool Csock::SSLServerSetup()
 	m_ssl_ctx = SSL_CTX_new ( m_ssl_method );
 	if ( !m_ssl_ctx )
 		return( false );
+	SSL_CTX_set_default_verify_paths( m_ssl_ctx );
 
 	// set the pemfile password
 	SSL_CTX_set_default_passwd_cb( m_ssl_ctx, PemPassCB );
@@ -1135,9 +1164,9 @@ bool Csock::SSLServerSetup()
 	SSL_set_rfd( m_ssl, m_iReadSock );
 	SSL_set_wfd( m_ssl, m_iWriteSock );
 	SSL_set_accept_state( m_ssl );
-	if ( m_bRequireClientCert )
+	if ( m_iRequireClientCertFlags )
 	{
-		SSL_set_verify( m_ssl, SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_PEER, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
+		SSL_set_verify( m_ssl, m_iRequireClientCertFlags, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
 		SSL_set_ex_data( m_ssl, GetCsockClassIdx(), this );
 	}
 
@@ -1233,6 +1262,14 @@ bool Csock::Write( const char *data, int len )
 	// rate shaping
 	u_int iBytesToSend = 0;
 
+#ifdef HAVE_LIBSSL
+	if( m_bssl && m_sSSLBuffer.empty() && !m_bsslEstablished )
+	{
+		// to keep openssl from spinning, just initiate the connection with 1 byte so the connection establishes faster
+		iBytesToSend = 1;
+	}
+	else
+#endif /* HAVE_LIBSSL */
 	if ( ( m_iMaxBytes > 0 ) && ( m_iMaxMilliSeconds > 0 ) )
 	{
 		unsigned long long iNOW = millitime();
@@ -1874,8 +1911,8 @@ CS_STRING Csock::GetPeerPubKey()
 	}
 	return( sKey );
 }
-bool Csock::RequiresClientCert() { return( m_bRequireClientCert ); }
-void Csock::SetRequiresClientCert( bool bRequiresCert ) { m_bRequireClientCert = bRequiresCert; }
+unsigned int Csock::GetRequireClientCertFlags() { return( m_iRequireClientCertFlags ); }
+void Csock::SetRequiresClientCert( bool bRequiresCert ) { m_iRequireClientCertFlags = ( bRequiresCert ? SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_PEER : 0 ); }
 
 #endif /* HAVE_LIBSSL */
 
@@ -1974,6 +2011,11 @@ int Csock::GetPending()
 #endif /* HAVE_LIBSSL */
 }
 
+int Csock::GetAddrInfo( const CS_STRING & sHostname, CSSockAddr & csSockAddr )
+{
+	return( ::GetAddrInfo( sHostname, this, csSockAddr ) );
+}
+
 int Csock::DNSLookup( EDNSLType eDNSLType )
 {
 	if ( eDNSLType == DNS_VHOST )
@@ -1992,7 +2034,7 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 	int iRet = ETIMEDOUT;
 	if ( eDNSLType == DNS_VHOST )
 	{
-		iRet = GetAddrInfo( m_sBindHost, this, m_bindhost );
+		iRet = GetAddrInfo( m_sBindHost, m_bindhost );
 #ifdef HAVE_IPV6
 		if( m_bindhost.GetIPv6() )
 		{
@@ -2006,20 +2048,23 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 	}
 	else
 	{
-		iRet = GetAddrInfo( m_shostname, this, m_address );
+		iRet = GetAddrInfo( m_shostname, m_address );
 	}
 
-	if( !CreateSocksFD() )
-		iRet = ETIMEDOUT;
-
 	if ( iRet == 0 )
 	{
+		if( !CreateSocksFD() )
+		{
+			m_iDNSTryCount = 0;
+			return ETIMEDOUT;
+		}
+
 		if ( m_eConState != CST_OK )
 			m_eConState = ( ( eDNSLType == DNS_VHOST ) ? CST_BINDVHOST : CST_CONNECT );
 		m_iDNSTryCount = 0;
 		return( 0 );
 	}
-	else if ( iRet == TRY_AGAIN )
+	else if ( iRet == EAGAIN )
 	{
 		m_iDNSTryCount++;
 		if ( m_iDNSTryCount > 20 )
@@ -2115,6 +2160,7 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 #ifdef HAVE_LIBSSL
 	m_ssl = NULL;
 	m_ssl_ctx = NULL;
+	m_iRequireClientCertFlags = 0;
 #endif /* HAVE_LIBSSL */
 	m_iTcount = 0;
 	m_iReadSock = -1;
@@ -2136,7 +2182,6 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_bFullsslAccept = false;
 	m_bsslEstablished = false;
 	m_bEnableReadLine = false;
-	m_bRequireClientCert = false;
 	m_iMaxStoredBufferLength = 1024;
 	m_iConnType = INBOUND;
 	m_iRemotePort = 0;
