@@ -1,6 +1,6 @@
 /**
 *
-*    Copyright (c) 1999-2007 Jim Hull <imaginos@imaginos.net>
+*    Copyright (c) 1999-2009 Jim Hull <imaginos@imaginos.net>
 *    All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without modification,
@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.199 $
+* $Revision: 1.209 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
@@ -85,7 +85,7 @@
 #include <set>
 #include <map>
 
-#include "main.h" // require this as a general rule, most projects have a defines.h or the like
+#include "defines.h" // require this as a general rule, most projects have a defines.h or the like
 
 #ifndef CS_STRING
 #	ifdef _HAS_CSTRING_
@@ -103,11 +103,13 @@
 #endif /* __DEBUG__ */
 #endif /* CS_DEBUG */
 
+#ifndef PERROR
 #ifdef __DEBUG__
 #	define PERROR( f ) __Perror( f, __FILE__, __LINE__ )
 #else
 #	define PERROR( f )	(void)0
 #endif /* __DEBUG__ */
+#endif /* PERROR */
 
 #ifndef _NO_CSOCKET_NS // some people may not want to use a namespace
 namespace Csocket
@@ -261,15 +263,27 @@ enum ECompType
 	CT_RLE	= 2
 };
 
+void SSLErrors( const char *filename, u_int iLineNum );
+
 /**
- * @brief You HAVE to call this in order to use the SSL library
+ * @brief You HAVE to call this in order to use the SSL library, calling InitCsocket() also calls this
+ * so unless you need to call InitSSL for a specific reason call InitCsocket()
  * @return true on success
  */
 
 bool InitSSL( ECompType eCompressionType = CT_NONE );
-void SSLErrors( const char *filename, u_int iLineNum );
+
 #endif /* HAVE_LIBSSL */
 
+/**
+ * This does all the csocket initialized inclusing InitSSL() and win32 specific initializations, only needs to be called once
+ */
+bool InitCsocket();
+/**
+ * Shutdown and release global allocated memory
+ */
+void ShutdownCsocket();
+
 //! @todo need to make this sock specific via getsockopt
 inline int GetSockError()
 {
@@ -280,27 +294,6 @@ inline int GetSockError()
 #endif /* _WIN32 */
 }
 
-#ifdef _WIN32
-inline bool InitWin32()
-{
-	WSADATA wsaData;
-	int iResult = WSAStartup( MAKEWORD( 2, 2 ), &wsaData );
-	if( iResult != NO_ERROR )
-		return( false );
-
-	return( true );
-}
-inline void ShutdownWin32()
-{
-	WSACleanup();
-}
-#define InitCsocket InitWin32
-#define ShutdownCsocket ShutdownWin32
-#else
-#define InitCsocket (void)0
-#define ShutdownCsocket (void)0
-#endif /* _WIN32 */
-
 //! wrappers for FD_SET and such to work in templates.
 inline void TFD_ZERO( fd_set *set )
 {
@@ -482,7 +475,8 @@ public:
 	{
 		CLT_DONT			= 0, //! don't close DER
 		CLT_NOW				= 1, //! close immediatly
-		CLT_AFTERWRITE		= 2  //! close after finishing writing the buffer
+		CLT_AFTERWRITE		= 2,  //! close after finishing writing the buffer
+		CLT_DEREFERENCE		= 3	 //! used after copy in Csock::Dereference() to cleanup a sock thats being shutdown
 	};
 
 	Csock & operator<<( const CS_STRING & s );
@@ -752,8 +746,11 @@ public:
 
 	//! Returns The Peers Public Key
 	CS_STRING GetPeerPubKey();
-	bool RequiresClientCert();
+	unsigned int GetRequireClientCertFlags();
+	//! legacy, deprecated @see SetRequireClientCertFlags
 	void SetRequiresClientCert( bool bRequiresCert );
+	//! bitwise flags, 0 means don't require cert, SSL_VERIFY_PEER verifies peers, SSL_VERIFY_FAIL_IF_NO_PEER_CERT will cause the connection to fail if no cert 
+	void SetRequireClientCertFlags( unsigned int iRequireClientCertFlags ) { m_iRequireClientCertFlags = iRequireClientCertFlags; }
 
 #endif /* HAVE_LIBSSL */
 
@@ -961,6 +958,14 @@ public:
 
 	void SetSkipConnect( bool b ) { m_bSkipConnect = b; }
 
+	/**
+	 * @brief override this call with your own DNS lookup method if you have one. By default this function is blocking
+	 * @param sHostname the hostname to resolve
+	 * @param csSockAddr the destination sock address info @see CSSockAddr
+	 * @return 0 on success, ETIMEDOUT if no lookup was found, EAGAIN if you should check again later for an answer
+	 */
+	virtual int GetAddrInfo( const CS_STRING & sHostname, CSSockAddr & csSockAddr );
+
 private:
 	//! making private for safety
 	Csock( const Csock & cCopy ) {}
@@ -969,7 +974,7 @@ private:
 	u_short		m_iport, m_iRemotePort, m_iLocalPort;
 	int			m_iReadSock, m_iWriteSock, m_itimeout, m_iConnType, m_iMethod, m_iTcount;
 	bool		m_bssl, m_bIsConnected, m_bBLOCK, m_bFullsslAccept;
-	bool		m_bsslEstablished, m_bEnableReadLine, m_bRequireClientCert, m_bPauseRead;
+	bool		m_bsslEstablished, m_bEnableReadLine, m_bPauseRead;
 	CS_STRING	m_shostname, m_sbuffer, m_sSockName, m_sPemFile, m_sCipherType, m_sParentName;
 	CS_STRING	m_sSend, m_sPemPass, m_sLocalIP, m_sRemoteIP;
 	ECloseType	m_eCloseType;
@@ -986,6 +991,7 @@ private:
 	SSL 				*m_ssl;
 	SSL_CTX				*m_ssl_ctx;
 	SSL_METHOD			*m_ssl_method;
+	unsigned int		m_iRequireClientCertFlags;
 
 	FPCertVerifyCB		m_pCerVerifyCB;
 
@@ -1114,7 +1120,7 @@ public:
 		m_iAFrequire = CSSockAddr::RAF_ANY;
 #ifdef HAVE_LIBSSL
 		m_sCipher = "HIGH";
-		m_bRequiresClientCert = false;
+		m_iRequireCertFlags = 0;
 #endif /* HAVE_LIBSSL */
 	}
 	virtual ~CSListener() {}
@@ -1130,7 +1136,7 @@ public:
 	const CS_STRING & GetCipher() const { return( m_sCipher ); }
 	const CS_STRING & GetPemLocation() const { return( m_sPemLocation ); }
 	const CS_STRING & GetPemPass() const { return( m_sPemPass ); }
-	bool GetRequiresClientCert() const { return( m_bRequiresClientCert ); }
+	unsigned int GetRequireClientCertFlags() const { return( m_iRequireCertFlags ); }
 #endif /* HAVE_LIBSSL */
 
 	//! sets the port to listen on. Set to 0 to listen on a random port
@@ -1155,8 +1161,10 @@ public:
 	void SetPemLocation( const CS_STRING & s ) { m_sPemLocation = s; }
 	//! set the pemfile pass
 	void SetPemPass( const CS_STRING & s ) { m_sPemPass = s; }
-	//! set to true if require a client certificate
-	void SetRequiresClientCert( bool b ) { m_bRequiresClientCert = b; }
+	//! set to true if require a client certificate (deprecated @see SetRequireClientCertFlags)
+	void SetRequiresClientCert( bool b ) { m_iRequireCertFlags = ( b ? SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT : 0 ); }
+	//! bitwise flags, 0 means don't require cert, SSL_VERIFY_PEER verifies peers, SSL_VERIFY_FAIL_IF_NO_PEER_CERT will cause the connection to fail if no cert 
+	void SetRequireClientCertFlags( unsigned int iRequireCertFlags ) { m_iRequireCertFlags = iRequireCertFlags; }
 #endif /* HAVE_LIBSSL */
 private:
 	u_short		m_iPort;
@@ -1168,7 +1176,7 @@ private:
 
 #ifdef HAVE_LIBSSL
 	CS_STRING	m_sPemLocation, m_sPemPass, m_sCipher;
-	bool		m_bRequiresClientCert;
+	unsigned int		m_iRequireCertFlags;
 #endif /* HAVE_LIBSSL */
 };
 
@@ -1316,7 +1324,7 @@ public:
 			pcSock->SetPemLocation( cListen.GetPemLocation() );
 			pcSock->SetPemPass( cListen.GetPemPass() );
 			pcSock->SetCipher( cListen.GetCipher() );
-			pcSock->SetRequiresClientCert( cListen.GetRequiresClientCert() );
+			pcSock->SetRequireClientCertFlags( cListen.GetRequireClientCertFlags() );
 		}
 #endif /* HAVE_LIBSSL */
 
@@ -1735,11 +1743,14 @@ public:
 
 		T * pSock = (*this)[iPos];
 
-		if ( pSock->IsConnected() )
-			pSock->Disconnected(); // only call disconnected event if connected event was called (IE IsConnected was set)
+		if( pSock->GetCloseType() != T::CLT_DEREFERENCE )
+		{
+			if ( pSock->IsConnected() )
+				pSock->Disconnected(); // only call disconnected event if connected event was called (IE IsConnected was set)
 
-		m_iBytesRead += pSock->GetBytesRead();
-		m_iBytesWritten += pSock->GetBytesWritten();
+			m_iBytesRead += pSock->GetBytesRead();
+			m_iBytesWritten += pSock->GetBytesWritten();
+		}
 
 		CS_Delete( pSock );
 		this->erase( this->begin() + iPos );
@@ -1762,8 +1773,8 @@ public:
 		Csock *pSock = (*this)[iOrginalSockIdx];
 		pNewSock->Copy( *pSock );
 		pSock->Dereference();
-		CS_Delete( pSock );
 		(*this)[iOrginalSockIdx] = (T *)pNewSock;
+		this->push_back( (T *)pSock ); // this allows it to get cleaned up
 		return( true );
 	}
 
@@ -1809,7 +1820,13 @@ public:
 		return( iRet );
 	}
 
-private:
+protected:
+	//! this is a strict wrapper around C-api select(). Added in the event you need to do special work here
+	virtual int Select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout)
+	{
+		return( select( nfds, readfds, writefds, exceptfds, timeout ) );
+	}
+protected:
 	/**
 	* fills a map of socks to a message for check
 	* map is empty if none are ready, check GetErrno() for the error, if not SUCCESS Select() failed
@@ -1834,7 +1851,8 @@ private:
 		// before we go any further, Process work needing to be done on the job
 		for( unsigned int i = 0; i < this->size(); i++ )
 		{
-			if ( ( (*this)[i]->GetCloseType() == T::CLT_NOW ) || ( ( (*this)[i]->GetCloseType() == T::CLT_AFTERWRITE ) && ( (*this)[i]->GetWriteBuffer().empty() ) ) )
+			Csock::ECloseType eCloseType = (*this)[i]->GetCloseType();
+			if( eCloseType == T::CLT_NOW || eCloseType == T::CLT_DEREFERENCE || ( eCloseType == T::CLT_AFTERWRITE && (*this)[i]->GetWriteBuffer().empty() ) )
 				DelSock( i-- ); // close any socks that have requested it
 			else
 				(*this)[i]->Cron(); // call the Cron handler here
@@ -1892,6 +1910,12 @@ private:
 					{
 						pcSock->Close();
 					}
+					if( pcSock->GetWriteBuffer().size() )
+					{ // this means we need to write again, not everything got knocked out
+						TFD_SET( iWSock, &wfds );
+						bHasWriteable = true;
+					}
+
 				} else
 				{
 					if ( !bIsReadPaused )
@@ -1940,9 +1964,9 @@ private:
 
 
 		if ( bHasWriteable )
-			iSel = select(FD_SETSIZE, &rfds, &wfds, NULL, &tv);
+			iSel = Select(FD_SETSIZE, &rfds, &wfds, NULL, &tv);
 		else
-			iSel = select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
+			iSel = Select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
 		if ( iSel == 0 )
 		{
 			if ( mpeSocks.empty() )
@@ -2053,7 +2077,7 @@ private:
 							NewpcSock->SetCipher( pcSock->GetCipher() );
 							NewpcSock->SetPemLocation( pcSock->GetPemLocation() );
 							NewpcSock->SetPemPass( pcSock->GetPemPass() );
-							NewpcSock->SetRequiresClientCert( pcSock->RequiresClientCert() );
+							NewpcSock->SetRequireClientCertFlags( pcSock->GetRequireClientCertFlags() );
 							bAddSock = NewpcSock->AcceptSSL();
 						}
 
@@ -2118,7 +2142,7 @@ private:
 			const std::vector<CCron *> & vCrons = pSock->GetCrons();
 			std::vector<CCron *>::const_iterator cit;
 			std::vector<CCron *>::const_iterator cit_end = vCrons.end();
-			for (cit = vCrons.begin(); cit != cit_end; it++)
+			for (cit = vCrons.begin(); cit != cit_end; cit++)
 				iNextRunTime = std::min( iNextRunTime, (*cit)->GetNextRun() );
 		}
 		std::vector<CCron *>::const_iterator cit;
