@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -8,31 +8,70 @@
 
 #include "DCCSock.h"
 #include "User.h"
+#include "Utils.h"
+
+CDCCSock::CDCCSock(CUser* pUser, const CString& sRemoteNick, const CString& sLocalFile, const CString& sModuleName,
+		unsigned long uFileSize, CFile* pFile) : CZNCSock() {
+	m_sRemoteNick = sRemoteNick;
+	m_uFileSize = uFileSize;
+	m_uRemotePort = 0;
+	m_uBytesSoFar = 0;
+	m_pUser = pUser;
+	m_pFile = pFile;
+	m_sLocalFile = sLocalFile;
+	m_sModuleName = sModuleName;
+	m_bSend = true;
+	m_bNoDelFile = false;
+
+	m_pUser->AddDCCSock(this);
+}
+
+CDCCSock::CDCCSock(CUser* pUser, const CString& sRemoteNick, const CString& sRemoteIP, unsigned short uRemotePort,
+		const CString& sLocalFile, unsigned long uFileSize, const CString& sModuleName) : CZNCSock() {
+	m_sRemoteNick = sRemoteNick;
+	m_sRemoteIP = sRemoteIP;
+	m_uRemotePort = uRemotePort;
+	m_uFileSize = uFileSize;
+	m_uBytesSoFar = 0;
+	m_pUser = pUser;
+	m_pFile = NULL;
+	m_sLocalFile = sLocalFile;
+	m_sModuleName = sModuleName;
+	m_bSend = false;
+	m_bNoDelFile = false;
+
+	m_pUser->AddDCCSock(this);
+}
 
 CDCCSock::~CDCCSock() {
 	if ((m_pFile) && (!m_bNoDelFile)) {
 		m_pFile->Close();
 		delete m_pFile;
 	}
-	if (m_pUser) {
-		m_pUser->AddBytesRead(GetBytesRead());
-		m_pUser->AddBytesWritten(GetBytesWritten());
-	}
+
+	m_pUser->AddBytesRead(GetBytesRead());
+	m_pUser->AddBytesWritten(GetBytesWritten());
+
+	m_pUser->DelDCCSock(this);
 }
 
 void CDCCSock::ReadData(const char* data, int len) {
 	if (!m_pFile) {
-		DEBUG_ONLY(cout << "File not open! closing get." << endl);
+		DEBUG("File not open! closing get.");
 		m_pUser->PutModule(m_sModuleName, ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - File not open!");
 		Close();
 	}
 
+	// DCC specs says the receiving end sends the number of bytes it
+	// received so far as a 4 byte integer in network byte order, so we need
+	// uint32_t to do the job portably. This also means that the maximum
+	// file that we can transfer is 4 GiB big (see OpenFile()).
 	if (m_bSend) {
 		m_sSendBuf.append(data, len);
 
 		while (m_sSendBuf.size() >= 4) {
-			unsigned int iRemoteSoFar;
-			memcpy(&iRemoteSoFar, m_sSendBuf.data(), 4);
+			uint32_t iRemoteSoFar;
+			memcpy(&iRemoteSoFar, m_sSendBuf.data(), sizeof(iRemoteSoFar));
 			iRemoteSoFar = ntohl(iRemoteSoFar);
 
 			if ((iRemoteSoFar + 65536) >= m_uBytesSoFar) {
@@ -44,8 +83,8 @@ void CDCCSock::ReadData(const char* data, int len) {
 	} else {
 		m_pFile->Write(data, len);
 		m_uBytesSoFar += len;
-		unsigned long uSoFar = htonl(m_uBytesSoFar);
-		Write((char*) &uSoFar, sizeof(unsigned long));
+		uint32_t uSoFar = htonl(m_uBytesSoFar);
+		Write((char*) &uSoFar, sizeof(uSoFar));
 
 		if (m_uBytesSoFar >= m_uFileSize) {
 			Close();
@@ -54,22 +93,22 @@ void CDCCSock::ReadData(const char* data, int len) {
 }
 
 void CDCCSock::ConnectionRefused() {
-	DEBUG_ONLY(cout << GetSockName() << " == ConnectionRefused()" << endl);
+	DEBUG(GetSockName() << " == ConnectionRefused()");
 	m_pUser->PutModule(m_sModuleName, ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - Connection Refused.");
 }
 
 void CDCCSock::Timeout() {
-	DEBUG_ONLY(cout << GetSockName() << " == Timeout()" << endl);
+	DEBUG(GetSockName() << " == Timeout()");
 	m_pUser->PutModule(m_sModuleName, ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - Timed Out.");
 }
 
 void CDCCSock::SockError(int iErrno) {
-	DEBUG_ONLY(cout << GetSockName() << " == SockError(" << iErrno << ")" << endl);
+	DEBUG(GetSockName() << " == SockError(" << iErrno << ")");
 	m_pUser->PutModule(m_sModuleName, ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - Socket Error [" + CString(iErrno) + "]");
 }
 
 void CDCCSock::Connected() {
-	DEBUG_ONLY(cout << GetSockName() << " == Connected(" << GetRemoteIP() << ")" << endl);
+	DEBUG(GetSockName() << " == Connected(" << GetRemoteIP() << ")");
 	m_pUser->PutModule(m_sModuleName, ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - Transfer Started.");
 
 	if (m_bSend) {
@@ -82,7 +121,7 @@ void CDCCSock::Connected() {
 void CDCCSock::Disconnected() {
 	const CString sStart = ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - ";
 
-	DEBUG_ONLY(cout << GetSockName() << " == Disconnected()" << endl);
+	DEBUG(GetSockName() << " == Disconnected()");
 
 	if (m_uBytesSoFar > m_uFileSize) {
 		m_pUser->PutModule(m_sModuleName, sStart + "TooMuchData!");
@@ -106,6 +145,14 @@ void CDCCSock::SendPacket() {
 		return;
 	}
 
+	if (GetInternalWriteBuffer().size() > 1024 * 1024) {
+		// There is still enough data to be written, don't add more
+		// stuff to that buffer.
+		DEBUG("SendPacket(): Skipping send, buffer still full enough [" << GetInternalWriteBuffer().size() << "]["
+				<< m_sRemoteNick << "][" << m_sFileName << "]");
+		return;
+	}
+
 	char szBuf[4096];
 	int iLen = m_pFile->Read(szBuf, 4096);
 
@@ -137,7 +184,7 @@ Csock* CDCCSock::GetSockObj(const CString& sHost, unsigned short uPort) {
 CFile* CDCCSock::OpenFile(bool bWrite) {
 	if ((m_pFile) || (m_sLocalFile.empty())) {
 		m_pUser->PutModule(m_sModuleName, ((bWrite) ? "DCC <- [" : "DCC -> [") + m_sRemoteNick + "][" + m_sLocalFile + "] - Unable to open file.");
-		return false;
+		return NULL;
 	}
 
 	m_pFile = new CFile(m_sLocalFile);
@@ -164,14 +211,24 @@ CFile* CDCCSock::OpenFile(bool bWrite) {
 			return NULL;
 		}
 
-		if (!m_pFile->Open(O_RDONLY)) {
+		if (!m_pFile->Open()) {
 			delete m_pFile;
 			m_pFile = NULL;
 			m_pUser->PutModule(m_sModuleName, "DCC -> [" + m_sRemoteNick + "] - Could not open file [" + m_sLocalFile + "]");
 			return NULL;
 		}
 
-		m_uFileSize = m_pFile->GetSize();
+		// The DCC specs only allow file transfers with files smaller
+		// than 4GiB (see ReadData()).
+		unsigned long long uFileSize = m_pFile->GetSize();
+		if (uFileSize > (unsigned long long) 0xffffffff) {
+			delete m_pFile;
+			m_pFile = NULL;
+			m_pUser->PutModule(m_sModuleName, "DCC -> [" + m_sRemoteNick + "] - File too large (>4 GiB) [" + m_sLocalFile + "]");
+			return NULL;
+		}
+
+		m_uFileSize = (unsigned long) uFileSize;
 	}
 
 	m_sFileName = m_pFile->GetShortName();
