@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,34 +7,32 @@
  */
 
 #include "FileUtils.h"
+#include "znc.h"
 #include "Utils.h"
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#ifndef HAVE_LSTAT
+#  define lstat(a, b)	stat(a, b)
+#endif
+
+#ifndef O_BINARY
+#  define O_BINARY	0
+#endif
+
 CFile::CFile() {
 	m_iFD = -1;
-	m_bClose = false;
 }
 
 CFile::CFile(const CString& sLongName) {
 	m_iFD = -1;
-	m_bClose = false;
-
-	SetFileName(sLongName);
-}
-
-CFile::CFile(int iFD, const CString& sLongName) {
-	m_iFD = iFD;
-	m_bClose = false;
 
 	SetFileName(sLongName);
 }
 
 CFile::~CFile() {
-	if (m_bClose && m_iFD != -1) {
-		Close();
-	}
+	Close();
 }
 
 void CFile::SetFileName(const CString& sLongName) {
@@ -49,8 +47,16 @@ void CFile::SetFileName(const CString& sLongName) {
 	}
 }
 
+bool CFile::IsDir(const CString& sLongName, bool bUseLstat) {
+	if (sLongName.Equals("/"))
+		return CFile::FType(sLongName, FT_DIRECTORY, bUseLstat);
+
+	// Some OS don't like trailing slashes for directories
+	return CFile::FType(sLongName.TrimRight_n("/"),
+			FT_DIRECTORY, bUseLstat);
+}
+
 bool CFile::IsReg(const CString& sLongName, bool bUseLstat) { return CFile::FType(sLongName, FT_REGULAR, bUseLstat); }
-bool CFile::IsDir(const CString& sLongName, bool bUseLstat) { return CFile::FType(sLongName, FT_DIRECTORY, bUseLstat); }
 bool CFile::IsChr(const CString& sLongName, bool bUseLstat)  { return CFile::FType(sLongName, FT_CHARACTER, bUseLstat); }
 bool CFile::IsBlk(const CString& sLongName, bool bUseLstat)  { return CFile::FType(sLongName, FT_BLOCK, bUseLstat); }
 bool CFile::IsFifo(const CString& sLongName, bool bUseLstat)  { return CFile::FType(sLongName, FT_FIFO, bUseLstat); }
@@ -65,8 +71,6 @@ bool CFile::IsFifo(bool bUseLstat) const { return CFile::IsFifo(m_sLongName, bUs
 bool CFile::IsLnk(bool bUseLstat) const { return CFile::IsLnk(m_sLongName, bUseLstat); }
 bool CFile::IsSock(bool bUseLstat) const { return CFile::IsSock(m_sLongName, bUseLstat); }
 
-bool CFile::access(int mode) { return (::access(m_sLongName.c_str(), mode) == 0); }
-
 // for gettin file types, using fstat instead
 bool CFile::FType(const CString sFileName, EFileTypes eType, bool bUseLstat) {
 	struct stat st;
@@ -97,7 +101,7 @@ bool CFile::FType(const CString sFileName, EFileTypes eType, bool bUseLstat) {
 		case FT_SOCK:
 			return S_ISSOCK(st.st_mode);
 		default:
-			return false;
+			break;
 	}
 	return false;
 }
@@ -106,18 +110,18 @@ bool CFile::FType(const CString sFileName, EFileTypes eType, bool bUseLstat) {
 // Functions to retrieve file information
 //
 bool CFile::Exists() const { return CFile::Exists(m_sLongName); }
-unsigned long long CFile::GetSize() const { return CFile::GetSize(m_sLongName); }
-unsigned int CFile::GetATime() const { return CFile::GetATime(m_sLongName); }
-unsigned int CFile::GetMTime() const { return CFile::GetMTime(m_sLongName); }
-unsigned int CFile::GetCTime() const { return CFile::GetCTime(m_sLongName); }
-int CFile::GetUID() const { return CFile::GetUID(m_sLongName); }
-int CFile::GetGID() const { return CFile::GetGID(m_sLongName); }
+off_t CFile::GetSize() const { return CFile::GetSize(m_sLongName); }
+time_t CFile::GetATime() const { return CFile::GetATime(m_sLongName); }
+time_t CFile::GetMTime() const { return CFile::GetMTime(m_sLongName); }
+time_t CFile::GetCTime() const { return CFile::GetCTime(m_sLongName); }
+uid_t CFile::GetUID() const { return CFile::GetUID(m_sLongName); }
+gid_t CFile::GetGID() const { return CFile::GetGID(m_sLongName); }
 bool CFile::Exists(const CString& sFile) {
 	struct stat st;
 	return (stat(sFile.c_str(), &st) == 0);
 }
 
-unsigned long long CFile::GetSize(const CString& sFile) {
+off_t CFile::GetSize(const CString& sFile) {
 	struct stat st;
 	if (stat(sFile.c_str(), &st) != 0) {
 		return 0;
@@ -126,27 +130,27 @@ unsigned long long CFile::GetSize(const CString& sFile) {
 	return (S_ISREG(st.st_mode)) ? st.st_size : 0;
 }
 
-unsigned int CFile::GetATime(const CString& sFile) {
+time_t CFile::GetATime(const CString& sFile) {
 	struct stat st;
 	return (stat(sFile.c_str(), &st) != 0) ? 0 : st.st_atime;
 }
 
-unsigned int CFile::GetMTime(const CString& sFile) {
+time_t CFile::GetMTime(const CString& sFile) {
 	struct stat st;
 	return (stat(sFile.c_str(), &st) != 0) ? 0 : st.st_mtime;
 }
 
-unsigned int CFile::GetCTime(const CString& sFile) {
+time_t CFile::GetCTime(const CString& sFile) {
 	struct stat st;
 	return (stat(sFile.c_str(), &st) != 0) ? 0 : st.st_ctime;
 }
 
-int CFile::GetUID(const CString& sFile) {
+uid_t CFile::GetUID(const CString& sFile) {
 	struct stat st;
 	return (stat(sFile.c_str(), &st) != 0) ? -1 : (int) st.st_uid;
 }
 
-int CFile::GetGID(const CString& sFile) {
+gid_t CFile::GetGID(const CString& sFile) {
 	struct stat st;
 	return (stat(sFile.c_str(), &st) != 0) ? -1 : (int) st.st_gid;
 }
@@ -158,11 +162,11 @@ int CFile::GetInfo(const CString& sFile, struct stat& st) {
 // Functions to manipulate the file on the filesystem
 //
 bool CFile::Delete() { return CFile::Delete(m_sLongName); }
-int CFile::Move(const CString& sNewFileName, bool bOverwrite) {
+bool CFile::Move(const CString& sNewFileName, bool bOverwrite) {
 	return CFile::Move(m_sLongName, sNewFileName, bOverwrite);
 }
 
-int CFile::Copy(const CString& sNewFileName, bool bOverwrite) {
+bool CFile::Copy(const CString& sNewFileName, bool bOverwrite) {
 	return CFile::Copy(m_sLongName, sNewFileName, bOverwrite);
 }
 
@@ -191,7 +195,7 @@ bool CFile::Copy(const CString& sOldFileName, const CString& sNewFileName, bool
 	CFile OldFile(sOldFileName);
 	CFile NewFile(sNewFileName);
 
-	if (!OldFile.Open(O_RDONLY)) {
+	if (!OldFile.Open()) {
 		return false;
 	}
 
@@ -200,9 +204,19 @@ bool CFile::Copy(const CString& sOldFileName, const CString& sNewFileName, bool
 	}
 
 	char szBuf[8192];
-	unsigned int len = 0;
+	int len = 0;
 
 	while ((len = OldFile.Read(szBuf, 8192))) {
+		if (len < 0) {
+			DEBUG("CFile::Copy() failed: " << strerror(errno));
+			OldFile.Close();
+
+			// That file is only a partial copy, get rid of it
+			NewFile.Close();
+			NewFile.Delete();
+
+			return false;
+		}
 		NewFile.Write(szBuf, len);
 	}
 
@@ -213,15 +227,18 @@ bool CFile::Copy(const CString& sOldFileName, const CString& sNewFileName, bool
 }
 
 bool CFile::Chmod(mode_t mode) {
-	return CFile::Chmod(m_sLongName, mode);
+	if (m_iFD == -1) {
+		return false;
+	}
+	return (fchmod(m_iFD, mode) == 0);
 }
 
 bool CFile::Chmod(const CString& sFile, mode_t mode) {
 	return (chmod(sFile.c_str(), mode) == 0);
 }
 
-bool CFile::Seek(unsigned long uPos) {
-	if (m_iFD != -1 && (unsigned int) lseek(m_iFD, uPos, SEEK_SET) == uPos) {
+bool CFile::Seek(off_t uPos) {
+	if (m_iFD != -1 && lseek(m_iFD, uPos, SEEK_SET) == uPos) {
 		ClearBuffer();
 		return true;
 	}
@@ -238,6 +255,14 @@ bool CFile::Truncate() {
 	return false;
 }
 
+bool CFile::Sync() {
+	if (m_iFD != -1 && fsync(m_iFD) == 0) {
+		return true;
+	}
+
+	return false;
+}
+
 bool CFile::Open(const CString& sFileName, int iFlags, mode_t iMode) {
 	SetFileName(sFileName);
 	return Open(iFlags, iMode);
@@ -248,6 +273,13 @@ bool CFile::Open(int iFlags, mode_t iMode) {
 		return false;
 	}
 
+	// We never want to get a controlling TTY through this -> O_NOCTTY
+	iMode |= O_NOCTTY;
+
+	// Some weird OS from MS needs O_BINARY or else it generates fake EOFs
+	// when reading ^Z from a file.
+	iMode |= O_BINARY;
+
 	m_iFD = open(m_sLongName.c_str(), iFlags, iMode);
 	if (m_iFD < 0)
 		return false;
@@ -255,7 +287,6 @@ bool CFile::Open(int iFlags, mode_t iMode) {
 	/* Make sure this FD isn't given to childs */
 	SetFdCloseOnExec(m_iFD);
 
-	m_bClose = true;
 	return true;
 }
 
@@ -268,58 +299,65 @@ int CFile::Read(char *pszBuffer, int iBytes) {
 }
 
 bool CFile::ReadLine(CString& sData, const CString & sDelimiter) {
-	char buff[64];
-	sData.clear();
+	char buff[4096];
+	int iBytes;
 
 	if (m_iFD == -1) {
 		return false;
 	}
 
-	bool bEOF = false;
-
-	while (true) {
+	do {
 		CString::size_type iFind = m_sBuffer.find(sDelimiter);
 		if (iFind != CString::npos) {
-			sData = m_sBuffer.substr(0, (iFind + 1));
-			m_sBuffer.erase(0, (iFind + 1));
-			break;
+			// We found a line, return it
+			sData = m_sBuffer.substr(0, iFind + sDelimiter.length());
+			m_sBuffer.erase(0, iFind + sDelimiter.length());
+			return true;
 		}
 
-		memset((char *)buff, '\0', 64);
-		int iBytes = read(m_iFD, buff, 64);
+		iBytes = read(m_iFD, buff, sizeof(buff));
 
-		switch(iBytes) {
-			case -1: {
-				bEOF = true;
-				break;
-			}
-			case 0: {
-				bEOF = true;
-				break;
-			}
-			default: {
-				m_sBuffer.append(buff, iBytes);
-				break;
-			}
+		if (iBytes > 0) {
+			m_sBuffer.append(buff, iBytes);
 		}
+	} while (iBytes > 0);
 
-		if (bEOF) {
-			break;
-		}
-	}
+	// We are at the end of the file or an error happened
 
-	CString::size_type iFind = m_sBuffer.find(sDelimiter);
-	if (iFind != CString::npos) {
-		return true;
-	}
-
-	if (bEOF && m_sBuffer.size()) {
+	if (!m_sBuffer.empty()) {
+		// ..but there is still some partial line in the buffer
 		sData = m_sBuffer;
 		m_sBuffer.clear();
 		return true;
 	}
 
-	return !bEOF;
+	// Nothing left for reading :(
+	return false;
+}
+
+bool CFile::ReadFile(CString& sData, size_t iMaxSize) {
+	char buff[4096];
+	size_t iBytesRead = 0;
+
+	sData.clear();
+
+	while (iBytesRead < iMaxSize) {
+		int iBytes = Read(buff, sizeof(buff));
+
+		if (iBytes < 0)
+			// Error
+			return false;
+
+		if (iBytes == 0)
+			// EOF
+			return true;
+
+		sData.append(buff, iBytes);
+		iBytesRead += iBytes;
+	}
+
+	// Buffer limit reached
+	return false;
 }
 
 int CFile::Write(const char *pszBuffer, u_int iBytes) {
@@ -334,14 +372,42 @@ int CFile::Write(const CString & sData) {
 	return Write(sData.data(), sData.size());
 }
 void CFile::Close() {
-	if (m_iFD >= 0 && m_bClose) {
-		close(m_iFD);
-		m_iFD = -1;
-		m_bClose = false;
+	if (m_iFD >= 0) {
+		if (close(m_iFD) < 0) {
+			DEBUG("CFile::Close(): close() failed with ["
+					<< strerror(errno) << "]");
+		}
 	}
+	m_iFD = -1;
+	ClearBuffer();
 }
 void CFile::ClearBuffer() { m_sBuffer.clear(); }
 
+bool CFile::TryExLock(const CString& sLockFile, int iFlags) {
+	Open(sLockFile, iFlags);
+	return TryExLock();
+}
+
+bool CFile::TryExLock() {
+	return Lock(LOCK_EX|LOCK_NB);
+}
+
+bool CFile::UnLock() {
+	return Lock(LOCK_UN);
+}
+
+bool CFile::Lock(int iOperation) {
+	if (m_iFD == -1) {
+		return false;
+	}
+
+	if (flock(m_iFD, iOperation) != 0) {
+		return false;
+	}
+
+	return true;
+}
+
 bool CFile::IsOpen() const { return (m_iFD != -1); }
 CString CFile::GetLongName() const { return m_sLongName; }
 CString CFile::GetShortName() const { return m_sShortName; }
@@ -355,14 +421,18 @@ CString CFile::GetDir() const {
 	return sDir;
 }
 
-void CFile::SetFD(int iFD) { m_iFD = iFD; }
+CString CDir::ChangeDir(const CString& sPath, const CString& sAdd, const CString& sHome) {
+	CString sHomeDir(sHome);
+
+	if (sHomeDir.empty()) {
+		sHomeDir = CZNC::Get().GetHomePath();
+	}
 
-CString CDir::ChangeDir(const CString& sPath, const CString& sAdd, const CString& sHomeDir) {
 	if (sAdd == "~") {
 		return sHomeDir;
 	}
 
-	CString sAddDir = sAdd;
+	CString sAddDir(sAdd);
 
 	if (sAddDir.Left(2) == "~/") {
 		sAddDir.LeftChomp();
@@ -397,6 +467,15 @@ CString CDir::ChangeDir(const CString& sPath, const CString& sAdd, const CString
 	return (sRet.empty()) ? "/" : sRet;
 }
 
+CString CDir::CheckPathPrefix(const CString& sPath, const CString& sAdd, const CString& sHomeDir) {
+	CString sPrefix = sPath.Replace_n("//", "/").TrimRight_n("/") + "/";
+	CString sAbsolutePath = ChangeDir(sPrefix, sAdd, sHomeDir);
+
+	if (sAbsolutePath.Left(sPrefix.length()) != sPrefix)
+		return "";
+	return sAbsolutePath;
+}
+
 bool CDir::MakeDir(const CString& sPath, mode_t iMode) {
 	CString sDir;
 	VCString dirs;
@@ -476,7 +555,9 @@ int CExecSock::popen2(int & iReadFD, int & iWriteFD, const CString & sCommand) {
 			NULL
 		};
 		execvp("sh", (char * const *) pArgv);
-		exit(0);
+		// if execvp returns, there was an error
+		perror("execvp");
+		exit(1);
 	}
 
 	close(wpipes[0]);
@@ -494,7 +575,7 @@ void CExecSock::close2(int iPid, int iReadFD, int iWriteFD) {
 	u_int iNow = time(NULL);
 	while (waitpid(iPid, NULL, WNOHANG) == 0) {
 		if ((time(NULL) - iNow) > 5)
-			break;	// giveup
+			break;  // giveup
 		usleep(100);
 	}
 	return;
