@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -13,7 +13,7 @@
 #include "User.h"
 #include "znc.h"
 
-CIRCSock::CIRCSock(CUser* pUser) : Csock() {
+CIRCSock::CIRCSock(CUser* pUser) : CZNCSock() {
 	m_pUser = pUser;
 	m_bISpoofReleased = false;
 	m_bAuthed = false;
@@ -71,16 +71,19 @@ void CIRCSock::Quit() {
 void CIRCSock::ReadLine(const CString& sData) {
 	CString sLine = sData;
 
-	while ((sLine.Right(1) == "\r") || (sLine.Right(1) == "\n")) {
-		sLine.RightChomp();
-	}
+	sLine.TrimRight("\n\r");
 
-	DEBUG_ONLY(cout << "(" << m_pUser->GetUserName() << ") IRC -> ZNC [" << sLine << "]" << endl);
+	DEBUG("(" << m_pUser->GetUserName() << ") IRC -> ZNC [" << sLine << "]");
 
 	MODULECALL(OnRaw(sLine), m_pUser, NULL, return);
 
 	if (sLine.Equals("PING ", false, 5)) {
 		PutIRC("PONG " + sLine.substr(5));
+		m_pUser->PutUser(sLine);
+		return;
+	} else if (sLine.Equals("PONG ", false, 5) && sLine.Token(3).Equals(":ZNC")) {
+		// We asked for this so don't forward the reply to clients.
+		return;
 	} else if (sLine.Equals("ERROR ", false, 6)) {
 		//ERROR :Closing Link: nick[24.24.24.24] (Excess Flood)
 		CString sError(sLine.substr(7));
@@ -90,536 +93,532 @@ void CIRCSock::ReadLine(const CString& sData) {
 		}
 
 		m_pUser->PutStatus("Error from Server [" + sError + "]");
-	} else if (sLine.WildCmp(":* * *")) { //"^:(\\S+) (\\d\\d\\d) (.*?) (.*)$", vCap)) {
-		CString sCmd = sLine.Token(1);
+		return;
+	}
 
-		if ((sCmd.length() == 3) && (isdigit(sCmd[0])) && (isdigit(sCmd[1])) && (isdigit(sCmd[2]))) {
-			CString sServer = sLine.Token(0); sServer.LeftChomp();
-			unsigned int uRaw = strtoul(sCmd.c_str(), NULL, 10);
-			CString sNick = sLine.Token(2);
-			CString sRest = sLine.Token(3, true);
+	CString sCmd = sLine.Token(1);
 
-			switch (uRaw) {
-				case 1:	{// :irc.server.com 001 nick :Welcome to the Internet Relay Network nick
-					m_pUser->SetIRCServer(sServer);
-					SetTimeout(900);	// Now that we are connected, let nature take its course
-					PutIRC("WHO " + sNick);
+	if ((sCmd.length() == 3) && (isdigit(sCmd[0])) && (isdigit(sCmd[1])) && (isdigit(sCmd[2]))) {
+		CString sServer = sLine.Token(0); sServer.LeftChomp();
+		unsigned int uRaw = sCmd.ToUInt();
+		CString sNick = sLine.Token(2);
+		CString sRest = sLine.Token(3, true);
 
-					m_bAuthed = true;
-					m_pUser->PutStatus("Connected!");
+		switch (uRaw) {
+			case 1:	{// :irc.server.com 001 nick :Welcome to the Internet Relay Network nick
+				m_pUser->SetIRCServer(sServer);
+				SetTimeout(240, TMO_READ);	// Now that we are connected, let nature take its course
+				PutIRC("WHO " + sNick);
 
-					vector<CClient*>& vClients = m_pUser->GetClients();
+				m_bAuthed = true;
+				m_pUser->PutStatus("Connected!");
 
-					for (unsigned int a = 0; a < vClients.size(); a++) {
-						CClient* pClient = vClients[a];
-						CString sClientNick = pClient->GetNick(false);
+				vector<CClient*>& vClients = m_pUser->GetClients();
 
-						if (!sClientNick.Equals(sNick)) {
-							// If they connected with a nick that doesn't match the one we got on irc, then we need to update them
-							pClient->PutClient(":" + sClientNick + "!" + m_Nick.GetIdent() + "@" + m_Nick.GetHost() + " NICK :" + sNick);
-						}
-					}
+				for (unsigned int a = 0; a < vClients.size(); a++) {
+					CClient* pClient = vClients[a];
+					CString sClientNick = pClient->GetNick(false);
 
-					SetNick(sNick);
-
-					MODULECALL(OnIRCConnected(), m_pUser, NULL, );
-
-					m_pUser->ClearRawBuffer();
-					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
-
-					CZNC::Get().ReleaseISpoof();
-					m_bISpoofReleased = true;
-
-					break;
-				}
-				case 5:
-					ParseISupport(sRest);
-					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
-					break;
-				case 2:
-				case 3:
-				case 4:
-				case 250:	// highest connection count
-				case 251:	// user count
-				case 252:	// oper count
-				case 254:	// channel count
-				case 255:	// client count
-				case 265:	// local users
-				case 266:	// global users
-					m_pUser->UpdateRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
-					break;
-				case 422:	// MOTD File is missing
-				case 375: 	// begin motd
-					m_pUser->ClearMotdBuffer();
-				case 372:	// motd
-				case 376:	// end motd
-					m_pUser->AddMotdBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
-					break;
-				case 471: // :irc.server.net 471 nick #chan :Cannot join channel (+l)
-				case 473: // :irc.server.net 473 nick #chan :Cannot join channel (+i)
-				case 475: // :irc.server.net 475 nick #chan :Cannot join channel (+k)
-					if (m_pUser->IsUserAttached()) {
-						CChan* pChan = m_pUser->FindChan(sRest.substr(0, sRest.find(' ')));
-
-						if ((pChan) && (!pChan->IsOn())) {
-							if (!pChan->DecClientRequests()) {
-								return;
-							}
-						}
-					}
-
-					break;
-				case 437:
-					// :irc.server.net 437 * badnick :Nick/channel is temporarily unavailable
-					// :irc.server.net 437 mynick badnick :Nick/channel is temporarily unavailable
-					// :irc.server.net 437 mynick badnick :Cannot change nickname while banned on channel
-					if (m_pUser->IsChan(sRest.Token(0)) || sNick != "*")
-						break;
-				case 433: {
-					CString sBadNick = sRest.Token(0);
-					CString sConfNick = m_pUser->GetNick().Left(GetMaxNickLen());
-
-					if (sNick == "*") {
-						SendAltNick(sBadNick);
-						return;
-					}
-					break;
-				}
-				case 315: {
-					// :irc.server.com 315 yournick #chan :End of /WHO list.
-					CChan* pChan = m_pUser->FindChan(sLine.Token(3));
-
-					if (pChan) {
-						pChan->SetWhoDone();
-					}
-
-					break;
-				}
-				case 331: {
-					// :irc.server.com 331 yournick #chan :No topic is set.
-					CChan* pChan = m_pUser->FindChan(sLine.Token(3));
-
-					if (pChan) {
-						pChan->SetTopic("");
-					}
-
-					break;
-				}
-				case 332: {
-					// :irc.server.com 332 yournick #chan :This is a topic
-					CChan* pChan = m_pUser->FindChan(sLine.Token(3));
-
-					if (pChan) {
-						CString sTopic = sLine.Token(4, true);
-						sTopic.LeftChomp();
-						pChan->SetTopic(sTopic);
-					}
-
-					break;
-				}
-				case 333: {
-					// :irc.server.com 333 yournick #chan setternick 1112320796
-					CChan* pChan = m_pUser->FindChan(sLine.Token(3));
-
-					if (pChan) {
-						sNick = sLine.Token(4);
-						unsigned long ulDate = strtoul(sLine.Token(5).c_str(), NULL, 10);
-
-						pChan->SetTopicOwner(sNick);
-						pChan->SetTopicDate(ulDate);
-					}
-
-					break;
-				}
-				case 352: {
-					// :irc.yourserver.com 352 yournick #chan ident theirhost.com irc.theirserver.com theirnick H :0 Real Name
-					sServer = sLine.Token(0);
-					sNick = sLine.Token(7);
-					CString sIdent = sLine.Token(4);
-					CString sHost = sLine.Token(5);
-
-					sServer.LeftChomp();
-
-					if (sNick.Equals(GetNick())) {
-						m_Nick.SetIdent(sIdent);
-						m_Nick.SetHost(sHost);
-					}
-
-					m_pUser->SetIRCNick(m_Nick);
-					m_pUser->SetIRCServer(sServer);
-
-					const vector<CChan*>& vChans = m_pUser->GetChans();
-
-					for (unsigned int a = 0; a < vChans.size(); a++) {
-						vChans[a]->OnWho(sNick, sIdent, sHost);
-					}
-
-					break;
-				}
-				case 324: {	// MODE
-					sRest.Trim();
-					CChan* pChan = m_pUser->FindChan(sRest.Token(0));
-
-					if (pChan) {
-						pChan->SetModes(sRest.Token(1, true));
+					if (!sClientNick.Equals(sNick)) {
+						// If they connected with a nick that doesn't match the one we got on irc, then we need to update them
+						pClient->PutClient(":" + sClientNick + "!" + m_Nick.GetIdent() + "@" + m_Nick.GetHost() + " NICK :" + sNick);
 					}
 				}
-					break;
-				case 329: {
-					sRest.Trim();
-					CChan* pChan = m_pUser->FindChan(sRest.Token(0));
 
-					if (pChan) {
-						unsigned long ulDate = strtoul(sLine.Token(4).c_str(), NULL, 10);
-						pChan->SetCreationDate(ulDate);
-					}
+				SetNick(sNick);
+
+				MODULECALL(OnIRCConnected(), m_pUser, NULL, );
+
+				m_pUser->ClearRawBuffer();
+				m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+
+				CZNC::Get().ReleaseISpoof();
+				m_bISpoofReleased = true;
+
+				break;
+			}
+			case 5:
+				ParseISupport(sRest);
+				m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+				break;
+			case 2:
+			case 3:
+			case 4:
+			case 250:	// highest connection count
+			case 251:	// user count
+			case 252:	// oper count
+			case 254:	// channel count
+			case 255:	// client count
+			case 265:	// local users
+			case 266:	// global users
+				m_pUser->UpdateRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+				break;
+			case 422:	// MOTD File is missing
+			case 375: 	// begin motd
+				m_pUser->ClearMotdBuffer();
+			case 372:	// motd
+			case 376:	// end motd
+				m_pUser->AddMotdBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+				break;
+			case 470: {
+				// :irc.unreal.net 470 mynick [Link] #chan1 has become full, so you are automatically being transferred to the linked channel #chan2
+				// :mccaffrey.freenode.net 470 mynick #electronics ##electronics :Forwarding to another channel
+
+				// freenode style numeric
+				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+				if (!pChan) {
+					// unreal style numeric
+					pChan = m_pUser->FindChan(sRest.Token(1));
 				}
+				if (pChan) {
+					pChan->Disable();
+					m_pUser->PutStatus("Channel [" + pChan->GetName() + "] is linked to "
+							"another channel and was thus disabled.");
+				}
+				break;
+			}
+			case 437:
+				// :irc.server.net 437 * badnick :Nick/channel is temporarily unavailable
+				// :irc.server.net 437 mynick badnick :Nick/channel is temporarily unavailable
+				// :irc.server.net 437 mynick badnick :Cannot change nickname while banned on channel
+				if (m_pUser->IsChan(sRest.Token(0)) || sNick != "*")
 					break;
-				case 353: {	// NAMES
-					sRest.Trim();
-					// Todo: allow for non @+= server msgs
-					CChan* pChan = m_pUser->FindChan(sRest.Token(1));
-					// If we don't know that channel, some client might have
-					// requested a /names for it and we really should forward this.
-					if (pChan) {
-						CString sNicks = sRest.Token(2, true);
-						if (sNicks.Left(1) == ":") {
-							sNicks.LeftChomp();
-						}
+			case 432: // :irc.server.com 432 * nick :Erroneous Nickname: Illegal characters
+			case 433: {
+				CString sBadNick = sRest.Token(0);
 
-						pChan->AddNicks(sNicks);
-					}
-
-					ForwardRaw353(sLine);
-
-					// We forwarded it already, so return
+				if (!m_bAuthed) {
+					SendAltNick(sBadNick);
 					return;
 				}
-				case 366: {	// end of names list
-					m_pUser->PutUser(sLine);	// First send them the raw
+				break;
+			}
+			case 331: {
+				// :irc.server.com 331 yournick #chan :No topic is set.
+				CChan* pChan = m_pUser->FindChan(sLine.Token(3));
 
-					// :irc.server.com 366 nick #chan :End of /NAMES list.
-					CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+				if (pChan) {
+					pChan->SetTopic("");
+				}
 
-					if (pChan) {
-						if (pChan->IsOn()) {
-							// If we are the only one in the chan, set our default modes
-							if (pChan->GetNickCount() == 1) {
-								CString sModes = pChan->GetDefaultModes();
+				break;
+			}
+			case 332: {
+				// :irc.server.com 332 yournick #chan :This is a topic
+				CChan* pChan = m_pUser->FindChan(sLine.Token(3));
 
-								if (sModes.empty()) {
-									sModes = m_pUser->GetDefaultChanModes();
-								}
+				if (pChan) {
+					CString sTopic = sLine.Token(4, true);
+					sTopic.LeftChomp();
+					pChan->SetTopic(sTopic);
+				}
 
-								if (!sModes.empty()) {
-									PutIRC("MODE " + pChan->GetName() + " " + sModes);
-								}
-							}
-						}
-					}
+				break;
+			}
+			case 333: {
+				// :irc.server.com 333 yournick #chan setternick 1112320796
+				CChan* pChan = m_pUser->FindChan(sLine.Token(3));
 
-					return;	// return so we don't send them the raw twice
+				if (pChan) {
+					sNick = sLine.Token(4);
+					unsigned long ulDate = sLine.Token(5).ToULong();
+
+					pChan->SetTopicOwner(sNick);
+					pChan->SetTopicDate(ulDate);
+				}
+
+				break;
+			}
+			case 352: {
+				// :irc.yourserver.com 352 yournick #chan ident theirhost.com irc.theirserver.com theirnick H :0 Real Name
+				sServer = sLine.Token(0);
+				sNick = sLine.Token(7);
+				CString sIdent = sLine.Token(4);
+				CString sHost = sLine.Token(5);
+
+				sServer.LeftChomp();
+
+				if (sNick.Equals(GetNick())) {
+					m_Nick.SetIdent(sIdent);
+					m_Nick.SetHost(sHost);
+				}
+
+				m_pUser->SetIRCNick(m_Nick);
+				m_pUser->SetIRCServer(sServer);
+
+				const vector<CChan*>& vChans = m_pUser->GetChans();
+
+				for (unsigned int a = 0; a < vChans.size(); a++) {
+					vChans[a]->OnWho(sNick, sIdent, sHost);
+				}
+
+				break;
+			}
+			case 324: {	// MODE
+				sRest.Trim();
+				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+
+				if (pChan) {
+					pChan->SetModes(sRest.Token(1, true));
 				}
 			}
-		} else { //if (CUtils::wildcmp(":*!*@* * *", sLine.c_str())) {
-			CNick Nick(sLine.Token(0).LeftChomp_n());
-			sCmd = sLine.Token(1);
-			CString sRest = sLine.Token(2, true);
-
-			if (sCmd.Equals("NICK")) {
-				CString sNewNick = sRest;
-				bool bIsVisible = false;
-
-				if (sNewNick.Left(1) == ":") {
-					sNewNick.LeftChomp();
-				}
-
-				vector<CChan*> vFoundChans;
-				const vector<CChan*>& vChans = m_pUser->GetChans();
-
-				for (unsigned int a = 0; a < vChans.size(); a++) {
-					CChan* pChan = vChans[a];
-
-					if (pChan->ChangeNick(Nick.GetNick(), sNewNick)) {
-						vFoundChans.push_back(pChan);
-
-						if (!pChan->IsDetached()) {
-							bIsVisible = true;
-						}
-					}
-				}
-
-				// Todo: use nick compare function here
-				if (Nick.GetNick().Equals(GetNick())) {
-					// We are changing our own nick, the clients always must see this!
-					bIsVisible = true;
-					SetNick(sNewNick);
-				}
-
-				MODULECALL(OnNick(Nick, sNewNick, vFoundChans), m_pUser, NULL, );
-
-				if (!bIsVisible) {
-					return;
-				}
-			} else if (sCmd.Equals("QUIT")) {
-				CString sMessage = sRest;
-				bool bIsVisible = false;
-
-				if (sMessage.Left(1) == ":") {
-					sMessage.LeftChomp();
-				}
-
-				// :nick!ident@host.com QUIT :message
-
-				if (Nick.GetNick().Equals(GetNick())) {
-					m_pUser->PutStatus("You quit [" + sMessage + "]");
-				}
-
-				vector<CChan*> vFoundChans;
-				const vector<CChan*>& vChans = m_pUser->GetChans();
-
-				for (unsigned int a = 0; a < vChans.size(); a++) {
-					CChan* pChan = vChans[a];
-
-					if (pChan->RemNick(Nick.GetNick())) {
-						vFoundChans.push_back(pChan);
-
-						if (!pChan->IsDetached()) {
-							bIsVisible = true;
-						}
-					}
-				}
-
-				MODULECALL(OnQuit(Nick, sMessage, vFoundChans), m_pUser, NULL, );
-
-				if (!bIsVisible) {
-					return;
-				}
-			} else if (sCmd.Equals("JOIN")) {
-				CString sChan = sRest.Token(0);
-				if (sChan.Left(1) == ":") {
-					sChan.LeftChomp();
-				}
-
-				CChan* pChan;
-
-				// Todo: use nick compare function
-				if (Nick.GetNick().Equals(GetNick())) {
-					m_pUser->AddChan(sChan, false);
-					pChan = m_pUser->FindChan(sChan);
-					if (pChan) {
-						pChan->ResetJoinTries();
-						pChan->Enable();
-						pChan->SetIsOn(true);
-						PutIRC("MODE " + pChan->GetName());
-
-					}
-				} else {
-					pChan = m_pUser->FindChan(sChan);
-				}
+				break;
+			case 329: {
+				sRest.Trim();
+				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
 
 				if (pChan) {
-					pChan->AddNick(Nick.GetNickMask());
-					MODULECALL(OnJoin(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
-
-					if (pChan->IsDetached()) {
-						return;
-					}
+					unsigned long ulDate = sLine.Token(4).ToULong();
+					pChan->SetCreationDate(ulDate);
 				}
-			} else if (sCmd.Equals("PART")) {
-				CString sChan = sRest.Token(0);
-				if (sChan.Left(1) == ":") {
-					sChan.LeftChomp();
-				}
-
-				CChan* pChan = m_pUser->FindChan(sChan);
-				bool bDetached = false;
+			}
+				break;
+			case 353: {	// NAMES
+				sRest.Trim();
+				// Todo: allow for non @+= server msgs
+				CChan* pChan = m_pUser->FindChan(sRest.Token(1));
+				// If we don't know that channel, some client might have
+				// requested a /names for it and we really should forward this.
 				if (pChan) {
-					pChan->RemNick(Nick.GetNick());
-					MODULECALL(OnPart(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
-
-					if (pChan->IsDetached())
-						bDetached = true;
-				}
-
-				// Todo: use nick compare function
-				if (Nick.GetNick().Equals(GetNick())) {
-					m_pUser->DelChan(sChan);
-				}
-
-				/*
-				 * We use this boolean because
-				 * m_pUser->DelChan() will delete this channel
-				 * and thus we would dereference an
-				 * already-freed pointer!
-				 */
-				if (bDetached) {
-					return;
-				}
-			} else if (sCmd.Equals("MODE")) {
-				CString sTarget = sRest.Token(0);
-				CString sModes = sRest.Token(1, true);
-				if (sModes.Left(1) == ":")
-					sModes = sModes.substr(1);
-
-				CChan* pChan = m_pUser->FindChan(sTarget);
-				if (pChan) {
-					pChan->ModeChange(sModes, Nick.GetNick());
-
-					if (pChan->IsDetached()) {
-						return;
+					CString sNicks = sRest.Token(2, true);
+					if (sNicks.Left(1) == ":") {
+						sNicks.LeftChomp();
 					}
-				} else if (sTarget == m_Nick.GetNick()) {
-					CString sModeArg = sModes.Token(0);
-//					CString sArgs = sModes.Token(1, true); Usermode changes got no params
-					bool bAdd = true;
-/* no module call defined (yet?)
-					MODULECALL(OnRawUserMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
-*/
-					for (unsigned int a = 0; a < sModeArg.size(); a++) {
-						const unsigned char& uMode = sModeArg[a];
 
-						if (uMode == '+') {
-							bAdd = true;
-						} else if (uMode == '-') {
-							bAdd = false;
-						} else {
-							if (bAdd) {
-								m_scUserModes.insert(uMode);
-							} else {
-								m_scUserModes.erase(uMode);
+					pChan->AddNicks(sNicks);
+				}
+
+				ForwardRaw353(sLine);
+
+				// We forwarded it already, so return
+				return;
+			}
+			case 366: {	// end of names list
+				m_pUser->PutUser(sLine);	// First send them the raw
+
+				// :irc.server.com 366 nick #chan :End of /NAMES list.
+				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+
+				if (pChan) {
+					if (pChan->IsOn()) {
+						// If we are the only one in the chan, set our default modes
+						if (pChan->GetNickCount() == 1) {
+							CString sModes = pChan->GetDefaultModes();
+
+							if (sModes.empty()) {
+								sModes = m_pUser->GetDefaultChanModes();
+							}
+
+							if (!sModes.empty()) {
+								PutIRC("MODE " + pChan->GetName() + " " + sModes);
 							}
 						}
 					}
 				}
-			} else if (sCmd.Equals("KICK")) {
-				// :opnick!ident@host.com KICK #chan nick :msg
-				CString sChan = sRest.Token(0);
-				CString sKickedNick = sRest.Token(1);
-				CString sMsg = sRest.Token(2, true);
-				sMsg.LeftChomp();
 
-				CChan* pChan = m_pUser->FindChan(sChan);
+				return;	// return so we don't send them the raw twice
+			}
+		}
+	} else {
+		CNick Nick(sLine.Token(0).LeftChomp_n());
+		sCmd = sLine.Token(1);
+		CString sRest = sLine.Token(2, true);
 
-				if (pChan) {
-					pChan->RemNick(sKickedNick);
-					MODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg), m_pUser, NULL, );
-				}
+		if (sCmd.Equals("NICK")) {
+			CString sNewNick = sRest;
+			bool bIsVisible = false;
 
-				if (GetNick().Equals(sKickedNick) && pChan) {
-					pChan->SetIsOn(false);
+			if (sNewNick.Left(1) == ":") {
+				sNewNick.LeftChomp();
+			}
 
-					// Don't try to rejoin!
-					pChan->Disable();
-				}
+			vector<CChan*> vFoundChans;
+			const vector<CChan*>& vChans = m_pUser->GetChans();
 
-				if ((pChan) && (pChan->IsDetached())) {
-					return;
-				}
-			} else if (sCmd.Equals("NOTICE")) {
-				// :nick!ident@host.com NOTICE #chan :Message
-				CString sTarget = sRest.Token(0);
-				CString sMsg = sRest.Token(1, true);
-				sMsg.LeftChomp();
+			for (unsigned int a = 0; a < vChans.size(); a++) {
+				CChan* pChan = vChans[a];
 
-				if (sMsg.WildCmp("\001*\001")) {
-					sMsg.LeftChomp();
-					sMsg.RightChomp();
+				if (pChan->ChangeNick(Nick.GetNick(), sNewNick)) {
+					vFoundChans.push_back(pChan);
 
-					if (sTarget.Equals(GetNick())) {
-						if (OnCTCPReply(Nick, sMsg)) {
-							return;
-						}
-					}
-
-					m_pUser->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :\001" + sMsg + "\001");
-					return;
-				} else {
-					if (sTarget.Equals(GetNick())) {
-						if (OnPrivNotice(Nick, sMsg)) {
-							return;
-						}
-					} else {
-						if (OnChanNotice(Nick, sTarget, sMsg)) {
-							return;
-						}
+					if (!pChan->IsDetached()) {
+						bIsVisible = true;
 					}
 				}
+			}
 
-				if (Nick.GetNick().Equals(m_pUser->GetIRCServer())) {
-					m_pUser->PutUser(":" + Nick.GetNick() + " NOTICE " + sTarget + " :" + sMsg);
-				} else {
-					m_pUser->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
-				}
+			// Todo: use nick compare function here
+			if (Nick.GetNick().Equals(GetNick())) {
+				// We are changing our own nick, the clients always must see this!
+				bIsVisible = true;
+				SetNick(sNewNick);
+			}
 
+			MODULECALL(OnNick(Nick, sNewNick, vFoundChans), m_pUser, NULL, );
+
+			if (!bIsVisible) {
 				return;
-			} else if (sCmd.Equals("TOPIC")) {
-				// :nick!ident@host.com TOPIC #chan :This is a topic
-				CChan* pChan = m_pUser->FindChan(sLine.Token(2));
+			}
+		} else if (sCmd.Equals("QUIT")) {
+			CString sMessage = sRest;
+			bool bIsVisible = false;
 
+			if (sMessage.Left(1) == ":") {
+				sMessage.LeftChomp();
+			}
+
+			// :nick!ident@host.com QUIT :message
+
+			if (Nick.GetNick().Equals(GetNick())) {
+				m_pUser->PutStatus("You quit [" + sMessage + "]");
+			}
+
+			vector<CChan*> vFoundChans;
+			const vector<CChan*>& vChans = m_pUser->GetChans();
+
+			for (unsigned int a = 0; a < vChans.size(); a++) {
+				CChan* pChan = vChans[a];
+
+				if (pChan->RemNick(Nick.GetNick())) {
+					vFoundChans.push_back(pChan);
+
+					if (!pChan->IsDetached()) {
+						bIsVisible = true;
+					}
+				}
+			}
+
+			MODULECALL(OnQuit(Nick, sMessage, vFoundChans), m_pUser, NULL, );
+
+			if (!bIsVisible) {
+				return;
+			}
+		} else if (sCmd.Equals("JOIN")) {
+			CString sChan = sRest.Token(0);
+			if (sChan.Left(1) == ":") {
+				sChan.LeftChomp();
+			}
+
+			CChan* pChan;
+
+			// Todo: use nick compare function
+			if (Nick.GetNick().Equals(GetNick())) {
+				m_pUser->AddChan(sChan, false);
+				pChan = m_pUser->FindChan(sChan);
 				if (pChan) {
-					CString sTopic = sLine.Token(3, true);
-					sTopic.LeftChomp();
+					pChan->ResetJoinTries();
+					pChan->Enable();
+					pChan->SetIsOn(true);
+					PutIRC("MODE " + pChan->GetName());
 
-					MODULECALL(OnTopic(Nick, *pChan, sTopic), m_pUser, NULL, return)
-
-					pChan->SetTopicOwner(Nick.GetNick());
-					pChan->SetTopicDate((unsigned long) time(NULL));
-					pChan->SetTopic(sTopic);
-
-					if (pChan->IsDetached()) {
-						return; // Don't forward this
-					}
-
-					sLine = ":" + Nick.GetNickMask() + " TOPIC " + pChan->GetName() + " :" + sTopic;
 				}
-			} else if (sCmd.Equals("PRIVMSG")) {
-				// :nick!ident@host.com PRIVMSG #chan :Message
-				CString sTarget = sRest.Token(0);
-				CString sMsg = sRest.Token(1, true);
+			} else {
+				pChan = m_pUser->FindChan(sChan);
+			}
 
-				if (sMsg.Left(1) == ":") {
-					sMsg.LeftChomp();
-				}
+			if (pChan) {
+				pChan->AddNick(Nick.GetNickMask());
+				MODULECALL(OnJoin(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
 
-				if (sMsg.WildCmp("\001*\001")) {
-					sMsg.LeftChomp();
-					sMsg.RightChomp();
-
-					if (sTarget.Equals(GetNick())) {
-						if (OnPrivCTCP(Nick, sMsg)) {
-							return;
-						}
-					} else {
-						if (OnChanCTCP(Nick, sTarget, sMsg)) {
-							return;
-						}
-					}
-
-					m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :\001" + sMsg + "\001");
+				if (pChan->IsDetached()) {
 					return;
+				}
+			}
+		} else if (sCmd.Equals("PART")) {
+			CString sChan = sRest.Token(0);
+			if (sChan.Left(1) == ":") {
+				sChan.LeftChomp();
+			}
+
+			CChan* pChan = m_pUser->FindChan(sChan);
+			bool bDetached = false;
+			if (pChan) {
+				pChan->RemNick(Nick.GetNick());
+				MODULECALL(OnPart(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
+
+				if (pChan->IsDetached())
+					bDetached = true;
+			}
+
+			// Todo: use nick compare function
+			if (Nick.GetNick().Equals(GetNick())) {
+				m_pUser->DelChan(sChan);
+			}
+
+			/*
+			 * We use this boolean because
+			 * m_pUser->DelChan() will delete this channel
+			 * and thus we would dereference an
+			 * already-freed pointer!
+			 */
+			if (bDetached) {
+				return;
+			}
+		} else if (sCmd.Equals("MODE")) {
+			CString sTarget = sRest.Token(0);
+			CString sModes = sRest.Token(1, true);
+			if (sModes.Left(1) == ":")
+				sModes = sModes.substr(1);
+
+			CChan* pChan = m_pUser->FindChan(sTarget);
+			if (pChan) {
+				pChan->ModeChange(sModes, Nick.GetNick());
+
+				if (pChan->IsDetached()) {
+					return;
+				}
+			} else if (sTarget == m_Nick.GetNick()) {
+				CString sModeArg = sModes.Token(0);
+//					CString sArgs = sModes.Token(1, true); Usermode changes got no params
+				bool bAdd = true;
+/* no module call defined (yet?)
+				MODULECALL(OnRawUserMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
+*/
+				for (unsigned int a = 0; a < sModeArg.size(); a++) {
+					const unsigned char& uMode = sModeArg[a];
+
+					if (uMode == '+') {
+						bAdd = true;
+					} else if (uMode == '-') {
+						bAdd = false;
+					} else {
+						if (bAdd) {
+							m_scUserModes.insert(uMode);
+						} else {
+							m_scUserModes.erase(uMode);
+						}
+					}
+				}
+			}
+		} else if (sCmd.Equals("KICK")) {
+			// :opnick!ident@host.com KICK #chan nick :msg
+			CString sChan = sRest.Token(0);
+			CString sKickedNick = sRest.Token(1);
+			CString sMsg = sRest.Token(2, true);
+			sMsg.LeftChomp();
+
+			CChan* pChan = m_pUser->FindChan(sChan);
+
+			if (pChan) {
+				pChan->RemNick(sKickedNick);
+				MODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg), m_pUser, NULL, );
+			}
+
+			if (GetNick().Equals(sKickedNick) && pChan) {
+				pChan->SetIsOn(false);
+
+				// Don't try to rejoin!
+				pChan->Disable();
+			}
+
+			if ((pChan) && (pChan->IsDetached())) {
+				return;
+			}
+		} else if (sCmd.Equals("NOTICE")) {
+			// :nick!ident@host.com NOTICE #chan :Message
+			CString sTarget = sRest.Token(0);
+			CString sMsg = sRest.Token(1, true);
+			sMsg.LeftChomp();
+
+			if (sMsg.WildCmp("\001*\001")) {
+				sMsg.LeftChomp();
+				sMsg.RightChomp();
+
+				if (sTarget.Equals(GetNick())) {
+					if (OnCTCPReply(Nick, sMsg)) {
+						return;
+					}
+				}
+
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :\001" + sMsg + "\001");
+				return;
+			} else {
+				if (sTarget.Equals(GetNick())) {
+					if (OnPrivNotice(Nick, sMsg)) {
+						return;
+					}
 				} else {
-					if (sTarget.Equals(GetNick())) {
-						if (OnPrivMsg(Nick, sMsg)) {
-							return;
-						}
-					} else {
-						if (OnChanMsg(Nick, sTarget, sMsg)) {
-							return;
-						}
+					if (OnChanNotice(Nick, sTarget, sMsg)) {
+						return;
 					}
-
-					m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :" + sMsg);
-					return;
 				}
-			} else if (sCmd.Equals("WALLOPS")) {
-				// :blub!dummy@rox-8DBEFE92 WALLOPS :this is a test
-				CString sMsg = sRest.Token(0, true);
+			}
 
-				if (sMsg.Left(1) == ":") {
-					sMsg.LeftChomp();
+			if (Nick.GetNick().Equals(m_pUser->GetIRCServer())) {
+				m_pUser->PutUser(":" + Nick.GetNick() + " NOTICE " + sTarget + " :" + sMsg);
+			} else {
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
+			}
+
+			return;
+		} else if (sCmd.Equals("TOPIC")) {
+			// :nick!ident@host.com TOPIC #chan :This is a topic
+			CChan* pChan = m_pUser->FindChan(sLine.Token(2));
+
+			if (pChan) {
+				CString sTopic = sLine.Token(3, true);
+				sTopic.LeftChomp();
+
+				MODULECALL(OnTopic(Nick, *pChan, sTopic), m_pUser, NULL, return)
+
+				pChan->SetTopicOwner(Nick.GetNick());
+				pChan->SetTopicDate((unsigned long) time(NULL));
+				pChan->SetTopic(sTopic);
+
+				if (pChan->IsDetached()) {
+					return; // Don't forward this
 				}
 
+				sLine = ":" + Nick.GetNickMask() + " TOPIC " + pChan->GetName() + " :" + sTopic;
+			}
+		} else if (sCmd.Equals("PRIVMSG")) {
+			// :nick!ident@host.com PRIVMSG #chan :Message
+			CString sTarget = sRest.Token(0);
+			CString sMsg = sRest.Token(1, true);
+
+			if (sMsg.Left(1) == ":") {
+				sMsg.LeftChomp();
+			}
+
+			if (sMsg.WildCmp("\001*\001")) {
+				sMsg.LeftChomp();
+				sMsg.RightChomp();
+
+				if (sTarget.Equals(GetNick())) {
+					if (OnPrivCTCP(Nick, sMsg)) {
+						return;
+					}
+				} else {
+					if (OnChanCTCP(Nick, sTarget, sMsg)) {
+						return;
+					}
+				}
+
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :\001" + sMsg + "\001");
+				return;
+			} else {
+				if (sTarget.Equals(GetNick())) {
+					if (OnPrivMsg(Nick, sMsg)) {
+						return;
+					}
+				} else {
+					if (OnChanMsg(Nick, sTarget, sMsg)) {
+						return;
+					}
+				}
+
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :" + sMsg);
+				return;
+			}
+		} else if (sCmd.Equals("WALLOPS")) {
+			// :blub!dummy@rox-8DBEFE92 WALLOPS :this is a test
+			CString sMsg = sRest.Token(0, true);
+
+			if (sMsg.Left(1) == ":") {
+				sMsg.LeftChomp();
+			}
+
+			if (!m_pUser->IsUserAttached()) {
 				m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " WALLOPS ", ":" + m_pUser->AddTimestamp(sMsg), false);
 			}
 		}
@@ -645,9 +644,9 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 		// DCC CHAT chat 2453612361 44592
 		CString sType = sMessage.Token(1);
 		CString sFile = sMessage.Token(2);
-		unsigned long uLongIP = strtoul(sMessage.Token(3).c_str(), NULL, 10);
-		unsigned short uPort = strtoul(sMessage.Token(4).c_str(), NULL, 10);
-		unsigned long uFileSize = strtoul(sMessage.Token(5).c_str(), NULL, 10);
+		unsigned long uLongIP = sMessage.Token(3).ToULong();
+		unsigned short uPort = sMessage.Token(4).ToUShort();
+		unsigned long uFileSize = sMessage.Token(5).ToULong();
 
 		if (sType.Equals("CHAT")) {
 			CNick FromNick(Nick.GetNickMask());
@@ -676,7 +675,7 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 				CDCCBounce* pSock = (CDCCBounce*) Manager[a];
 
 				if (pSock && pSock->GetSockName().Equals("DCC::", false, 5)) {
-					if (pSock->GetUserPort() == atoi(sMessage.Token(3).c_str())) {
+					if (pSock->GetUserPort() == sMessage.Token(3).ToUShort()) {
 						m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetLocalPort()) + " " + sMessage.Token(4) + "\001");
 					}
 				}
@@ -684,11 +683,10 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 		}
 
 		return true;
-	} else {
-		return OnGeneralCTCP(Nick, sMessage);
 	}
 
-	return false;
+	// This handles everything which wasn't handled yet
+	return OnGeneralCTCP(Nick, sMessage);
 }
 
 bool CIRCSock::OnGeneralCTCP(CNick& Nick, CString& sMessage) {
@@ -750,7 +748,7 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 	if (pChan) {
 		// Record a /me
 		if (sMessage.TrimPrefix("ACTION ")) {
-			if (pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
+			if (pChan->KeepBuffer() || !m_pUser->IsUserAttached() || pChan->IsDetached()) {
 				pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
 			}
 			MODULECALL(OnChanAction(Nick, *pChan, sMessage), m_pUser, NULL, return true);
@@ -770,7 +768,7 @@ bool CIRCSock::OnChanNotice(CNick& Nick, const CString& sChan, CString& sMessage
 	if (pChan) {
 		MODULECALL(OnChanNotice(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 
-		if ((pChan->KeepBuffer()) || (!m_pUser->IsUserAttached())) {
+		if (pChan->KeepBuffer() || !m_pUser->IsUserAttached() || pChan->IsDetached()) {
 			pChan->AddBuffer(":" + Nick.GetNickMask() + " NOTICE " + sChan + " :" + m_pUser->AddTimestamp(sMessage));
 		}
 	}
@@ -783,7 +781,7 @@ bool CIRCSock::OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage) {
 	if (pChan) {
 		MODULECALL(OnChanMsg(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 
-		if (pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
+		if (pChan->KeepBuffer() || !m_pUser->IsUserAttached() || pChan->IsDetached()) {
 			pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :" + m_pUser->AddTimestamp(sMessage));
 		}
 	}
@@ -792,7 +790,7 @@ bool CIRCSock::OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage) {
 }
 
 void CIRCSock::PutIRC(const CString& sLine) {
-	DEBUG_ONLY(cout << "(" << m_pUser->GetUserName() << ") ZNC -> IRC [" << sLine << "]" << endl);
+	DEBUG("(" << m_pUser->GetUserName() << ") ZNC -> IRC [" << sLine << "]");
 	Write(sLine + "\r\n");
 }
 
@@ -802,7 +800,7 @@ void CIRCSock::SetNick(const CString& sNick) {
 }
 
 void CIRCSock::Connected() {
-	DEBUG_ONLY(cout << GetSockName() << " == Connected()" << endl);
+	DEBUG(GetSockName() << " == Connected()");
 	m_pUser->IRCConnected(this);
 
 	CString sPass = m_sPass;
@@ -823,7 +821,7 @@ void CIRCSock::Connected() {
 void CIRCSock::Disconnected() {
 	MODULECALL(OnIRCDisconnected(), m_pUser, NULL, );
 
-	DEBUG_ONLY(cout << GetSockName() << " == Disconnected()" << endl);
+	DEBUG(GetSockName() << " == Disconnected()");
 	if (!m_pUser->IsBeingDeleted() && m_pUser->GetIRCConnectEnabled() &&
 			m_pUser->GetServers().size() != 0) {
 		m_pUser->PutStatus("Disconnected from IRC. Reconnecting...");
@@ -842,14 +840,18 @@ void CIRCSock::SockError(int iErrno) {
 		sError = "Your VHost could not be resolved";
 	} else if (iErrno == EADDRNOTAVAIL) {
 		// Csocket uses this if it can't resolve the dest host name
+		// ...but it also does generate this if bind() fails -.-
 		sError = strerror(iErrno);
-		sError += " (Is your IRC server's host name valid?)";
+		if (GetBindHost().empty())
+			sError += " (Is your IRC server's host name valid?)";
+		else
+			sError += " (Is your IRC server's host name and vhost valid?)";
 	} else {
 		sError = strerror(iErrno);
 	}
 
-	DEBUG_ONLY(cout << GetSockName() << " == SockError(" << iErrno << " "
-			<< sError << ")" << endl);
+	DEBUG(GetSockName() << " == SockError(" << iErrno << " "
+			<< sError << ")");
 	if (!m_pUser->IsBeingDeleted()) {
 		if (GetConState() != CST_OK)
 			m_pUser->PutStatus("Cannot connect to IRC (" +
@@ -866,7 +868,7 @@ void CIRCSock::SockError(int iErrno) {
 }
 
 void CIRCSock::Timeout() {
-	DEBUG_ONLY(cout << GetSockName() << " == Timeout()" << endl);
+	DEBUG(GetSockName() << " == Timeout()");
 	if (!m_pUser->IsBeingDeleted()) {
 		m_pUser->PutStatus("IRC connection timed out.  Reconnecting...");
 	}
@@ -878,7 +880,7 @@ void CIRCSock::Timeout() {
 }
 
 void CIRCSock::ConnectionRefused() {
-	DEBUG_ONLY(cout << GetSockName() << " == ConnectionRefused()" << endl);
+	DEBUG(GetSockName() << " == ConnectionRefused()");
 	if (!m_pUser->IsBeingDeleted()) {
 		m_pUser->PutStatus("Connection Refused.  Reconnecting...");
 	}
@@ -887,7 +889,7 @@ void CIRCSock::ConnectionRefused() {
 }
 
 void CIRCSock::ReachedMaxBuffer() {
-	DEBUG_ONLY(cout << GetSockName() << " == ReachedMaxBuffer()" << endl);
+	DEBUG(GetSockName() << " == ReachedMaxBuffer()");
 	m_pUser->PutStatus("Received a too long line from the IRC server!");
 	Quit();
 }
