@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,15 +22,17 @@
 	for (unsigned int a = 0; a < size(); a++) {			\
 		try {											\
 			type* pMod = (type *) (*this)[a];					\
+			CClient* pOldClient = pMod->GetClient();			\
 			pMod->SetClient(m_pClient);					\
 			if (m_pUser) {							\
+				CUser* pOldUser = pMod->GetUser();			\
 				pMod->SetUser(m_pUser);					\
 				pMod->func;						\
-				pMod->SetUser(NULL);					\
+				pMod->SetUser(pOldUser);				\
 			} else {							\
 				pMod->func;						\
 			}								\
-			pMod->SetClient(NULL);						\
+			pMod->SetClient(pOldClient);					\
 		} catch (CModule::EModException e) {			\
 			if (e == CModule::UNLOAD) {					\
 				UnloadModule((*this)[a]->GetModName());	\
@@ -47,15 +49,17 @@
 		try {											\
 			type* pMod = (type*) (*this)[a];			\
 			CModule::EModRet e = CModule::CONTINUE;		\
+			CClient* pOldClient = pMod->GetClient();			\
 			pMod->SetClient(m_pClient);				\
 			if (m_pUser) {								\
+				CUser* pOldUser = pMod->GetUser();			\
 				pMod->SetUser(m_pUser);					\
 				e = pMod->func;							\
-				pMod->SetUser(NULL);					\
+				pMod->SetUser(pOldUser);				\
 			} else {									\
 				e = pMod->func;							\
 			}											\
-			pMod->SetClient(NULL);					\
+			pMod->SetClient(pOldClient);				\
 			if (e == CModule::HALTMODS) {				\
 				break;									\
 			} else if (e == CModule::HALTCORE) {		\
@@ -99,14 +103,14 @@ const CString& CTimer::GetDescription() const { return m_sDescription; }
 /////////////////// !Timer ///////////////////
 
 /////////////////// Socket ///////////////////
-CSocket::CSocket(CModule* pModule) : Csock() {
+CSocket::CSocket(CModule* pModule) : CZNCSock() {
 	m_pModule = pModule;
 	m_pModule->AddSocket(this);
 	EnableReadLine();
 	SetMaxBufferThreshold(10240);
 }
 
-CSocket::CSocket(CModule* pModule, const CString& sHostname, unsigned short uPort, int iTimeout) : Csock(sHostname, uPort, iTimeout) {
+CSocket::CSocket(CModule* pModule, const CString& sHostname, unsigned short uPort, int iTimeout) : CZNCSock(sHostname, uPort, iTimeout) {
 	m_pModule = pModule;
 	m_pModule->AddSocket(this);
 	EnableReadLine();
@@ -114,15 +118,37 @@ CSocket::CSocket(CModule* pModule, const CString& sHostname, unsigned short uPor
 }
 
 CSocket::~CSocket() {
+	CUser *pUser = m_pModule->GetUser();
+
 	m_pModule->UnlinkSocket(this);
+
+	if (!m_pModule->IsGlobal() && pUser) {
+		pUser->AddBytesWritten(GetBytesWritten());
+		pUser->AddBytesRead(GetBytesRead());
+	} else {
+		CZNC::Get().AddBytesWritten(GetBytesWritten());
+		CZNC::Get().AddBytesRead(GetBytesRead());
+	}
 }
 
 void CSocket::ReachedMaxBuffer() {
-	DEBUG_ONLY(cout << GetSockName() << " == ReachedMaxBuffer()" << endl);
+	DEBUG(GetSockName() << " == ReachedMaxBuffer()");
 	PutModule("Some socket reached its max buffer limit and was closed!");
 	Close();
 }
 
+void CSocket::SockError(int iErrno) {
+	DEBUG(GetSockName() << " == SockError(" << strerror(iErrno) << ")");
+	if (iErrno == EMFILE) {
+		// We have too many open fds, this can cause a busy loop.
+		Close();
+	}
+}
+
+bool CSocket::ConnectionFrom(const CString& sHost, unsigned short uPort) {
+	return CZNC::Get().AllowConnectionFrom(sHost);
+}
+
 bool CSocket::Connect(const CString& sHostname, unsigned short uPort, bool bSSL, unsigned int uTimeout) {
 	CUser* pUser = m_pModule->GetUser();
 	CString sSockName = "MOD::C::" + m_pModule->GetModName();
@@ -138,7 +164,7 @@ bool CSocket::Connect(const CString& sHostname, unsigned short uPort, bool bSSL,
 		sSockName = GetSockName();
 	}
 
-	return m_pModule->GetManager()->Connect(sHostname, uPort, sSockName, uTimeout, bSSL, sVHost, (Csock*) this);
+	return m_pModule->GetManager()->Connect(sHostname, uPort, sSockName, uTimeout, bSSL, sVHost, this);
 }
 
 bool CSocket::Listen(unsigned short uPort, bool bSSL, unsigned int uTimeout) {
@@ -153,7 +179,7 @@ bool CSocket::Listen(unsigned short uPort, bool bSSL, unsigned int uTimeout) {
 		sSockName = GetSockName();
 	}
 
-	return m_pModule->GetManager()->ListenAll(uPort, sSockName, bSSL, SOMAXCONN, (Csock*) this);
+	return m_pModule->GetManager()->ListenAll(uPort, sSockName, bSSL, SOMAXCONN, this);
 }
 
 bool CSocket::PutIRC(const CString& sLine) {
@@ -179,8 +205,9 @@ void CSocket::SetModule(CModule* p) { m_pModule = p; }
 CModule* CSocket::GetModule() const { return m_pModule; }
 /////////////////// !Socket ///////////////////
 
-CModule::CModule(void* pDLL, CUser* pUser, const CString& sModName, const CString& sDataDir) {
+CModule::CModule(ModHandle pDLL, CUser* pUser, const CString& sModName, const CString& sDataDir) {
 	m_bFake = false;
+	m_bGlobal = false;
 	m_pDLL = pDLL;
 	m_pManager = &(CZNC::Get().GetManager());;
 	m_pUser = pUser;
@@ -194,7 +221,7 @@ CModule::CModule(void* pDLL, CUser* pUser, const CString& sModName, const CStrin
 	}
 }
 
-CModule::CModule(void* pDLL, const CString& sModName, const CString& sDataDir) {
+CModule::CModule(ModHandle pDLL, const CString& sModName, const CString& sDataDir) {
 	m_bFake = false;
 	m_pDLL = pDLL;
 	m_pManager = &(CZNC::Get().GetManager());
@@ -268,6 +295,15 @@ bool CModule::DelNV(const CString & sName, bool bWriteToDisk) {
 	return true;
 }
 
+bool CModule::ClearNV(bool bWriteToDisk) {
+	m_mssRegistry.clear();
+
+	if (bWriteToDisk) {
+		return SaveRegistry();
+	}
+	return true;
+}
+
 bool CModule::AddTimer(CTimer* pTimer) {
 	if ((!pTimer) || (FindTimer(pTimer->GetName()))) {
 		delete pTimer;
@@ -474,6 +510,11 @@ void CModule::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Chann
 void CModule::OnJoin(const CNick& Nick, CChan& Channel) {}
 void CModule::OnPart(const CNick& Nick, CChan& Channel) {}
 
+CModule::EModRet CModule::OnChanBufferStarting(CChan& Chan, CClient& Client) { return CONTINUE; }
+CModule::EModRet CModule::OnChanBufferEnding(CChan& Chan, CClient& Client) { return CONTINUE; }
+CModule::EModRet CModule::OnChanBufferPlayLine(CChan& Chan, CClient& Client, CString& sLine) { return CONTINUE; }
+CModule::EModRet CModule::OnPrivBufferPlayLine(CClient& Client, CString& sLine) { return CONTINUE; }
+
 void CModule::OnClientLogin() {}
 void CModule::OnClientDisconnect() {}
 CModule::EModRet CModule::OnUserRaw(CString& sLine) { return CONTINUE; }
@@ -499,7 +540,7 @@ CModule::EModRet CModule::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMe
 CModule::EModRet CModule::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { return CONTINUE; }
 CModule::EModRet CModule::OnTimerAutoJoin(CChan& Channel) { return CONTINUE; }
 
-void* CModule::GetDLL() { return m_pDLL; }
+ModHandle CModule::GetDLL() { return m_pDLL; }
 bool CModule::PutIRC(const CString& sLine) {
 	return (m_pUser) ? m_pUser->PutIRC(sLine) : false;
 }
@@ -544,6 +585,7 @@ CModule::EModRet CGlobalModule::OnDeleteUser(CUser& User) { return CONTINUE; }
 void CGlobalModule::OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort) {}
 CModule::EModRet CGlobalModule::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) { return CONTINUE; }
 void CGlobalModule::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {}
+CModule::EModRet CGlobalModule::OnUnknownUserRaw(CClient* pClient, CString& sLine) { return CONTINUE; }
 
 
 CModules::CModules() {
@@ -613,6 +655,10 @@ bool CModules::OnNick(const CNick& Nick, const CString& sNewNick, const vector<C
 bool CModules::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Channel, const CString& sMessage) { MODUNLOADCHK(OnKick(Nick, sKickedNick, Channel, sMessage)); return false; }
 bool CModules::OnJoin(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnJoin(Nick, Channel)); return false; }
 bool CModules::OnPart(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnPart(Nick, Channel)); return false; }
+bool CModules::OnChanBufferStarting(CChan& Chan, CClient& Client) { MODHALTCHK(OnChanBufferStarting(Chan, Client)); }
+bool CModules::OnChanBufferEnding(CChan& Chan, CClient& Client) { MODHALTCHK(OnChanBufferEnding(Chan, Client)); }
+bool CModules::OnChanBufferPlayLine(CChan& Chan, CClient& Client, CString& sLine) { MODHALTCHK(OnChanBufferPlayLine(Chan, Client, sLine)); }
+bool CModules::OnPrivBufferPlayLine(CClient& Client, CString& sLine) { MODHALTCHK(OnPrivBufferPlayLine(Client, sLine)); }
 bool CModules::OnCTCPReply(CNick& Nick, CString& sMessage) { MODHALTCHK(OnCTCPReply(Nick, sMessage)); }
 bool CModules::OnPrivCTCP(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivCTCP(Nick, sMessage)); }
 bool CModules::OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanCTCP(Nick, Channel, sMessage)); }
@@ -652,6 +698,10 @@ void CGlobalModules::OnFailedLogin(const CString& sUsername, const CString& sRem
 	GLOBALMODCALL(OnFailedLogin(sUsername, sRemoteIP));
 }
 
+bool CGlobalModules::OnUnknownUserRaw(CClient* pClient, CString& sLine) {
+	GLOBALMODHALTCHK(OnUnknownUserRaw(pClient, sLine));
+}
+
 CModule* CModules::FindModule(const CString& sModule) const {
 	for (unsigned int a = 0; a < size(); a++) {
 		if (sModule.Equals((*this)[a]->GetModName())) {
@@ -663,10 +713,6 @@ CModule* CModules::FindModule(const CString& sModule) const {
 }
 
 bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg, bool bFake) {
-#ifndef _MODULES
-	sRetMsg = "Unable to load module [" + sModule + "] module support was not enabled.";
-	return false;
-#else
 	sRetMsg = "";
 
 	for (unsigned int a = 0; a < sModule.length(); a++) {
@@ -698,10 +744,10 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 		return true;
 	}
 
-	unsigned int uDLFlags = RTLD_LAZY;
+	unsigned int uDLFlags = RTLD_NOW;
 	uDLFlags |= (pUser) ? RTLD_LOCAL : RTLD_GLOBAL;
 
-	void* p = dlopen((sModPath).c_str(), uDLFlags);
+	ModHandle p = dlopen((sModPath).c_str(), uDLFlags);
 
 	if (!p) {
 		sRetMsg = "Unable to load module [" + sModule + "] [" + dlerror() + "]";
@@ -709,11 +755,11 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	}
 
 	typedef double (*dFP)();
-	dFP Version = (dFP) dlsym(p, "GetVersion");
+	dFP Version = (dFP) dlsym(p, "ZNCModVersion");
 
 	if (!Version) {
 		dlclose(p);
-		sRetMsg = "Could not find Version() in module [" + sModule + "]";
+		sRetMsg = "Could not find ZNCModVersion() in module [" + sModule + "]";
 		return false;
 	}
 
@@ -724,20 +770,20 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	}
 
 	typedef bool (*bFP)();
-	bFP IsGlobal = (bFP) dlsym(p, "IsGlobal");
+	bFP IsGlobal = (bFP) dlsym(p, "ZNCModGlobal");
 
 	if (!IsGlobal) {
 		dlclose(p);
-		sRetMsg = "Could not find IsGlobal() in module [" + sModule + "]";
+		sRetMsg = "Could not find ZNCModGlobal() in module [" + sModule + "]";
 		return false;
 	}
 
 	typedef CString (*sFP)();
-	sFP GetDesc = (sFP) dlsym(p, "GetDescription");
+	sFP GetDesc = (sFP) dlsym(p, "ZNCModDescription");
 
 	if (!GetDesc) {
 		dlclose(p);
-		sRetMsg = "Could not find GetDescription() in module [" + sModule + "]";
+		sRetMsg = "Could not find ZNCModDescription() in module [" + sModule + "]";
 		return false;
 	}
 
@@ -753,25 +799,25 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	CModule* pModule = NULL;
 
 	if (pUser) {
-		typedef CModule* (*fp)(void*, CUser* pUser,
+		typedef CModule* (*fp)(ModHandle, CUser* pUser,
 				const CString& sModName, const CString& sDataPath);
-		fp Load = (fp) dlsym(p, "Load");
+		fp Load = (fp) dlsym(p, "ZNCModLoad");
 
 		if (!Load) {
 			dlclose(p);
-			sRetMsg = "Could not find Load() in module [" + sModule + "]";
+			sRetMsg = "Could not find ZNCModLoad() in module [" + sModule + "]";
 			return false;
 		}
 
 		pModule = Load(p, pUser, sModule, sDataPath);
 	} else {
-		typedef CModule* (*fp)(void*, const CString& sModName,
+		typedef CModule* (*fp)(ModHandle, const CString& sModName,
 				const CString& sDataPath);
-		fp Load = (fp) dlsym(p, "Load");
+		fp Load = (fp) dlsym(p, "ZNCModLoad");
 
 		if (!Load) {
 			dlclose(p);
-			sRetMsg = "Could not find Load() in module [" + sModule + "]";
+			sRetMsg = "Could not find ZNCModLoad() in module [" + sModule + "]";
 			return false;
 		}
 
@@ -779,12 +825,14 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	}
 
 	pModule->SetDescription(GetDesc());
+	pModule->SetGlobal(bIsGlobal);
+	pModule->SetArgs(sArgs);
 	push_back(pModule);
 
 	bool bLoaded;
 	try {
 		bLoaded = pModule->OnLoad(sArgs, sRetMsg);
-	} catch (CModule::EModException e) {
+	} catch (CModule::EModException) {
 		bLoaded = false;
 		sRetMsg = "Caught an exception";
 	}
@@ -798,15 +846,12 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 		return false;
 	}
 
-	pModule->SetArgs(sArgs);
-
 	if (!sRetMsg.empty()) {
 		sRetMsg = "Loaded module [" + sModule + "] [" + sRetMsg + "] [" + sModPath + "]";
 	} else {
 		sRetMsg = "Loaded module [" + sModule + "] [" + sModPath + "]";
 	}
 	return true;
-#endif // !_MODULES
 }
 
 bool CModules::UnloadModule(const CString& sModule) {
@@ -816,10 +861,6 @@ bool CModules::UnloadModule(const CString& sModule) {
 
 bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 	CString sMod = sModule;	// Make a copy incase the reference passed in is from CModule::GetModName()
-#ifndef _MODULES
-	sRetMsg = "Unable to unload module [" + sMod + "] module support was not enabled.";
-	return false;
-#else
 	CModule* pModule = FindModule(sMod);
 	sRetMsg = "";
 
@@ -841,11 +882,11 @@ bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 		return false;
 	}
 
-	void* p = pModule->GetDLL();
+	ModHandle p = pModule->GetDLL();
 
 	if (p) {
 		typedef void (*fp)(CModule*);
-		fp Unload = (fp)dlsym(p, "Unload");
+		fp Unload = (fp)dlsym(p, "ZNCModUnload");
 
 		if (Unload) {
 			Unload(pModule);
@@ -862,14 +903,13 @@ bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 
 			return true;
 		} else {
-			sRetMsg = "Unable to unload module [" + sMod + "] could not find Unload()";
+			sRetMsg = "Unable to unload module [" + sMod + "] could not find ZNCModUnload()";
 			return false;
 		}
 	}
 
 	sRetMsg = "Unable to unload module [" + sMod + "]";
 	return false;
-#endif // !_MODULES
 }
 
 bool CModules::ReloadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg) {
@@ -902,16 +942,16 @@ bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
 		return false;
 	}
 
-	unsigned int uDLFlags = RTLD_LAZY | RTLD_GLOBAL;
+	unsigned int uDLFlags = RTLD_LAZY | RTLD_LOCAL;
 
-	void* p = dlopen((sModPath).c_str(), uDLFlags);
+	ModHandle p = dlopen((sModPath).c_str(), uDLFlags);
 
 	if (!p) {
 		return false;
 	}
 
 	typedef double (*dFP)();
-	dFP Version = (dFP) dlsym(p, "GetVersion");
+	dFP Version = (dFP) dlsym(p, "ZNCModVersion");
 
 	if (!Version) {
 		dlclose(p);
@@ -919,7 +959,7 @@ bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
 	}
 
 	typedef bool (*bFP)();
-	bFP IsGlobal = (bFP) dlsym(p, "IsGlobal");
+	bFP IsGlobal = (bFP) dlsym(p, "ZNCModGlobal");
 
 	if (!IsGlobal) {
 		dlclose(p);
@@ -927,7 +967,7 @@ bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
 	}
 
 	typedef CString (*sFP)();
-	sFP GetDescription = (sFP) dlsym(p, "GetDescription");
+	sFP GetDescription = (sFP) dlsym(p, "ZNCModDescription");
 
 	if (!GetDescription) {
 		dlclose(p);
@@ -938,6 +978,11 @@ bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
 	ModInfo.SetDescription(GetDescription());
 	ModInfo.SetName(sModule);
 	ModInfo.SetPath(sModPath);
+
+	if (CModule::GetCoreVersion() != Version()) {
+		ModInfo.SetDescription("--- Version mismatch, recompile this module. ---");
+	}
+
 	dlclose(p);
 
 	return true;
@@ -958,7 +1003,6 @@ void CModules::GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
 
 		if (GetModInfo(ModInfo, sName)) {
 			if (ModInfo.IsGlobal() == bGlobal) {
-				ModInfo.SetSystem(false);
 				ssMods.insert(ModInfo);
 			}
 		}
@@ -973,7 +1017,6 @@ void CModules::GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
 
 		if (GetModInfo(ModInfo, sName)) {
 			if (ModInfo.IsGlobal() == bGlobal) {
-				ModInfo.SetSystem(false);
 				ssMods.insert(ModInfo);
 			}
 		}
@@ -988,7 +1031,6 @@ void CModules::GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
 
 		if (GetModInfo(ModInfo, sName)) {
 			if (ModInfo.IsGlobal() == bGlobal) {
-				ModInfo.SetSystem(true);
 				ssMods.insert(ModInfo);
 			}
 		}
