@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -36,41 +36,51 @@ class CClient;
 #endif
 #endif
 
+typedef void* ModHandle;
+
 #define MODCONSTRUCTOR(CLASS) \
-	CLASS(void *pDLL, CUser* pUser, const CString& sModName, \
+	CLASS(ModHandle pDLL, CUser* pUser, const CString& sModName, \
 			const CString& sModPath) \
 			: CModule(pDLL, pUser, sModName, sModPath)
 #define MODULEDEFS(CLASS, DESCRIPTION) \
 	extern "C" { \
-		CString GetDescription() { return DESCRIPTION; } \
-		bool IsGlobal() { return false; } \
-		CModule* Load(void* p, CUser* pUser, const CString& sModName, \
+		/* First the definitions to shut up some compiler warnings */ \
+		CString ZNCModDescription(); \
+		bool ZNCModGlobal(); \
+		double ZNCModVersion(); \
+		CModule* ZNCModLoad(ModHandle p, CUser* pUser, const CString& sModName, \
 				const CString& sModPath); \
-		void Unload(CModule* pMod); double GetVersion(); } \
-		double GetVersion() { return VERSION; } \
-		CModule* Load(void* p, CUser* pUser, const CString& sModName, \
+		void ZNCModUnload(CModule* pMod); \
+		CString ZNCModDescription() { return DESCRIPTION; } \
+		bool ZNCModGlobal() { return false; } \
+		double ZNCModVersion() { return VERSION; } \
+		CModule* ZNCModLoad(ModHandle p, CUser* pUser, const CString& sModName, \
 				const CString& sModPath) \
 		{ return new CLASS(p, pUser, sModName, sModPath); } \
-		void Unload(CModule* pMod) { if (pMod) { delete pMod; } \
+		void ZNCModUnload(CModule* pMod) { if (pMod) { delete pMod; } } \
 	}
 // !User Module Macros
 
 // Global Module Macros
 #define GLOBALMODCONSTRUCTOR(CLASS) \
-	CLASS(void *pDLL, const CString& sModName, const CString& sModPath) \
+	CLASS(ModHandle pDLL, const CString& sModName, const CString& sModPath) \
 			: CGlobalModule(pDLL, sModName, sModPath)
 #define GLOBALMODULEDEFS(CLASS, DESCRIPTION) \
 	extern "C" { \
-		CString GetDescription() { return DESCRIPTION; } \
-		bool IsGlobal() { return true; } \
-		CGlobalModule* Load(void* p, const CString& sModName, \
+		/* First the definitions to shut up some compiler warnings */ \
+		CString ZNCModDescription(); \
+		bool ZNCModGlobal(); \
+		double ZNCModVersion(); \
+		CGlobalModule* ZNCModLoad(ModHandle p, const CString& sModName, \
 				const CString& sModPath); \
-		void Unload(CGlobalModule* pMod); double GetVersion(); } \
-		double GetVersion() { return VERSION; } \
-		CGlobalModule* Load(void* p, const CString& sModName, \
+		void ZNCModUnload(CGlobalModule* pMod); \
+		CString ZNCModDescription() { return DESCRIPTION; } \
+		bool ZNCModGlobal() { return true; } \
+		double ZNCModVersion() { return VERSION; } \
+		CGlobalModule* ZNCModLoad(ModHandle p, const CString& sModName, \
 				const CString& sModPath) \
 		{ return new CLASS(p, sModName, sModPath); } \
-		void Unload(CGlobalModule* pMod) { if (pMod) { delete pMod; } \
+		void ZNCModUnload(CGlobalModule* pMod) { if (pMod) { delete pMod; } } \
 	}
 // !Global Module Macros
 
@@ -79,7 +89,6 @@ class CZNC;
 class CUser;
 class CNick;
 class CChan;
-class Csock;
 class CModule;
 class CFPTimer;
 class CSockManager;
@@ -130,7 +139,7 @@ private:
 	FPTimer_t	m_pFBCallback;
 };
 
-class CSocket : public Csock {
+class CSocket : public CZNCSock {
 public:
 	CSocket(CModule* pModule);
 	CSocket(CModule* pModule, const CString& sHostname, unsigned short uPort, int iTimeout = 60);
@@ -141,6 +150,10 @@ public:
 
 	// This defaults to closing the socket, feel free to override
 	virtual void ReachedMaxBuffer();
+	virtual void SockError(int iErrno);
+	// This limits the global connections from this IP to defeat DoS
+	// attacks, feel free to override
+	virtual bool ConnectionFrom(const CString& sHost, unsigned short uPort);
 
 	bool Connect(const CString& sHostname, unsigned short uPort, bool bSSL = false, unsigned int uTimeout = 60);
 	bool Listen(unsigned short uPort, bool bSSL = false, unsigned int uTimeout = 0);
@@ -165,13 +178,12 @@ protected:
 class CModInfo {
 public:
 	CModInfo() {}
-	CModInfo(const CString& sName, const CString& sPath, bool bSystem, bool bGlobal) {
-		m_bSystem = bSystem;
+	CModInfo(const CString& sName, const CString& sPath, bool bGlobal) {
 		m_bGlobal = bGlobal;
 		m_sName = sName;
 		m_sPath = sPath;
 	}
-	virtual ~CModInfo() {}
+	~CModInfo() {}
 
 	bool operator < (const CModInfo& Info) const {
 		return (GetName() < Info.GetName());
@@ -181,7 +193,6 @@ public:
 	const CString& GetName() const { return m_sName; }
 	const CString& GetPath() const { return m_sPath; }
 	const CString& GetDescription() const { return m_sDescription; }
-	bool IsSystem() const { return m_bSystem; }
 	bool IsGlobal() const { return m_bGlobal; }
 	// !Getters
 
@@ -189,12 +200,10 @@ public:
 	void SetName(const CString& s) { m_sName = s; }
 	void SetPath(const CString& s) { m_sPath = s; }
 	void SetDescription(const CString& s) { m_sDescription = s; }
-	void SetSystem(bool b) { m_bSystem = b; }
 	void SetGlobal(bool b) { m_bGlobal = b; }
 	// !Setters
 private:
 protected:
-	bool	m_bSystem;
 	bool	m_bGlobal;
 	CString	m_sName;
 	CString	m_sPath;
@@ -203,9 +212,9 @@ protected:
 
 class CModule {
 public:
-	CModule(void* pDLL, CUser* pUser, const CString& sModName,
+	CModule(ModHandle pDLL, CUser* pUser, const CString& sModName,
 			const CString& sDataDir);
-	CModule(void* pDLL, const CString& sModName, const CString& sDataDir);
+	CModule(ModHandle pDLL, const CString& sModName, const CString& sDataDir);
 	virtual ~CModule();
 
 	typedef enum {
@@ -255,6 +264,11 @@ public:
 	virtual void OnJoin(const CNick& Nick, CChan& Channel);
 	virtual void OnPart(const CNick& Nick, CChan& Channel);
 
+	virtual EModRet OnChanBufferStarting(CChan& Chan, CClient& Client);
+	virtual EModRet OnChanBufferEnding(CChan& Chan, CClient& Client);
+	virtual EModRet OnChanBufferPlayLine(CChan& Chan, CClient& Client, CString& sLine);
+	virtual EModRet OnPrivBufferPlayLine(CClient& Client, CString& sLine);
+
 	virtual void OnUserAttached(bool thisFunctionWasRemoved, bool UseOnClientLogin) {}
 	virtual void OnUserDetached(bool thisFunctionWasRemoved, bool UseOnClientDisconnect) {}
 	virtual void OnClientLogin();
@@ -282,7 +296,7 @@ public:
 	virtual EModRet OnTopic(CNick& Nick, CChan& Channel, CString& sTopic);
 	virtual EModRet OnTimerAutoJoin(CChan& Channel);
 
-	void * GetDLL();
+	ModHandle GetDLL();
 	static double GetCoreVersion() { return VERSION; }
 
 	virtual bool PutIRC(const CString& sLine);
@@ -303,6 +317,8 @@ public:
 	bool RemTimer(const CString& sLabel);
 	bool UnlinkTimer(CTimer* pTimer);
 	CTimer* FindTimer(const CString& sLabel);
+	vector<CTimer*>::const_iterator BeginTimers() const { return m_vTimers.begin(); }
+	vector<CTimer*>::const_iterator EndTimers() const { return m_vTimers.end(); }
 	virtual void ListTimers();
 	// !Timer stuff
 
@@ -312,6 +328,8 @@ public:
 	bool RemSocket(const CString& sSockName);
 	bool UnlinkSocket(CSocket* pSocket);
 	CSocket* FindSocket(const CString& sSockName);
+	vector<CSocket*>::const_iterator BeginSockets() const { return m_vSockets.begin(); }
+	vector<CSocket*>::const_iterator EndSockets() const { return m_vSockets.end(); }
 	virtual void ListSockets();
 	// !Socket stuff
 
@@ -324,17 +342,20 @@ public:
 	MCString::iterator EndNV() { return m_mssRegistry.end(); }
 	MCString::iterator BeginNV() { return m_mssRegistry.begin(); }
 	void DelNV(MCString::iterator it) { m_mssRegistry.erase(it); }
+	bool ClearNV(bool bWriteToDisk = true);
 
 	const CString& GetSavePath() const { if (!CFile::Exists(m_sSavePath)) { CDir::MakeDir(m_sSavePath); } return m_sSavePath; }
 
 	// Setters
 	void SetFake(bool b) { m_bFake = b; }
+	void SetGlobal(bool b) { m_bGlobal = b; }
 	void SetDescription(const CString& s) { m_sDescription = s; }
 	void SetArgs(const CString& s) { m_sArgs = s; }
 	// !Setters
 
 	// Getters
 	bool IsFake() const { return m_bFake; }
+	bool IsGlobal() const { return m_bGlobal; }
 	const CString& GetDescription() const { return m_sDescription; }
 	const CString& GetArgs() const { return m_sArgs; }
 	CUser* GetUser() { return m_pUser; }
@@ -344,10 +365,11 @@ public:
 
 protected:
 	bool			m_bFake;
+	bool			m_bGlobal;
 	CString			m_sDescription;
 	vector<CTimer*>		m_vTimers;
 	vector<CSocket*>	m_vSockets;
-	void*			m_pDLL;
+	ModHandle 		m_pDLL;
 	CSockManager*		m_pManager;
 	CUser*			m_pUser;
 	CClient*		m_pClient;
@@ -362,68 +384,75 @@ private:
 class CModules : public vector<CModule*> {
 public:
 	CModules();
-	virtual ~CModules();
+	~CModules();
 
 	void SetUser(CUser* pUser) { m_pUser = pUser; }
 	void SetClient(CClient* pClient) { m_pClient = pClient; }
+	CUser* GetUser() { return m_pUser; }
+	CClient* GetClient() { return m_pClient; }
 
 	void UnloadAll();
 
-	virtual bool OnBoot();						// Return false to abort
-	virtual bool OnPreRehash();
-	virtual bool OnPostRehash();
-	virtual bool OnIRCDisconnected();
-	virtual bool OnIRCConnected();
-	virtual bool OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName);
-	virtual bool OnBroadcast(CString& sMessage);
+	bool OnBoot();						// Return false to abort
+	bool OnPreRehash();
+	bool OnPostRehash();
+	bool OnIRCDisconnected();
+	bool OnIRCConnected();
+	bool OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName);
+	bool OnBroadcast(CString& sMessage);
 
-	virtual bool OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize);
+	bool OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize);
 
-	virtual bool OnChanPermission(const CNick& OpNick, const CNick& Nick, CChan& Channel, unsigned char uMode, bool bAdded, bool bNoChange);
-	virtual bool OnOp(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
-	virtual bool OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
-	virtual bool OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
-	virtual bool OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
-	virtual bool OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs);
-	virtual bool OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CString& sArg, bool bAdded, bool bNoChange);
+	bool OnChanPermission(const CNick& OpNick, const CNick& Nick, CChan& Channel, unsigned char uMode, bool bAdded, bool bNoChange);
+	bool OnOp(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
+	bool OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
+	bool OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
+	bool OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
+	bool OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs);
+	bool OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CString& sArg, bool bAdded, bool bNoChange);
 
-	virtual bool OnRaw(CString& sLine);
+	bool OnRaw(CString& sLine);
 
-	virtual bool OnStatusCommand(CString& sCommand);
-	virtual bool OnModCommand(const CString& sCommand);
-	virtual bool OnModNotice(const CString& sMessage);
-	virtual bool OnModCTCP(const CString& sMessage);
+	bool OnStatusCommand(CString& sCommand);
+	bool OnModCommand(const CString& sCommand);
+	bool OnModNotice(const CString& sMessage);
+	bool OnModCTCP(const CString& sMessage);
 
-	virtual bool OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans);
-	virtual bool OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans);
-	virtual bool OnKick(const CNick& Nick, const CString& sOpNick, CChan& Channel, const CString& sMessage);
-	virtual bool OnJoin(const CNick& Nick, CChan& Channel);
-	virtual bool OnPart(const CNick& Nick, CChan& Channel);
+	bool OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans);
+	bool OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans);
+	bool OnKick(const CNick& Nick, const CString& sOpNick, CChan& Channel, const CString& sMessage);
+	bool OnJoin(const CNick& Nick, CChan& Channel);
+	bool OnPart(const CNick& Nick, CChan& Channel);
 
-	virtual bool OnClientLogin();
-	virtual bool OnClientDisconnect();
-	virtual bool OnUserRaw(CString& sLine);
-	virtual bool OnUserCTCPReply(CString& sTarget, CString& sMessage);
-	virtual bool OnUserCTCP(CString& sTarget, CString& sMessage);
-	virtual bool OnUserAction(CString& sTarget, CString& sMessage);
-	virtual bool OnUserMsg(CString& sTarget, CString& sMessage);
-	virtual bool OnUserNotice(CString& sTarget, CString& sMessage);
-	virtual bool OnUserJoin(CString& sChannel, CString& sKey);
-	virtual bool OnUserPart(CString& sChannel, CString& sMessage);
-	virtual bool OnUserTopic(CString& sChannel, CString& sTopic);
-	virtual bool OnUserTopicRequest(CString& sChannel);
+	bool OnChanBufferStarting(CChan& Chan, CClient& Client);
+	bool OnChanBufferEnding(CChan& Chan, CClient& Client);
+	bool OnChanBufferPlayLine(CChan& Chan, CClient& Client, CString& sLine);
+	bool OnPrivBufferPlayLine(CClient& Client, CString& sLine);
 
-	virtual bool OnCTCPReply(CNick& Nick, CString& sMessage);
-	virtual bool OnPrivCTCP(CNick& Nick, CString& sMessage);
-	virtual bool OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage);
-	virtual bool OnPrivAction(CNick& Nick, CString& sMessage);
-	virtual bool OnChanAction(CNick& Nick, CChan& Channel, CString& sMessage);
-	virtual bool OnPrivMsg(CNick& Nick, CString& sMessage);
-	virtual bool OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage);
-	virtual bool OnPrivNotice(CNick& Nick, CString& sMessage);
-	virtual bool OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage);
-	virtual bool OnTopic(CNick& Nick, CChan& Channel, CString& sTopic);
-	virtual bool OnTimerAutoJoin(CChan& Channel);
+	bool OnClientLogin();
+	bool OnClientDisconnect();
+	bool OnUserRaw(CString& sLine);
+	bool OnUserCTCPReply(CString& sTarget, CString& sMessage);
+	bool OnUserCTCP(CString& sTarget, CString& sMessage);
+	bool OnUserAction(CString& sTarget, CString& sMessage);
+	bool OnUserMsg(CString& sTarget, CString& sMessage);
+	bool OnUserNotice(CString& sTarget, CString& sMessage);
+	bool OnUserJoin(CString& sChannel, CString& sKey);
+	bool OnUserPart(CString& sChannel, CString& sMessage);
+	bool OnUserTopic(CString& sChannel, CString& sTopic);
+	bool OnUserTopicRequest(CString& sChannel);
+
+	bool OnCTCPReply(CNick& Nick, CString& sMessage);
+	bool OnPrivCTCP(CNick& Nick, CString& sMessage);
+	bool OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage);
+	bool OnPrivAction(CNick& Nick, CString& sMessage);
+	bool OnChanAction(CNick& Nick, CChan& Channel, CString& sMessage);
+	bool OnPrivMsg(CNick& Nick, CString& sMessage);
+	bool OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage);
+	bool OnPrivNotice(CNick& Nick, CString& sMessage);
+	bool OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage);
+	bool OnTopic(CNick& Nick, CChan& Channel, CString& sTopic);
+	bool OnTimerAutoJoin(CChan& Channel);
 
 	CModule* FindModule(const CString& sModule) const;
 	bool LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg, bool bFake = false);
@@ -441,7 +470,7 @@ protected:
 
 class CGlobalModule : public CModule {
 public:
-	CGlobalModule(void* pDLL, const CString& sModName,
+	CGlobalModule(ModHandle pDLL, const CString& sModName,
 			const CString &sDataDir) : CModule(pDLL, sModName, sDataDir) {}
 	virtual ~CGlobalModule() {}
 
@@ -450,19 +479,21 @@ public:
 	virtual void OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort);
 	virtual EModRet OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
 	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
+	virtual EModRet OnUnknownUserRaw(CClient* pClient, CString& sLine);
 private:
 };
 
 class CGlobalModules : public CModules {
 public:
 	CGlobalModules() : CModules() {}
-	virtual ~CGlobalModules() {}
+	~CGlobalModules() {}
 
-	virtual bool OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan);
-	virtual bool OnDeleteUser(CUser& User);
-	virtual void OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort);
-	virtual bool OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
-	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
+	bool OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan);
+	bool OnDeleteUser(CUser& User);
+	void OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort);
+	bool OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
+	void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
+	bool OnUnknownUserRaw(CClient* pClient, CString& sLine);
 private:
 };
 
