@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -26,18 +26,15 @@ class CNick
 public:
 	CNick();
 	CNick(const CString& sNick);
-	virtual ~CNick();
+	~CNick();
 
 	void Reset();
 	void Parse(const CString& sNickMask);
 	CString GetHostMask() const;
 	unsigned int GetCommonChans(vector<CChan*>& vChans, CUser* pUser) const;
-	static CString Concat(const CString& sNick, const CString& sSuffix, unsigned int uMaxNickLen = 9);
 
 	// Setters
 	void SetUser(CUser* pUser);
-	void UpdatePermChar();
-	void SetPermChar(char c);
 	void SetNick(const CString& s);
 	void SetIdent(const CString& s);
 	void SetHost(const CString& s);
@@ -46,7 +43,6 @@ public:
 	// !Setters
 
 	// Getters
-	const set<unsigned char>& GetChanPerms() const;
 	CString GetPermStr() const;
 	unsigned char GetPermChar() const;
 	bool HasPerm(unsigned char uPerm) const;
@@ -57,8 +53,7 @@ public:
 	// !Getters
 private:
 protected:
-	set<unsigned char>	m_suChanPerms;
-	unsigned char		m_cPerm;
+	CString			m_sChanPerms;
 	CUser*				m_pUser;
 	CString				m_sNick;
 	CString				m_sIdent;
