@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -14,12 +14,36 @@
 #include "Utils.h"
 #include <iostream>
 
+using std::ostream;
 using std::cout;
 using std::endl;
-using std::ostream;
 
 class CTemplate;
 
+class CTemplateTagHandler {
+public:
+	CTemplateTagHandler() {}
+	virtual ~CTemplateTagHandler() {}
+
+	virtual bool HandleVar(CTemplate& Tmpl, const CString& sName, const CString& sArgs, CString& sOutput) {
+		return false;
+	}
+
+	virtual bool HandleTag(CTemplate& Tmpl, const CString& sName, const CString& sArgs, CString& sOutput) {
+		return false;
+	}
+
+	virtual bool HandleIf(CTemplate& Tmpl, const CString& sName, const CString& sArgs, CString& sOutput) {
+		return HandleVar(Tmpl, sName, sArgs, sOutput);
+	}
+
+	virtual bool HandleValue(CTemplate& Tmpl, CString& sValue, const MCString& msOptions) {
+		return false;
+	}
+private:
+};
+class CTemplate;
+
 class CTemplateOptions {
 public:
 	CTemplateOptions() {
@@ -43,16 +67,19 @@ private:
 
 class CTemplateLoopContext {
 public:
-	CTemplateLoopContext(unsigned long uFilePos, const CString& sLoopName, vector<CTemplate*>* pRows) {
+	CTemplateLoopContext(unsigned long uFilePos, const CString& sLoopName, bool bReverse, vector<CTemplate*>* pRows) {
 		m_uFilePosition = uFilePos;
 		m_sName = sLoopName;
 		m_uRowIndex = 0;
+		m_bReverse = bReverse;
 		m_pvRows = pRows;
+		m_bHasData = false;
 	}
 
 	virtual ~CTemplateLoopContext() {}
 
 	// Setters
+	void SetHasData(bool b = true) { m_bHasData = b; }
 	void SetName(const CString& s) { m_sName = s; }
 	void SetRowIndex(unsigned int u) { m_uRowIndex = u; }
 	unsigned int IncRowIndex() { return ++m_uRowIndex; }
@@ -61,6 +88,7 @@ public:
 	// !Setters
 
 	// Getters
+	bool HasData() const { return m_bHasData; }
 	const CString& GetName() const { return m_sName; }
 	unsigned long GetFilePosition() const { return m_uFilePosition; }
 	unsigned int GetRowIndex() const { return m_uRowIndex; }
@@ -70,32 +98,70 @@ public:
 	CTemplate* GetCurRow() { return GetRow(m_uRowIndex); }
 
 	CTemplate* GetRow(unsigned int uIndex);
-	CString GetValue(const CString& sName);
+	CString GetValue(const CString& sName, bool bFromIf = false);
 	// !Getters
 private:
-protected:
-	CString				m_sName;			//! The name portion of the <?LOOP name?> tag
-	unsigned int		m_uRowIndex;		//! The index of the current row we're on
-	unsigned long		m_uFilePosition;	//! The file position of the opening <?LOOP?> tag
-	vector<CTemplate*>*	m_pvRows;			//! This holds pointers to the templates associated with this loop
+	bool				m_bReverse;			//!< Iterate through this loop in reverse order
+	bool				m_bHasData;			//!< Tells whether this loop has real data or not
+	CString				m_sName;			//!< The name portion of the <?LOOP name?> tag
+	unsigned int		m_uRowIndex;		//!< The index of the current row we're on
+	unsigned long		m_uFilePosition;	//!< The file position of the opening <?LOOP?> tag
+	vector<CTemplate*>*	m_pvRows;			//!< This holds pointers to the templates associated with this loop
 };
 
 
 class CTemplate : public MCString {
 public:
-	CTemplate() : MCString(), m_spOptions(new CTemplateOptions) {}
-	CTemplate(const CString& sFileName) : MCString(), m_sFileName(sFileName), m_spOptions(new CTemplateOptions) {}
-	CTemplate(const CSmartPtr<CTemplateOptions>& Options) : MCString(), m_spOptions(Options) {}
+	CTemplate() : MCString(), m_spOptions(new CTemplateOptions) {
+		Init();
+	}
+
+	CTemplate(const CString& sFileName) : MCString(), m_sFileName(sFileName), m_spOptions(new CTemplateOptions) {
+		Init();
+	}
+
+	CTemplate(const CSmartPtr<CTemplateOptions>& Options, CTemplate* pParent = NULL) : MCString(), m_spOptions(Options) {
+		Init();
+		m_pParent = pParent;
+	}
+
 	virtual ~CTemplate();
 
+	//! Class for implementing custom tags in subclasses
+	void AddTagHandler(CSmartPtr<CTemplateTagHandler> spTagHandler) {
+		m_vspTagHandlers.push_back(spTagHandler);
+	}
+
+	vector<CSmartPtr<CTemplateTagHandler> >& GetTagHandlers() {
+		if (m_pParent) {
+			return m_pParent->GetTagHandlers();
+		}
+
+		return m_vspTagHandlers;
+	}
+ 
+	CString ResolveLiteral(const CString& sString);
+
+	void Init();
+
+	CTemplate* GetParent(bool bRoot);
+	CString ExpandFile(const CString& sFilename);
 	bool SetFile(const CString& sFileName);
+
+	void SetPath(const CString& sPath);	// Sets the dir:dir:dir type path to look at for templates, as of right now no ../../.. protection
+	void PrependPath(const CString& sPath);
+	void AppendPath(const CString& sPath);
+	void RemovePath(const CString& sPath);
+	void ClearPath();
+	CString ResolvePath(const CString& sPath, const CString& sFilename);
+	bool PrintString(CString& sRet);
 	bool Print(ostream& oOut = cout);
 	bool Print(const CString& sFileName, ostream& oOut = cout);
 	bool ValidIf(const CString& sArgs);
 	bool ValidExpr(const CString& sExpr);
 	bool IsTrue(const CString& sName);
 	bool HasLoop(const CString& sName);
-	CString GetValue(const CString& sName);
+	CString GetValue(const CString& sName, bool bFromIf = false);
 	CTemplate& AddRow(const CString& sName);
 	CTemplate* GetRow(const CString& sName, unsigned int uIndex);
 	vector<CTemplate*>* GetLoop(const CString& sName);
@@ -107,11 +173,13 @@ public:
 	const CString& GetFileName() const { return m_sFileName; }
 	// !Getters
 private:
-protected:
+	CTemplate*							m_pParent;
 	CString								m_sFileName;
+	LCString							m_lsPaths;
 	map<CString, vector<CTemplate*> >	m_mvLoops;
 	vector<CTemplateLoopContext*>		m_vLoopContexts;
 	CSmartPtr<CTemplateOptions>			m_spOptions;
+	vector<CSmartPtr<CTemplateTagHandler> >		m_vspTagHandlers;
 };
 
 #endif // !_TEMPLATE_H
