@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -40,7 +40,7 @@ CUser::CUser(const CString& sUserName) {
 	m_sStatusPrefix = "*";
 	m_sChanPrefixes = "";
 	m_uBufferCount = 50;
-	m_uMaxJoinTries = 0;
+	m_uMaxJoinTries = 10;
 	m_uMaxJoins = 5;
 	m_bKeepBuffer = false;
 	m_bBeingDeleted = false;
@@ -69,6 +69,13 @@ CUser::~CUser() {
 		delete m_vChans[b];
 	}
 
+	// This will cause an endless loop if the destructor doesn't remove the
+	// socket from this list / if the socket doesn't exist any more.
+	while (!m_sDCCBounces.empty())
+		CZNC::Get().GetManager().DelSockByAddr((CZNCSock*) *m_sDCCBounces.begin());
+	while (!m_sDCCSocks.empty())
+		CZNC::Get().GetManager().DelSockByAddr((CZNCSock*) *m_sDCCSocks.begin());
+
 	CZNC::Get().GetManager().DelCronByAddr(m_pJoinTimer);
 	CZNC::Get().GetManager().DelCronByAddr(m_pMiscTimer);
 }
@@ -80,6 +87,33 @@ void CUser::DelModules() {
 		m_pModules = NULL;
 	}
 }
+
+bool CUser::UpdateModule(const CString &sModule) {
+	const map<CString,CUser*>& Users = CZNC::Get().GetUserMap();
+	map<CString,CUser*>::const_iterator it;
+	map<CUser*, CString> Affected;
+	map<CUser*, CString>::iterator it2;
+	bool error = false;
+
+	for (it = Users.begin(); it != Users.end(); it++) {
+		CModule *pMod = it->second->GetModules().FindModule(sModule);
+		if (pMod) {
+			Affected[it->second] = pMod->GetArgs();
+			it->second->GetModules().UnloadModule(pMod->GetModName());
+		}
+	}
+
+	CString sErr;
+	for (it2 = Affected.begin(); it2 != Affected.end(); it2++) {
+		if (!it2->first->GetModules().LoadModule(sModule, it2->second, it2->first, sErr)) {
+			error = true;
+			DEBUG("Failed to reload [" << sModule << "] for [" << it2->first->GetUserName()
+					<< "]: " << sErr);
+		}
+	}
+
+	return !error;
+}
 #endif
 
 void CUser::DelClients() {
@@ -171,9 +205,13 @@ CString& CUser::AddTimestamp(const CString& sStr, CString& sRet) const {
 	} else {
 		time(&tm);
 		tm += (time_t)(m_fTimezoneOffset * 60 * 60); // offset is in hours
-		strftime(szTimestamp, sizeof(szTimestamp) / sizeof(char), GetTimestampFormat().c_str(), localtime(&tm));
+		size_t i = strftime(szTimestamp, sizeof(szTimestamp), GetTimestampFormat().c_str(), localtime(&tm));
+		if (i != 0) {
+			sRet = sStr;
+		} else {
+			sRet.clear();
+		}
 
-		sRet = sStr;
 		if (m_bPrependTimestamp) {
 			sRet = szTimestamp;
 			sRet += " " + sStr;
@@ -231,7 +269,7 @@ void CUser::UserConnected(CClient* pClient) {
 	if (GetIRCSock() != NULL) {
 		CString sUserMode("");
 		const set<unsigned char>& scUserModes = GetIRCSock()->GetUserModes();
-		for (set<unsigned char>::iterator it = scUserModes.begin();
+		for (set<unsigned char>::const_iterator it = scUserModes.begin();
 				it != scUserModes.end(); it++) {
 			sUserMode += *it;
 		}
@@ -249,6 +287,7 @@ void CUser::UserConnected(CClient* pClient) {
 
 	CString sBufLine;
 	while (m_QueryBuffer.GetNextLine(GetIRCNick().GetNick(), sBufLine)) {
+		MODULECALL(OnPrivBufferPlayLine(*pClient, sBufLine), this, NULL, continue);
 		pClient->PutClient(sBufLine);
 	}
 
@@ -451,7 +490,7 @@ bool CUser::IsHostAllowed(const CString& sHostMask) const {
 		return true;
 	}
 
-	for (set<CString>::iterator a = m_ssAllowedHosts.begin(); a != m_ssAllowedHosts.end(); a++) {
+	for (set<CString>::const_iterator a = m_ssAllowedHosts.begin(); a != m_ssAllowedHosts.end(); a++) {
 		if (sHostMask.WildCmp(*a)) {
 			return true;
 		}
@@ -480,7 +519,7 @@ bool CUser::IsValidUserName(const CString& sUserName) {
 			return false;
 		}
 
-		*p++;
+		p++;
 	}
 
 	return true;
@@ -550,11 +589,16 @@ bool CUser::PrintLine(CFile& File, const CString& sName, const CString& sValue)
 		return false;
 	}
 
-	return File.Write("\t" + sName + " = " + sValue + "\n");
+	// FirstLine() so that no one can inject new lines to the config if he
+	// manages to add "\n" to e.g. sValue.
+	CString sLine = "\t" + sName.FirstLine() + " = " + sValue.FirstLine() + "\n";
+	if (File.Write(sLine) <= 0)
+		return false;
+	return true;
 }
 
 bool CUser::WriteConfig(CFile& File) {
-	File.Write("<User " + GetUserName() + ">\n");
+	File.Write("<User " + GetUserName().FirstLine() + ">\n");
 
 	if (IsPassHashed()) {
 		if (m_sPassSalt.empty()) {
@@ -730,7 +774,7 @@ bool CUser::DelServer(const CString& sName) {
 	return false;
 }
 
-bool CUser::AddServer(const CString& sName, bool bIPV6) {
+bool CUser::AddServer(const CString& sName) {
 	if (sName.empty()) {
 		return false;
 	}
@@ -747,25 +791,19 @@ bool CUser::AddServer(const CString& sName, bool bIPV6) {
 		sPort.LeftChomp();
 	}
 
-	unsigned short uPort = strtoul(sPort.c_str(), NULL, 10);
+	unsigned short uPort = sPort.ToUShort();
 	CString sPass = sLine.Token(2, true);
 
-	return AddServer(sHost, uPort, sPass, bSSL, bIPV6);
+	return AddServer(sHost, uPort, sPass, bSSL);
 }
 
-bool CUser::AddServer(const CString& sName, unsigned short uPort, const CString& sPass, bool bSSL, bool bIPV6) {
+bool CUser::AddServer(const CString& sName, unsigned short uPort, const CString& sPass, bool bSSL) {
 #ifndef HAVE_LIBSSL
 	if (bSSL) {
 		return false;
 	}
 #endif
 
-#ifndef HAVE_IPV6
-	if (bIPV6) {
-		return false;
-	}
-#endif
-
 	if (sName.empty()) {
 		return false;
 	}
@@ -774,7 +812,27 @@ bool CUser::AddServer(const CString& sName, unsigned short uPort, const CString&
 		uPort = 6667;
 	}
 
-	CServer* pServer = new CServer(sName, uPort, sPass, bSSL, bIPV6);
+	// Check if server is already added
+	for (unsigned int a = 0; a < m_vServers.size(); a++) {
+		CServer* pServer = m_vServers[a];
+
+		if (!sName.Equals(pServer->GetName()))
+			continue;
+
+		if (uPort != pServer->GetPort())
+			continue;
+
+		if (sPass != pServer->GetPass())
+			continue;
+
+		if (bSSL != pServer->IsSSL())
+			continue;
+
+		// Server is already added
+		return false;
+	}
+
+	CServer* pServer = new CServer(sName, uPort, sPass, bSSL);
 	m_vServers.push_back(pServer);
 
 	CheckIRCConnect();
@@ -813,9 +871,7 @@ bool CUser::CheckPass(const CString& sPass) const {
 		return (sPass == m_sPass);
 	}
 
-	CString sSaltedPass = sPass + m_sPassSalt;
-
-	return (m_sPass.Equals(sSaltedPass.MD5()));
+	return m_sPass.Equals(CUtils::SaltedHash(sPass, m_sPassSalt));
 }
 
 /*CClient* CUser::GetClient() {
