@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -139,8 +139,8 @@ CString::CString(long i) : string() { stringstream s; s << i; *this = s.str(); }
 CString::CString(unsigned long i) : string() { stringstream s; s << i; *this = s.str(); }
 CString::CString(long long i) : string() { stringstream s; s << i; *this = s.str(); }
 CString::CString(unsigned long long i) : string() { stringstream s; s << i; *this = s.str(); }
-CString::CString(double i) : string() { stringstream s; s << i; *this = s.str(); }
-CString::CString(float i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(double i, int precision) : string() { stringstream s; s.precision(precision); s << std::fixed << i; *this = s.str(); }
+CString::CString(float i, int precision) : string() { stringstream s; s.precision(precision); s << std::fixed << i; *this = s.str(); }
 
 inline unsigned char* CString::strnchr(const unsigned char* src, unsigned char c, unsigned int iMaxBytes, unsigned char* pFill, unsigned int* piCount) const {
 	for (unsigned int a = 0; a < iMaxBytes && *src; a++, src++) {
@@ -187,10 +187,11 @@ int CString::StrCmp(const CString& s, unsigned long uLen) const {
 }
 
 bool CString::Equals(const CString& s, bool bCaseSensitive, unsigned long uLen) const {
-	if (bCaseSensitive)
+	if (bCaseSensitive) {
 		return (StrCmp(s, uLen) == 0);
-	else
+	} else {
 		return (CaseCmp(s, uLen) == 0);
+	}
 }
 
 bool CString::WildCmp(const CString& sWild, const CString& sString) {
@@ -296,8 +297,7 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 	const unsigned char *p = (const unsigned char*) data();
 	unsigned int iLength = length();
 	sRet.reserve(iLength *3);
-	unsigned int iMaxLen = (eFrom == EHTML) ? 20 : 0;
-	unsigned char pTmp[iMaxLen +1];
+	unsigned char pTmp[21];
 	unsigned int iCounted = 0;
 
 	for (unsigned int a = 0; a < iLength; a++, p = pStart + a) {
@@ -305,7 +305,7 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 
 		switch (eFrom) {
 			case EHTML:
-				if ((*p == '&') && (strnchr((unsigned char*) p, ';', iMaxLen, pTmp, &iCounted))) {
+				if ((*p == '&') && (strnchr((unsigned char*) p, ';', sizeof(pTmp) - 1, pTmp, &iCounted))) {
 					if ((iCounted >= 3) && (pTmp[1] == '#')) {	// do XML and HTML &#97; &#x3c
 						int base = 10;
 
@@ -496,7 +496,8 @@ unsigned int CString::Replace(CString& sStr, const CString& sReplace, const CStr
 	return uRet;
 }
 
-CString CString::Token(unsigned int uPos, bool bRest, const CString& sSep) const {
+CString CString::Token(unsigned int uPos, bool bRest, const CString& sSep, bool bAllowEmpty,
+                       const CString& sLeft, const CString& sRight, bool bTrimQuotes) const {
 	const char *sep_str = sSep.c_str();
 	size_t sep_len = sSep.length();
 	const char *str = c_str();
@@ -504,10 +505,22 @@ CString CString::Token(unsigned int uPos, bool bRest, const CString& sSep) const
 	size_t start_pos = 0;
 	size_t end_pos;
 
+	if (!bAllowEmpty) {
+		while (strncmp(&str[start_pos], sep_str, sep_len) == 0) {
+			start_pos += sep_len;
+		}
+	}
+
 	// First, find the start of our token
 	while (uPos != 0 && start_pos < str_len) {
-		if (strncmp(&str[start_pos], sep_str, sep_len) == 0) {
+		bool bFoundSep = false;
+
+		while (strncmp(&str[start_pos], sep_str, sep_len) == 0 && (!bFoundSep || !bAllowEmpty)) {
 			start_pos += sep_len;
+			bFoundSep = true;
+		}
+
+		if (bFoundSep) {
 			uPos--;
 		} else {
 			start_pos++;
@@ -582,7 +595,48 @@ unsigned int CString::URLSplit(MCString& msRet) const {
 	return msRet.size();
 }
 
-unsigned int CString::Split(const CString& sDelim, VCString& vsRet, bool bAllowEmpty, const CString& sLeft, const CString& sRight) const {
+unsigned int CString::OptionSplit(MCString& msRet, bool bUpperKeys) const {
+	CString sName;
+	CString sCopy(*this);
+	msRet.clear();
+
+	while (!sCopy.empty()) {
+		sName = sCopy.Token(0, false, "=", false, "\"", "\"", false).Trim_n();
+		sCopy = sCopy.Token(1, true, "=", false, "\"", "\"", false).TrimLeft_n();
+
+		if (sName.empty()) {
+			continue;
+		}
+
+		VCString vsNames;
+		sName.Split(" ", vsNames, false, "\"", "\"");
+
+		for (unsigned int a = 0; a < vsNames.size(); a++) {
+			CString sKeyName = vsNames[a];
+
+			if (bUpperKeys) {
+				sKeyName.MakeUpper();
+			}
+
+			if ((a +1) == vsNames.size()) {
+				msRet[sKeyName] = sCopy.Token(0, false, " ", false, "\"", "\"");
+				sCopy = sCopy.Token(1, true, " ", false, "\"", "\"", false);
+			} else {
+				msRet[sKeyName] = "";
+			}
+		}
+	}
+
+	return msRet.size();
+}
+
+unsigned int CString::QuoteSplit(VCString& vsRet) const {
+	vsRet.clear();
+	return Split(" ", vsRet, false, "\"", "\"", true);
+}
+
+unsigned int CString::Split(const CString& sDelim, VCString& vsRet, bool bAllowEmpty,
+		const CString& sLeft, const CString& sRight, bool bTrimQuotes, bool bTrimWhiteSpace) const {
 	vsRet.clear();
 
 	if (empty()) {
@@ -604,18 +658,30 @@ unsigned int CString::Split(const CString& sDelim, VCString& vsRet, bool bAllowE
 
 	while (*p) {
 		if (uLeftLen && uRightLen && !bInside && strncasecmp(p, sLeft.c_str(), uLeftLen) == 0) {
+			if (!bTrimQuotes) {
+				sTmp += sLeft;
+			}
+
 			p += uLeftLen;
 			bInside = true;
 			continue;
 		}
 
 		if (uLeftLen && uRightLen && bInside && strncasecmp(p, sRight.c_str(), uRightLen) == 0) {
+			if (!bTrimQuotes) {
+				sTmp += sRight;
+			}
+
 			p += uRightLen;
 			bInside = false;
 			continue;
 		}
 
 		if (uDelimLen && !bInside && strncasecmp(p, sDelim.c_str(), uDelimLen) == 0) {
+			if (bTrimWhiteSpace) {
+				sTmp.Trim();
+			}
+
 			vsRet.push_back(sTmp);
 			sTmp.clear();
 			p += uDelimLen;
@@ -640,32 +706,12 @@ unsigned int CString::Split(const CString& sDelim, VCString& vsRet, bool bAllowE
 	}
 
 	return vsRet.size();
-
-	/*vsRet.clear();
-	CString sTmp = *this;
-
-	while (sTmp.size()) {
-		CString sTok = sTmp.Token(0, false, sDelim);
-		CString sRest = sTmp.Token(1, true, sDelim);
-
-		if (bAllowEmpty || !sTok.empty()) {
-			vsRet.push_back(sTok);
-		}
-
-		if (bAllowEmpty && sRest.empty() && sTok.size() < sTmp.size()) {
-			vsRet.push_back("");
-		}
-
-		sTmp = sRest;
-	}
-
-	return vsRet.size();*/
 }
 
-unsigned int CString::Split(const CString& sDelim, SCString& ssRet, bool bAllowEmpty, const CString& sLeft, const CString& sRight) const {
+unsigned int CString::Split(const CString& sDelim, SCString& ssRet, bool bAllowEmpty, const CString& sLeft, const CString& sRight, bool bTrimQuotes, bool bTrimWhiteSpace) const {
 	VCString vsTokens;
 
-	Split(sDelim, vsTokens, bAllowEmpty, sLeft, sRight);
+	Split(sDelim, vsTokens, bAllowEmpty, sLeft, sRight, bTrimQuotes, bTrimWhiteSpace);
 
 	ssRet.clear();
 
@@ -735,13 +781,8 @@ bool CString::Base64Encode(CString& sRet, unsigned int uWrap) const {
 		return 0;
 	}
 
-	//p = output = (unsigned char *)malloc(toalloc);
 	p = output = new unsigned char [toalloc];
 
-	if (!p) {
-		return false;
-	}
-
 	while (i < len - mod) {
 		*p++ = b64table[input[i++] >> 2];
 		*p++ = b64table[((input[i - 1] << 4) | (input[i] >> 4)) & 0x3f];
@@ -781,11 +822,16 @@ bool CString::Base64Encode(CString& sRet, unsigned int uWrap) const {
 }
 
 unsigned long CString::Base64Decode(CString& sRet) const {
-	const char* in = c_str();
+	CString sTmp(*this);
+	// remove new lines
+	sTmp.Replace("\r", "");
+	sTmp.Replace("\n", "");
+
+	const char* in = sTmp.c_str();
 	char c, c1, *p;
 	unsigned long i;
-	unsigned long uLen = size();
-	char* out = (char*) malloc(size() +1);
+	unsigned long uLen = sTmp.size();
+	char* out = new char[uLen + 1];
 
 	for (i = 0, p = out; i < uLen; i++) {
 		c = (char)base64_table[(unsigned char)in[i++]];
@@ -812,7 +858,7 @@ unsigned long CString::Base64Decode(CString& sRet) const {
 	unsigned long uRet = p - out;
 	sRet.clear();
 	sRet.append(out, uRet);
-	free(out);
+	delete[] out;
 
 	return uRet;
 }
@@ -924,6 +970,7 @@ short CString::ToShort() const { return strtoul(this->c_str(), (char**) NULL, 10
 unsigned short CString::ToUShort() const { return strtoul(this->c_str(), (char**) NULL, 10); }
 unsigned int CString::ToUInt() const { return strtoul(this->c_str(), (char**) NULL, 10); }
 int CString::ToInt() const { return strtoul(this->c_str(), (char**) NULL, 10); }
+long CString::ToLong() const { return strtoul(this->c_str(), (char**) NULL, 10); }
 unsigned long CString::ToULong() const { return strtoul(c_str(), NULL, 10); }
 unsigned long long CString::ToULongLong() const { return strtoull(c_str(), NULL, 10); }
 long long CString::ToLongLong() const { return strtoll(c_str(), NULL, 10); }
@@ -936,25 +983,31 @@ bool CString::Trim(const CString& s) {
 }
 
 bool CString::TrimLeft(const CString& s) {
-	bool bRet = false;
+	size_type i = find_first_not_of(s);
 
-	while (length() && s.find(Left(1)) != CString::npos) {
-		LeftChomp();
-		bRet = true;
-	}
+	if (i == 0)
+		return false;
 
-	return bRet;
+	if (i != npos)
+		this->erase(0, i);
+	else
+		this->clear();
+
+	return true;
 }
 
 bool CString::TrimRight(const CString& s) {
-	bool bRet = false;
+	size_type i = find_last_not_of(s);
 
-	while (length() && s.find(Right(1)) != CString::npos) {
-		RightChomp();
-		bRet = true;
-	}
+	if (i + 1 == length())
+		return false;
 
-	return bRet;
+	if (i != npos)
+		this->erase(i + 1, npos);
+	else
+		this->clear();
+
+	return true;
 }
 
 CString CString::Trim_n(const CString& s) const {
