@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,8 +9,11 @@
 #ifndef _MAIN_H
 #define _MAIN_H
 
-// Keep the number in sync with configure.in (and also with configure)
-#define VERSION 0.062
+// The following defines are for #if comparison (preprocessor only likes ints)
+#define VERSION_MAJOR	0
+#define VERSION_MINOR	74
+// This one is for display purpose
+#define VERSION		(VERSION_MAJOR + VERSION_MINOR / 1000.0)
 
 // You can add -DVERSION_EXTRA="stuff" to your CXXFLAGS!
 #ifndef VERSION_EXTRA
@@ -30,44 +33,24 @@
 	if (macUSER) {											\
 		CGlobalModules& GMods = CZNC::Get().GetModules();	\
 		CModules& UMods = macUSER->GetModules();			\
-		GMods.SetUser(macUSER); GMods.SetClient(macCLIENT);	\
+		CUser* pOldGUser = GMods.GetUser();				\
+		CClient* pOldGClient = GMods.GetClient();			\
+		CClient* pOldUClient = UMods.GetClient();			\
+		GMods.SetUser(macUSER);						\
+		GMods.SetClient(macCLIENT);	\
 		UMods.SetClient(macCLIENT);							\
 		if (GMods.macFUNC || UMods.macFUNC) {				\
-			GMods.SetUser(NULL); GMods.SetClient(NULL);		\
-			UMods.SetClient(NULL);							\
+			GMods.SetUser(pOldGUser);				\
+			GMods.SetClient(pOldGClient);		\
+			UMods.SetClient(pOldUClient);							\
 			macEXITER;										\
 		}													\
-		GMods.SetUser(NULL); GMods.SetClient(NULL);			\
-		UMods.SetClient(NULL);								\
+		GMods.SetUser(pOldGUser);					\
+		GMods.SetClient(pOldGClient);			\
+		UMods.SetClient(pOldUClient);								\
 	}
 #else
 #define MODULECALL(macFUNC, macUSER, macCLIENT, macEXITER)
 #endif
 
-
-#ifndef CS_STRING
-#define CS_STRING CString
-#endif
-
-#ifndef _NO_CSOCKET_NS
-#define _NO_CSOCKET_NS
-#endif
-
-#ifdef _DEBUG
-#define __DEBUG__
-#endif
-
-#include <iostream>
-using std::cout;
-using std::cerr;
-using std::endl;
-
-#include <limits.h>
-#ifndef PATH_MAX
-#define PATH_MAX	1024
-#warning PATH_MAX is undefined
-#endif
-
-#include "ZNCString.h"
-
 #endif // !_MAIN_H
