@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -62,30 +62,20 @@ public:
 		if (!sMyArgs.empty())
 		{
 			m_sPassword = CBlowfish::MD5(sMyArgs);
-		}
-
-		return true;
-	}
-
-	virtual bool OnBoot()
-	{
-		if (m_sPassword.empty())
-		{
-			char *pTmp = CUtils::GetPass("Enter Encryption Key for away.so: ");
-
-			if (pTmp)
-				m_sPassword = CBlowfish::MD5(pTmp);
-
-			*pTmp = 0;
+		} else {
+			sMessage = "This module needs as an argument a keyphrase used for encryption";
+			return false;
 		}
 
 		if (!BootStrap())
 		{
+			sMessage = "Failed to decrypt your saved messages - "
+				"Did you give the right encryption key as an argument to this module?";
 			m_bBootError = true;
-			return(false);
+			return false;
 		}
 
-		return(true);
+		return true;
 	}
 
 	virtual void OnIRCConnected()
@@ -101,15 +91,17 @@ public:
 		CString sFile;
 		if (DecryptMessages(sFile))
 		{
-			CString sLine;
-			CString::size_type iPos = 0;
-			while (ReadLine(sFile, sLine, iPos))
-			{
+			VCString vsLines;
+			VCString::iterator it;
+
+			sFile.Split("\n", vsLines);
+
+			for (it = vsLines.begin(); it != vsLines.end(); it++) {
+				CString sLine(*it);
 				sLine.Trim();
 				AddMessage(sLine);
 			}
-		} else
-		{
+		} else {
 			m_sPassword = "";
 			CUtils::PrintError("[" + GetModName() + ".so] Failed to Decrypt Messages");
 			return(false);
@@ -132,8 +124,12 @@ public:
 			CString sPath = GetPath();
 			if (!sPath.empty())
 			{
-				WriteFile(sPath, sFile);
-				chmod(sPath.c_str(), 0600);
+				CFile File(sPath);
+				if (File.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
+					File.Chmod(0600);
+					File.Write(sFile);
+				}
+				File.Close();
 			}
 		}
 	}
@@ -189,7 +185,7 @@ public:
 				return;
 			} else
 			{
-				u_int iNum = atoi(sWhich.c_str());
+				u_int iNum = sWhich.ToUInt();
 				if (iNum >= m_vMessages.size())
 				{
 					PutModNotice("Illegal Message # Requested", "away");
@@ -235,7 +231,7 @@ public:
 					m_vMessages.erase(m_vMessages.begin() + a--);
 					continue;
 				}
-				time_t iTime = strtol(sTime.c_str(), NULL, 10);
+				time_t iTime = sTime.ToULong();
 				char szFormat[64];
 				struct tm t;
 				localtime_r(&iTime, &t);
@@ -379,12 +375,16 @@ private:
 		CString sFile;
 		sBuffer = "";
 
-		if ((sMessages.empty()) || (!ReadFile(sMessages, sFile)))
+		CFile File(sMessages);
+
+		if (sMessages.empty() || !File.Open() || !File.ReadFile(sFile))
 		{
 			 PutModule("Unable to find buffer");
 			 return(true); // gonna be successful here
 		}
 
+		File.Close();
+
 		if (!sFile.empty())
 		{
 			CBlowfish c(m_sPassword, BF_DECRYPT);
