@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -67,6 +67,11 @@ public:
 
 	void StartAwayNickTimer() {
 		RemTimer("AwayNickTimer");
+		if (FindTimer("BackNickTimer")) {
+			// Client disconnected before we got set back, so do nothing.
+			RemTimer("BackNickTimer");
+			return;
+		}
 		AddTimer(new CAwayNickTimer(*this));
 	}
 
@@ -76,17 +81,26 @@ public:
 		if (pIRCSock) {
 			CString sConfNick = m_pUser->GetNick();
 
-			if (pIRCSock->GetNick().Equals(GetAwayNick().Left(pIRCSock->GetNick().length()))) {
+			if (pIRCSock->GetNick().Equals(m_sAwayNick.Left(pIRCSock->GetNick().length()))) {
 				RemTimer("BackNickTimer");
 				AddTimer(new CBackNickTimer(*this));
 			}
 		}
 	}
 
-	virtual void OnIRCConnected() {
+	virtual EModRet OnIRCRegistration(CString& sPass, CString& sNick,
+			CString& sIdent, CString& sRealName) {
 		if (m_pUser && !m_pUser->IsUserAttached()) {
-			StartAwayNickTimer();
+			m_sAwayNick = m_sFormat;
+
+			// ExpandString doesn't know our nick yet, so do it by hand.
+			m_sAwayNick.Replace("%nick%", sNick);
+
+			// We don't limit this to NICKLEN, because we dont know
+			// NICKLEN yet.
+			sNick = m_sAwayNick = m_pUser->ExpandString(m_sAwayNick);
 		}
+		return CONTINUE;
 	}
 
 	virtual void OnIRCDisconnected() {
@@ -151,11 +165,13 @@ public:
 			uLen = pIRCSock->GetMaxNickLen();
 		}
 
-		return m_pUser->ExpandString(m_sFormat).Left(uLen);
+		m_sAwayNick = m_pUser->ExpandString(m_sFormat).Left(uLen);
+		return m_sAwayNick;
 	}
 
 private:
 	CString		m_sFormat;
+	CString		m_sAwayNick;
 };
 
 CAwayNickTimer::CAwayNickTimer(CAwayNickMod& Module)
