@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -116,16 +116,16 @@ typedef vector< PString > VPString;
 class CModPerl;
 static CModPerl *g_ModPerl = NULL;
 
-class CPerlSock : public Csock
+class CPerlSock : public CZNCSock
 {
 public:
-	CPerlSock() : Csock()
+	CPerlSock() : CZNCSock()
 	{
 		m_iParentFD = -1;
 		SetSockName(ZNCSOCK);
 	}
 	CPerlSock(const CS_STRING & sHost, u_short iPort, int iTimeout = 60)
-		: Csock(sHost, iPort, iTimeout)
+		: CZNCSock(sHost, iPort, iTimeout)
 	{
 		m_iParentFD = -1;
 		SetSockName(ZNCSOCK);
@@ -247,7 +247,7 @@ public:
 
 		CString sBuffer, sScript;
 		CFile cFile(sModule);
-		if (!cFile.Exists() || !cFile.Open(O_RDONLY))
+		if (!cFile.Exists() || !cFile.Open())
 			return false;
 
 		while (cFile.ReadLine(sBuffer))
@@ -282,7 +282,7 @@ public:
 				sTmp[a] = ' ';
 		}
 		PutModule(sTmp);
-		DEBUG_ONLY(cerr << sTmp << endl);
+		DEBUG(sTmp);
 	}
 
 	CSockManager * GetSockManager() { return(m_pManager); }
@@ -926,6 +926,7 @@ CModPerl::EModRet CModPerl::CallBack(const PString & sHookName, const VPString &
 		return(CONTINUE);
 
 	dSP;
+	ENTER;
 	SAVETMPS;
 
 	PUSHMARK(SP);
@@ -957,7 +958,7 @@ CModPerl::EModRet CModPerl::CallBack(const PString & sHookName, const VPString &
 
 	PUTBACK;
 
-	int iCount = call_pv(sFuncToCall.c_str(), G_EVAL|G_SCALAR);
+	int iCount = call_pv(sFuncToCall.c_str(), G_EVAL|G_SCALAR|G_KEEPERR);
 
 	SPAGAIN;
 	int iRet = CONTINUE;
@@ -969,6 +970,7 @@ CModPerl::EModRet CModPerl::CallBack(const PString & sHookName, const VPString &
 
 		if (eCBType == CB_TIMER)
 			iRet = HALT;
+		POPs;
 	} else
 	{
 		if (iCount == 1)
@@ -977,6 +979,7 @@ CModPerl::EModRet CModPerl::CallBack(const PString & sHookName, const VPString &
 
 	PUTBACK;
 	FREETMPS;
+	LEAVE;
 
 	return((CModPerl::EModRet)iRet);
 }
@@ -1039,10 +1042,10 @@ bool CModPerl::OnLoad(const CString & sArgs, CString & sMessage)
 
 	sv_2mortal((SV*)pZNCSpace);
 
-	newCONSTSUB(pZNCSpace, "CONTINUE", PString(CONTINUE).GetSV());
-	newCONSTSUB(pZNCSpace, "HALT", PString(HALT).GetSV());
-	newCONSTSUB(pZNCSpace, "HALTMODS", PString(HALTMODS).GetSV());
-	newCONSTSUB(pZNCSpace, "HALTCORE", PString(HALTCORE).GetSV());
+	newCONSTSUB(pZNCSpace, "CONTINUE", PString(CONTINUE).GetSV(false));
+	newCONSTSUB(pZNCSpace, "HALT", PString(HALT).GetSV(false));
+	newCONSTSUB(pZNCSpace, "HALTMODS", PString(HALTMODS).GetSV(false));
+	newCONSTSUB(pZNCSpace, "HALTCORE", PString(HALTCORE).GetSV(false));
 
 	return(true);
 }
@@ -1051,7 +1054,7 @@ void CModPerl::LoadPerlMod(const CString & sModule)
 {
 	if (!m_pUser)
 	{
-		DEBUG_ONLY(cerr << "LoadPerlMod: No User is set!!!" << endl);
+		DEBUG("LoadPerlMod: No User is set!!!");
 		return;
 	}
 
@@ -1082,7 +1085,7 @@ void CModPerl::UnloadPerlMod(const CString & sModule)
 	DestroyAllSocks(sModule);
 	if (!m_pUser)
 	{
-		DEBUG_ONLY(cerr << "UnloadPerlMod: No User is set!!!" << endl);
+		DEBUG("UnloadPerlMod: No User is set!!!");
 		return;
 	}
 	Eval("ZNC::COREUnloadMod('" + m_pUser->GetUserName() + "', '" + sModule + "');");
