@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -37,8 +37,11 @@ public:
 			m_sPass = sPass;
 			SetNV("Password", m_sPass);
 			PutModule("Password set");
+		} else if (sCmdName == "clear") {
+			m_sPass = "";
+			DelNV("Password");
 		} else {
-			PutModule("Commands: set <password>");
+			PutModule("Commands: set <password>, clear");
 		}
 	}
 
@@ -46,7 +49,8 @@ public:
 	{
 		if (!m_sPass.empty()
 				&& Nick.GetNick().Equals("NickServ")
-				&& sMessage.find("msg") != CString::npos
+				&& (sMessage.find("msg") != CString::npos
+				 || sMessage.find("authenticate") != CString::npos)
 				&& sMessage.AsUpper().find("IDENTIFY") != CString::npos
 				&& sMessage.find("help") == CString::npos) {
 			PutIRC("PRIVMSG NickServ :IDENTIFY " + m_sPass);
