@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -58,28 +58,16 @@ public:
 
 	virtual bool OnLoad(const CString& sArgs, CString& sMessage)
 	{
-		if (!sArgs.empty())
-		{
-			m_sPassword = CBlowfish::MD5(sArgs);
-			return(OnBoot());
-		}
-
-		return true;
-	}
-
-	virtual bool OnBoot()
-	{
-		if (m_sPassword.empty())
-		{
-			char *pTmp = CUtils::GetPass("Enter Encryption Key for " + GetModName() + ".so");
-
-			if (pTmp)
-				m_sPassword = CBlowfish::MD5(pTmp);
-
-			*pTmp = 0;
-		}
-
 		const vector<CChan *>& vChans = m_pUser->GetChans();
+
+		if (sArgs.empty())
+		{
+			sMessage = "This module needs as an argument a keyphrase used for encryption";
+			return false;
+		}
+
+		m_sPassword = CBlowfish::MD5(sArgs);
+
 		for (u_int a = 0; a < vChans.size(); a++)
 		{
 			if (!vChans[a]->KeepBuffer())
@@ -87,6 +75,8 @@ public:
 
 			if (!BootStrap(vChans[a]))
 			{
+				sMessage = "Failed to decrypt your saved messages - "
+					"Did you give the right encryption key as an argument to this module?";
 				m_bBootError = true;
 				return(false);
 			}
@@ -103,10 +93,13 @@ public:
 			if (!pChan->GetBuffer().empty())
 				return(true); // reloaded a module probably in this case, so just verify we can decrypt the file
 
-			CString sLine;
-			CString::size_type iPos = 0;
-			while (ReadLine(sFile, sLine, iPos))
-			{
+			VCString vsLines;
+			VCString::iterator it;
+
+			sFile.Split("\n", vsLines);
+
+			for (it = vsLines.begin(); it != vsLines.end(); it++) {
+				CString sLine(*it);
 				sLine.Trim();
 				pChan->AddBuffer(sLine);
 			}
@@ -127,19 +120,16 @@ public:
 			const vector<CChan *>& vChans = m_pUser->GetChans();
 			for (u_int a = 0; a < vChans.size(); a++)
 			{
-				if (!vChans[a]->KeepBuffer())
-					continue;
-
-				const vector<CString> & vBuffer = vChans[a]->GetBuffer();
-
-				if (vBuffer.empty())
-				{
-					if (!m_sPassword.empty())
-						BootStrap(vChans[a]);
+				CString sPath = GetPath(vChans[a]->GetName());
+				CFile File(sPath);
 
+				if (!vChans[a]->KeepBuffer()) {
+					File.Delete();
 					continue;
 				}
 
+				const vector<CString> & vBuffer = vChans[a]->GetBuffer();
+
 				CString sFile = CRYPT_VERIFICATION_TOKEN;
 
 				for (u_int b = 0; b < vBuffer.size(); b++)
@@ -147,11 +137,13 @@ public:
 
 				CBlowfish c(m_sPassword, BF_ENCRYPT);
 				sFile = c.Crypt(sFile);
-				CString sPath = GetPath(vChans[a]->GetName());
 				if (!sPath.empty())
 				{
-					WriteFile(sPath, sFile);
-					chmod(sPath.c_str(), 0600);
+					if (File.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
+						File.Chmod(0600);
+						File.Write(sFile);
+					}
+					File.Close();
 				}
 			}
 		}
@@ -172,10 +164,13 @@ public:
 			CString sFile;
 			if (DecryptChannel(sArgs, sFile))
 			{
-				CString sLine;
-				CString::size_type iPos = 0;
-				while (ReadLine(sFile, sLine, iPos))
-				{
+				VCString vsLines;
+				VCString::iterator it;
+
+				sFile.Split("\n", vsLines);
+
+				for (it = vsLines.begin(); it != vsLines.end(); it++) {
+					CString sLine(*it);
 					sLine.Trim();
 					PutModule("[" + sLine + "]");
 				}
@@ -200,10 +195,13 @@ public:
 		PutUser(":***!znc@znc.in PRIVMSG " + sChan + " :Buffer Playback...");
 		if (DecryptChannel(sChan, sFile))
 		{
-			CString sLine;
-			CString::size_type iPos = 0;
-			while (ReadLine(sFile, sLine, iPos))
-			{
+			VCString vsLines;
+			VCString::iterator it;
+
+			sFile.Split("\n", vsLines);
+
+			for (it = vsLines.begin(); it != vsLines.end(); it++) {
+				CString sLine(*it);
 				sLine.Trim();
 				PutUser(sLine);
 			}
@@ -213,7 +211,7 @@ public:
 
 	CString GetPath(const CString & sChannel)
 	{
-		CString sBuffer = m_pUser->GetUserName() + Lower(sChannel);
+		CString sBuffer = m_pUser->GetUserName() + sChannel.AsLower();
 		CString sRet = GetSavePath();
 		sRet += "/" + CBlowfish::MD5(sBuffer, true);
 		return(sRet);
@@ -227,17 +225,12 @@ public:
 
 	virtual void OnRawMode(const CNick& cOpNick, CChan& cChannel, const CString& sModes, const CString& sArgs)
 	{
-		if (!cChannel.KeepBuffer())
-			return;
-
-		((CChan &)cChannel).AddBuffer(SpoofChanMsg(cChannel.GetName(), cOpNick.GetNickMask() + " MODE " + sModes + " " + sArgs));
+		cChannel.AddBuffer(SpoofChanMsg(cChannel.GetName(), cOpNick.GetNickMask() + " MODE " + sModes + " " + sArgs));
 	}
 	virtual void OnQuit(const CNick& cNick, const CString& sMessage, const vector<CChan*>& vChans)
 	{
 		for (u_int a = 0; a < vChans.size(); a++)
 		{
-			if (!vChans[a]->KeepBuffer())
-				continue;
 			vChans[a]->AddBuffer(SpoofChanMsg(vChans[a]->GetName(), cNick.GetNickMask() + " QUIT " + sMessage));
 		}
 		if (cNick.GetNick().Equals(m_pUser->GetNick()))
@@ -248,16 +241,12 @@ public:
 	{
 		for (u_int a = 0; a < vChans.size(); a++)
 		{
-			if (!vChans[a]->KeepBuffer())
-				continue;
 			vChans[a]->AddBuffer(SpoofChanMsg(vChans[a]->GetName(), cNick.GetNickMask() + " NICK " + sNewNick));
 		}
 	}
 	virtual void OnKick(const CNick& cNick, const CString& sOpNick, CChan& cChannel, const CString& sMessage)
 	{
-		if (!cChannel.KeepBuffer())
-			return;
-		((CChan &)cChannel).AddBuffer(SpoofChanMsg(cChannel.GetName(), sOpNick + " KICK " + cNick.GetNickMask() + " " + sMessage));
+		cChannel.AddBuffer(SpoofChanMsg(cChannel.GetName(), sOpNick + " KICK " + cNick.GetNickMask() + " " + sMessage));
 	}
 	virtual void OnJoin(const CNick& cNick, CChan& cChannel)
 	{
@@ -267,15 +256,11 @@ public:
 			if (!cChannel.GetBuffer().empty())
 				Replay(cChannel.GetName());
 		}
-		if (!cChannel.KeepBuffer())
-			return;
-		((CChan &)cChannel).AddBuffer(SpoofChanMsg(cChannel.GetName(), cNick.GetNickMask() + " JOIN"));
+		cChannel.AddBuffer(SpoofChanMsg(cChannel.GetName(), cNick.GetNickMask() + " JOIN"));
 	}
 	virtual void OnPart(const CNick& cNick, CChan& cChannel)
 	{
-		if (!cChannel.KeepBuffer())
-			return;
-		((CChan &)cChannel).AddBuffer(SpoofChanMsg(cChannel.GetName(), cNick.GetNickMask() + " PART"));
+		cChannel.AddBuffer(SpoofChanMsg(cChannel.GetName(), cNick.GetNickMask() + " PART"));
 		if (cNick.GetNick().Equals(m_pUser->GetNick()))
 			SaveBufferToDisk(); // need to force a save here to see this!
 	}
@@ -289,15 +274,19 @@ private:
 		CString sFile;
 		sBuffer = "";
 
-		if ((sChannel.empty()) || (!ReadFile(sChannel, sFile)))
+		CFile File(sChannel);
+
+		if (sChannel.empty() || !File.Open() || !File.ReadFile(sFile))
 			 return(true); // gonna be successful here
 
+		File.Close();
+
 		if (!sFile.empty())
 		{
 			CBlowfish c(m_sPassword, BF_DECRYPT);
 			sBuffer = c.Crypt(sFile);
 
-			if (sBuffer.substr(0, strlen(CRYPT_VERIFICATION_TOKEN)) != CRYPT_VERIFICATION_TOKEN)
+			if (sBuffer.Left(strlen(CRYPT_VERIFICATION_TOKEN)) != CRYPT_VERIFICATION_TOKEN)
 			{
 				// failed to decode :(
 				PutModule("Unable to decode Encrypted file [" + sChannel + "]");
