@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -26,8 +26,8 @@ public:
 	virtual ~CWebAdminAuth() {}
 
 	void SetWebAdminSock(CWebAdminSock* pWebAdminSock) { m_pWebAdminSock = pWebAdminSock; }
-	void AcceptLogin(CUser& User);
-	void RefuseLogin(const CString& sReason);
+	void AcceptedLogin(CUser& User);
+	void RefusedLogin(const CString& sReason);
 private:
 protected:
 	CWebAdminSock*	m_pWebAdminSock;
@@ -95,7 +95,7 @@ public:
 	// !Setters
 
 	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort);
-	bool IsAdmin(bool bAllowUserAdmin = true) const { return m_bAdmin; }
+	bool IsAdmin() const { return m_bAdmin; }
 
 	CWebAdminMod* GetModule() const { return (CWebAdminMod*) m_pModule; }
 
@@ -122,29 +122,36 @@ public:
 		bool bIPv6 = false;
 		unsigned short uPort = 8080;
 		CString sArgs(sArgStr);
-		CString sOpt;
 		CString sPort;
 		CString sListenHost;
 
-		if (sArgs.Left(1) == "-") {
-			sOpt = sArgs.Token(0);
+		m_bShareIRCPorts = true;
+
+		while (sArgs.Left(1) == "-") {
+			CString sOpt = sArgs.Token(0);
 			sArgs = sArgs.Token(1, true);
 
 			if (sOpt.Equals("-IPV6")) {
 				bIPv6 = true;
 			} else if (sOpt.Equals("-IPV4")) {
 				bIPv6 = false;
+			} else if (sOpt.Equals("-noircport")) {
+				m_bShareIRCPorts = false;
 			} else {
-				sMessage = "Unknown option [" + sOpt + "] valid options are -ipv4 or -ipv6";
+				sMessage = "Unknown option [" + sOpt + "] valid options are -ipv4, -ipv6 or -noircport";
 				return false;
 			}
 		}
 
+		// No arguments left: Only port sharing
+		if (sArgs.empty() && m_bShareIRCPorts)
+			return true;
+
 		if (sArgs.find(" ") != CString::npos) {
 			sListenHost = sArgs.Token(0);
-			sPort = sArgs.Token(1);
+			sPort = sArgs.Token(1, true);
 		} else {
-			sPort = sArgs.Token(0);
+			sPort = sArgs;
 		}
 
 		if (sPort.Left(1) == "+") {
@@ -160,6 +167,11 @@ public:
 			uPort = sPort.ToUShort();
 		}
 
+		return OpenListener(sMessage, uPort, sListenHost, bSSL, bIPv6);
+	}
+
+	bool OpenListener(CString &sMessage, u_short uPort, const CString& sListenHost,
+			bool bSSL = false, bool bIPv6 = false) {
 		CWebAdminSock* pListenSock = new CWebAdminSock(this);
 #ifdef HAVE_LIBSSL
 		if (bSSL) {
@@ -207,8 +219,31 @@ public:
 		return ++it->second;
 	}
 
+	virtual EModRet OnUnknownUserRaw(CClient* pClient, CString& sLine) {
+		if (!m_bShareIRCPorts)
+			return CONTINUE;
+
+		// If this is a HTTP request, we should handle it
+		if (sLine.WildCmp("GET * HTTP/1.?")
+				|| sLine.WildCmp("POST * HTTP/1.?")) {
+			CWebAdminSock* pSock = new CWebAdminSock(this);
+			CZNC::Get().GetManager().SwapSockByAddr(pSock, pClient);
+			// And don't forget to give it some sane name / timeout
+			pSock->SetSockName("WebAdmin::Client");
+			pSock->SetTimeout(120);
+
+			// TODO can we somehow get rid of this?
+			pSock->ReadLine(sLine);
+			pSock->PushBuff("", 0, true);
+
+			return HALT;
+		}
+		return CONTINUE;
+	}
+
 private:
-	CString						m_sSkinName;
+	CString				m_sSkinName;
+	bool				m_bShareIRCPorts;
 	map<CString, unsigned int>	m_suSwitchCounters;
 };
 
@@ -217,10 +252,14 @@ CString CWebAdminSock::GetAvailSkinsDir() {
 }
 
 CString CWebAdminSock::GetSkinDir() {
-	CString sSkinDir = GetAvailSkinsDir() + GetModule()->GetSkinName() + "/";
+	CString sAvailSkins = GetAvailSkinsDir();
+	CString sSkinDir = GetModule()->GetSkinName() + "/";
+	CString sDir = CDir::CheckPathPrefix(sAvailSkins, sSkinDir, "/");
 
-	if (CFile::IsDir(sSkinDir)) {
-		return sSkinDir;
+	// Via CheckPrefix() we check if someone tries to use e.g. a skin name
+	// with embed .. or such evilness.
+	if (!sDir.empty() && CFile::IsDir(sDir)) {
+		return sDir + "/";
 	}
 
 	return m_pModule->GetModDataDir() + "/skins/default/";
@@ -228,8 +267,6 @@ CString CWebAdminSock::GetSkinDir() {
 
 void CWebAdminSock::PrintPage(CString& sPageRet, const CString& sTmplName) {
 	sPageRet.clear();
-	// @todo possibly standardize the location of meta files such as these skins
-	// @todo give an option for changing the current skin from 'default'
 	CString sTmpl;
 
 	if (IsAdmin()) {
@@ -322,9 +359,6 @@ CWebAdminSock::~CWebAdminSock() {
 		CWebAdminAuth* pAuth = (CWebAdminAuth*) &(*m_spAuth);
 		pAuth->SetWebAdminSock(NULL);
 	}
-
-	CZNC::Get().AddBytesRead(GetBytesRead());
-	CZNC::Get().AddBytesWritten(GetBytesWritten());
 }
 
 bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
@@ -332,9 +366,13 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 		return false;
 	}
 
+	const CString& sSkin = GetModule()->GetSkinName();
+	CString sTmp = sURI;
+
 	m_Template["SessionUser"] = GetUser();
 	m_Template["SessionIP"] = GetRemoteIP();
 	m_Template["Tag"] = CZNC::GetTag();
+	m_Template["Skin"] = sSkin;
 
 	if (IsAdmin()) {
 		m_Template["IsAdmin"] = "true";
@@ -349,17 +387,12 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 		m_Template["Title"] = "Main Page";
 		m_Template["Action"] = "home";
 		PrintPage(sPageRet, "Main.tmpl");
-	} else if (sURI.Equals("/css/", false, 5)) {
-		SetDocRoot(GetSkinDir() + "/css");
-		PrintFile(sURI.substr(5), "text/css");
+	} else if (sTmp.TrimPrefix("/" + sSkin + "/")) {
+		SetDocRoot(GetSkinDir() + "/data");
+		PrintFile(sTmp);
 		return false;
-	} else if (sURI.Equals("/img/", false, 5)) {
-		SetDocRoot(GetSkinDir() + "/img");
-		PrintFile(sURI.substr(5));
-		return false;
-	} else if (sURI.Equals("/js/", false, 4)) {
-		SetDocRoot(GetSkinDir() + "/js");
-		PrintFile(sURI.substr(4), "application/x-javascript");
+	} else if (sURI == "/favicon.ico") {
+		PrintFile("/data/favicon.ico", "image/x-icon");
 		return false;
 	} else if (sURI == "/home") {
 		m_Template["Title"] = "Main Page";
@@ -454,9 +487,6 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 		} else {
 			GetErrorPage(sPageRet, "No such username");
 		}
-	} else if (sURI == "/favicon.ico") {
-		PrintFile("/img/favicon.ico", "image/x-icon");
-		return false;
 	} else if (sURI == "/listusers") {
 		if (!IsAdmin()) {
 			return false;
@@ -582,7 +612,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 	//sArg = GetParam(""); if (!sArg.empty()) { CZNC::Get().Set(sArg); }
 
 	VCString vsArgs;
-	GetParam("motd").Split("\n", vsArgs);
+	GetRawParam("motd").Split("\n", vsArgs);
 	CZNC::Get().ClearMotd();
 
 	unsigned int a = 0;
@@ -590,7 +620,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 		CZNC::Get().AddMotd(vsArgs[a].TrimRight_n());
 	}
 
-	GetParam("vhosts").Split("\n", vsArgs);
+	GetRawParam("vhosts").Split("\n", vsArgs);
 	CZNC::Get().ClearVHosts();
 
 	for (a = 0; a < vsArgs.size(); a++) {
@@ -609,12 +639,17 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 		if (!sModName.empty()) {
 			CString sArgs = GetParam("modargs_" + sModName);
 
-			if (!CZNC::Get().GetModules().FindModule(sModName)) {
+			CModule *pMod = CZNC::Get().GetModules().FindModule(sModName);
+			if (!pMod) {
 				if (!CZNC::Get().GetModules().LoadModule(sModName, sArgs, NULL, sModRet)) {
-					DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sModRet << "]" << endl);
+					DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
+				}
+			} else if (pMod->GetArgs() != sArgs) {
+				if (!CZNC::Get().GetModules().ReloadModule(sModName, sArgs, NULL, sModRet)) {
+					DEBUG("Unable to reload module [" << sModName << "] [" << sModRet << "]");
 				}
 			} else {
-				DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] because it is already loaded" << endl);
+				DEBUG("Unable to load module [" << sModName << "] because it is already loaded");
 			}
 		}
 	}
@@ -1002,19 +1037,19 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 
 	if (!sArg.empty()) {
 		CString sSalt = CUtils::GetSalt();
-		CString sSaltedPass = sArg + sSalt;
-		pNewUser->SetPass(sSaltedPass.MD5(), true, sSalt);
+		CString sHash = CUtils::SaltedHash(sArg, sSalt);
+		pNewUser->SetPass(sHash, true, sSalt);
 	}
 
 	VCString vsArgs;
-	GetParam("servers").Split("\n", vsArgs);
+	GetRawParam("servers").Split("\n", vsArgs);
 	unsigned int a = 0;
 
 	for (a = 0; a < vsArgs.size(); a++) {
 		pNewUser->AddServer(vsArgs[a].Trim_n());
 	}
 
-	GetParam("allowedips").Split("\n", vsArgs);
+	GetRawParam("allowedips").Split("\n", vsArgs);
 	if (vsArgs.size()) {
 		for (a = 0; a < vsArgs.size(); a++) {
 			pNewUser->AddAllowedHost(vsArgs[a].Trim_n());
@@ -1027,7 +1062,7 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 		pNewUser->AddAllowedHost(GetParam("ownip"));
 	}
 
-	GetParam("ctcpreplies").Split("\n", vsArgs);
+	GetRawParam("ctcpreplies").Split("\n", vsArgs);
 	for (a = 0; a < vsArgs.size(); a++) {
 		CString sReply = vsArgs[a].TrimRight_n("\r");
 		pNewUser->AddCTCPReply(sReply.Token(0).Trim_n(), sReply.Token(1, true).Trim_n());
@@ -1045,10 +1080,10 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 
 				try {
 					if (!pNewUser->GetModules().LoadModule(sModName, sArgs, pNewUser, sModRet, (pUser != NULL))) {
-						DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sModRet << "]" << endl);
+						DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
 					}
 				} catch (...) {
-					DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sArgs << "]" << endl);
+					DEBUG("Unable to load module [" << sModName << "] [" << sArgs << "]");
 				}
 			}
 		}
@@ -1062,10 +1097,10 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 
 			try {
 				if (!pNewUser->GetModules().LoadModule(sModName, sArgs, pNewUser, sModRet, (pUser != NULL))) {
-					DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sModRet << "]" << endl);
+					DEBUG("Unable to load module [" << sModName << "] [" << sModRet << "]");
 				}
 			} catch (...) {
-				DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "]" << endl);
+				DEBUG("Unable to load module [" << sModName << "]");
 			}
 		}
 	}
@@ -1124,11 +1159,11 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 
 CWebAdminAuth::CWebAdminAuth(CWebAdminSock* pWebAdminSock, const CString& sUsername,
 		const CString& sPassword)
-		: CAuthBase(sUsername, sPassword, pWebAdminSock->GetRemoteIP()) {
+		: CAuthBase(sUsername, sPassword, pWebAdminSock) {
 	m_pWebAdminSock = pWebAdminSock;
 }
 
-void CWebAdminAuth::AcceptLogin(CUser& User) {
+void CWebAdminAuth::AcceptedLogin(CUser& User) {
 	if (m_pWebAdminSock) {
 		m_pWebAdminSock->SetSessionUser(&User);
 		m_pWebAdminSock->SetLoggedIn(true);
@@ -1136,7 +1171,7 @@ void CWebAdminAuth::AcceptLogin(CUser& User) {
 	}
 }
 
-void CWebAdminAuth::RefuseLogin(const CString& sReason) {
+void CWebAdminAuth::RefusedLogin(const CString& sReason) {
 	if (m_pWebAdminSock) {
 		m_pWebAdminSock->SetLoggedIn(false);
 		m_pWebAdminSock->UnPauseRead();
