@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2009  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -23,14 +23,20 @@ namespace
 		CUser	*m_pUser;
 		CChan	*m_pChan;
 	};
-};
+}
 
 CZNC::CZNC() {
+	if (!InitCsocket()) {
+		CUtils::PrintError("Failed to initialize Csocket!");
+		exit(-1);
+	}
+
 #ifdef _MODULES
 	m_pModules = new CGlobalModules();
 #endif
 	m_pISpoofLockFile = NULL;
 	m_uiConnectDelay = 30;
+	m_uiAnonIPLimit = 10;
 	SetISpoofFormat(""); // Set ISpoofFormat to default
 	m_uBytesRead = 0;
 	m_uBytesWritten = 0;
@@ -68,6 +74,7 @@ CZNC::~CZNC() {
 	delete m_pModules;
 #endif
 
+	ShutdownCsocket();
 	DeletePidFile();
 }
 
@@ -140,12 +147,12 @@ bool CZNC::ConnectUser(CUser *pUser) {
 		return false;
 
 	if (!WriteISpoof(pUser)) {
-		DEBUG_ONLY(cout << "ISpoof could not be written" << endl);
+		DEBUG("ISpoof could not be written");
 		pUser->PutStatus("ISpoof could not be written, retrying...");
 		return true;
 	}
 
-	DEBUG_ONLY(cout << "User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << ":" << pServer->GetPort() << "] ..." << endl);
+	DEBUG("User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << ":" << pServer->GetPort() << "] ...");
 	pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + ":" + CString(pServer->GetPort()) + "] ...");
 
 	pIRCSock = new CIRCSock(pUser);
@@ -187,7 +194,9 @@ bool CZNC::HandleUserDeletion()
 #endif
 		m_msUsers.erase(pUser->GetUserName());
 
-		CIRCSock* pIRCSock = pUser->GetIRCSock();
+		// Don't use pUser->GetIRCSock(), as that only returns something if the
+		// CIRCSock is already connected, not when it's still connecting!
+		CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName("IRC::" + pUser->GetUserName());
 
 		if (pIRCSock) {
 			m_Manager.DelSockByAddr(pIRCSock);
@@ -207,7 +216,7 @@ bool CZNC::HandleUserDeletion()
 	return true;
 }
 
-int CZNC::Loop() {
+void CZNC::Loop() {
 	while (true) {
 		CString sError;
 
@@ -232,8 +241,6 @@ int CZNC::Loop() {
 		// 500 msec to 600 sec
 		m_Manager.DynamicSelectLoop(500 * 1000, 600 * 1000 * 1000);
 	}
-
-	return 0;
 }
 
 bool CZNC::WriteISpoof(CUser* pUser) {
@@ -241,28 +248,26 @@ bool CZNC::WriteISpoof(CUser* pUser) {
 		return false;
 
 	if (!m_sISpoofFile.empty()) {
-		m_pISpoofLockFile = new CLockFile;
-		if (!m_pISpoofLockFile->TryExLock(m_sISpoofFile, true)) {
+		m_pISpoofLockFile = new CFile;
+		if (!m_pISpoofLockFile->TryExLock(m_sISpoofFile, O_RDWR | O_CREAT)) {
 			delete m_pISpoofLockFile;
 			m_pISpoofLockFile = NULL;
 			return false;
 		}
 
-		CFile File(m_pISpoofLockFile->GetFD(), m_pISpoofLockFile->GetFileName());
-
 		char buf[1024];
 		memset((char*) buf, 0, 1024);
-		File.Read(buf, 1023);
+		m_pISpoofLockFile->Read(buf, 1023);
 		m_sOrigISpoof = buf;
 
-		if (!File.Seek(0) || !File.Truncate()) {
+		if (!m_pISpoofLockFile->Seek(0) || !m_pISpoofLockFile->Truncate()) {
 			delete m_pISpoofLockFile;
 			m_pISpoofLockFile = NULL;
 			return false;
 		}
 
 		CString sData = m_sISpoofFormat.Token(0, false, "%") + pUser->GetIdent() + m_sISpoofFormat.Token(1, true, "%");
-		File.Write(sData + "\n");
+		m_pISpoofLockFile->Write(sData + "\n");
 	}
 	return true;
 }
@@ -272,10 +277,8 @@ void CZNC::ReleaseISpoof() {
 		return;
 
 	if (!m_sISpoofFile.empty()) {
-		CFile File(m_pISpoofLockFile->GetFD(), m_pISpoofLockFile->GetFileName());
-
-		if (File.Seek(0) && File.Truncate()) {
-			File.Write(m_sOrigISpoof);
+		if (m_pISpoofLockFile->Seek(0) && m_pISpoofLockFile->Truncate()) {
+			m_pISpoofLockFile->Write(m_sOrigISpoof);
 		}
 
 		m_sOrigISpoof = "";
@@ -331,7 +334,7 @@ bool CZNC::DeletePidFile() {
 	return false;
 }
 
-bool CZNC::WritePemFile(bool bEncPem) {
+bool CZNC::WritePemFile() {
 #ifndef HAVE_LIBSSL
 	CUtils::PrintError("ZNC was not compiled with ssl support.");
 	return false;
@@ -359,7 +362,7 @@ bool CZNC::WritePemFile(bool bEncPem) {
 		return false;
 	}
 
-	CUtils::GenerateCert(f, bEncPem, sHost);
+	CUtils::GenerateCert(f, sHost);
 	fclose(f);
 
 	CUtils::PrintStatus(true);
@@ -397,18 +400,23 @@ bool CZNC::IsHostAllowed(const CString& sHostMask) const {
 	return false;
 }
 
-void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
-	char buf[PATH_MAX];
-	char *home;
+bool CZNC::AllowConnectionFrom(const CString& sIP) const {
+	if (m_uiAnonIPLimit == 0)
+		return true;
+	if (GetManager().GetAnonConnectionCount(sIP) >= m_uiAnonIPLimit)
+		return false;
+	return true;
+}
 
-	if (getcwd(buf, PATH_MAX) == NULL) {
-		CUtils::PrintError("getcwd() failed, can't read my current dir");
-		exit(-1);
-	}
+void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
+	char *home;
 
 	// If the bin was not ran from the current directory, we need to add that dir onto our cwd
 	CString::size_type uPos = sArgvPath.rfind('/');
-	m_sCurPath = (uPos == CString::npos) ? CString(buf) : CDir::ChangeDir(buf, sArgvPath.substr(0, uPos), "");
+	if (uPos == CString::npos)
+		m_sCurPath = "./";
+	else
+		m_sCurPath = CDir::ChangeDir("./", sArgvPath.Left(uPos), "");
 
 	// Try to set the user's home dir, default to binpath on failure
 	home = getenv("HOME");
@@ -435,11 +443,34 @@ void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 	} else {
 		m_sZNCPath = sDataDir;
 	}
+}
 
-	// Other dirs that we use
-	m_sConfPath = m_sZNCPath + "/configs";
-	m_sModPath = m_sZNCPath + "/modules";
-	m_sUserPath = m_sZNCPath + "/users";
+CString CZNC::GetConfPath() const {
+	CString sConfPath = m_sZNCPath + "/configs";
+	if (!CFile::Exists(sConfPath)) {
+		CDir::MakeDir(sConfPath);
+	}
+
+	return sConfPath;
+}
+
+CString CZNC::GetUserPath() const {
+	CString sUserPath = m_sZNCPath + "/users";
+	if (!CFile::Exists(sUserPath)) {
+		CDir::MakeDir(sUserPath);
+	}
+
+	return sUserPath;
+}
+
+CString CZNC::GetModPath() const {
+	CString sModPath = m_sZNCPath + "/modules";
+
+	if (!CFile::Exists(sModPath)) {
+		CDir::MakeDir(sModPath);
+	}
+
+	return sModPath;
 }
 
 
@@ -461,74 +492,79 @@ CString CZNC::ExpandConfigPath(const CString& sConfigFile) {
 	return sRetPath;
 }
 
-bool CZNC::BackupConfig() const {
-	CString sBackup = GetConfigFile() + "-backup";
-
-	// Create a new backup overwriting an old one we might have
-	if (CFile::Copy(m_sConfigFile, sBackup, true))
-		return true;
-
-	// Don't abort if no config file exists
-	if (!CFile::Exists(m_sConfigFile))
-		// No backup because we got nothing to backup
-		return true;
-
-	return false;
-}
-
 bool CZNC::WriteConfig() {
-	CFile File(GetConfigFile());
-
-	if (!BackupConfig()) {
+	if (GetConfigFile().empty()) {
 		return false;
 	}
 
-	if (m_sConfigFile.empty() || !File.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
+	// Close the old handle to the config file, we are replacing that file.
+	m_LockFile.Close();
+
+	// We first write to a temporary file and then move it to the right place
+	m_LockFile.SetFileName(GetConfigFile() + "~");
+
+	if (!m_LockFile.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
 		return false;
 	}
 
+	// We have to "transfer" our lock on the config to the new file.
+	// The old file (= inode) is going away and thus a lock on it would be
+	// useless. These lock should always succeed (races, anyone?).
+	if (!m_LockFile.TryExLock()) {
+		return false;
+	}
+
+	m_LockFile.Write("AnonIPLimit  = " + CString(m_uiAnonIPLimit) + "\n");
+
 	for (size_t l = 0; l < m_vpListeners.size(); l++) {
 		CListener* pListener = m_vpListeners[l];
 		CString sHostPortion = pListener->GetBindHost();
 
 		if (!sHostPortion.empty()) {
-			sHostPortion += " ";
+			sHostPortion = sHostPortion.FirstLine() + " ";
 		}
 
 		CString s6 = (pListener->IsIPV6()) ? "6" : " ";
 
-		File.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\n");
+		m_LockFile.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\n");
 	}
 
-	File.Write("ConnectDelay = " + CString(m_uiConnectDelay) + "\n");
+	m_LockFile.Write("ConnectDelay = " + CString(m_uiConnectDelay) + "\n");
 
 	if (!m_sISpoofFile.empty()) {
-		File.Write("ISpoofFile   = " + m_sISpoofFile + "\n");
-		if (!m_sISpoofFormat.empty()) { File.Write("ISpoofFormat = " + m_sISpoofFormat + "\n"); }
+		m_LockFile.Write("ISpoofFile   = " + m_sISpoofFile.FirstLine() + "\n");
+		if (!m_sISpoofFormat.empty()) {
+			m_LockFile.Write("ISpoofFormat = " + m_sISpoofFormat.FirstLine() + "\n");
+		}
 	}
 
-	if (!m_sPidFile.empty()) { File.Write("PidFile      = " + m_sPidFile + "\n"); }
-	if (!m_sStatusPrefix.empty()) { File.Write("StatusPrefix = " + m_sStatusPrefix + "\n"); }
+	if (!m_sPidFile.empty()) {
+		m_LockFile.Write("PidFile      = " + m_sPidFile.FirstLine() + "\n");
+	}
+	if (!m_sStatusPrefix.empty()) {
+		m_LockFile.Write("StatusPrefix = " + m_sStatusPrefix.FirstLine() + "\n");
+	}
 
 	for (unsigned int m = 0; m < m_vsMotd.size(); m++) {
-		File.Write("Motd         = " + m_vsMotd[m] + "\n");
+		m_LockFile.Write("Motd         = " + m_vsMotd[m].FirstLine() + "\n");
 	}
 
 	for (unsigned int v = 0; v < m_vsVHosts.size(); v++) {
-		File.Write("VHost        = " + m_vsVHosts[v] + "\n");
+		m_LockFile.Write("VHost        = " + m_vsVHosts[v].FirstLine() + "\n");
 	}
 
 #ifdef _MODULES
 	CGlobalModules& Mods = GetModules();
 
 	for (unsigned int a = 0; a < Mods.size(); a++) {
+		CString sName = Mods[a]->GetModName();
 		CString sArgs = Mods[a]->GetArgs();
 
 		if (!sArgs.empty()) {
-			sArgs = " " + sArgs;
+			sArgs = " " + sArgs.FirstLine();
 		}
 
-		File.Write("LoadModule   = " + Mods[a]->GetModName() + sArgs + "\n");
+		m_LockFile.Write("LoadModule   = " + sName.FirstLine() + sArgs + "\n");
 	}
 #endif
 
@@ -536,27 +572,35 @@ bool CZNC::WriteConfig() {
 		CString sErr;
 
 		if (!it->second->IsValid(sErr)) {
-			DEBUG_ONLY(cerr << "** Error writing config for user [" << it->first << "] [" << sErr << "]" << endl);
+			DEBUG("** Error writing config for user [" << it->first << "] [" << sErr << "]");
 			continue;
 		}
 
-		File.Write("\n");
+		m_LockFile.Write("\n");
 
-		if (!it->second->WriteConfig(File)) {
-			DEBUG_ONLY(cerr << "** Error writing config for user [" << it->first << "]" << endl);
+		if (!it->second->WriteConfig(m_LockFile)) {
+			DEBUG("** Error writing config for user [" << it->first << "]");
 		}
 	}
 
-	File.Close();
+	// If Sync() fails... well, let's hope nothing important breaks..
+	m_LockFile.Sync();
+
+	// We wrote to a temporary name, move it to the right place
+	if (!m_LockFile.Move(GetConfigFile(), true))
+		return false;
+
+	// Everything went fine, just need to update the saved path.
+	m_LockFile.SetFileName(GetConfigFile());
 
 	return true;
 }
 
-bool CZNC::WriteNewConfig(CString& sConfigFile) {
+bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 	CString sAnswer, sUser;
 	VCString vsLines;
 
-	sConfigFile = ExpandConfigPath((sConfigFile.empty()) ? "znc.conf" : sConfigFile);
+	m_sConfigFile = ExpandConfigPath(sConfigFile);
 	CUtils::PrintMessage("Building new config");
 
 	CUtils::PrintMessage("");
@@ -633,12 +677,12 @@ bool CZNC::WriteNewConfig(CString& sConfigFile) {
 				const CModInfo& Info = *it;
 				CString sName = Info.GetName();
 
-				if (sName.Right(3).Equals(".so")) {
-					sName.RightChomp(3);
-				}
-
-				if (CUtils::GetBoolInput("Load global module <\033[1m" + sName + "\033[22m>?", false)) {
-					vsLines.push_back("LoadModule = " + sName);
+				if (CUtils::StdoutIsTTY()) {
+					if (CUtils::GetBoolInput("Load global module <\033[1m" + sName + "\033[22m>?", false))
+						vsLines.push_back("LoadModule = " + sName);
+				} else {
+					if (CUtils::GetBoolInput("Load global module <" + sName + ">?", false))
+						vsLines.push_back("LoadModule = " + sName);
 				}
 			}
 		}
@@ -737,8 +781,12 @@ bool CZNC::WriteNewConfig(CString& sConfigFile) {
 					const CModInfo& Info = *it;
 					CString sName = Info.GetName();
 
-					if (CUtils::GetBoolInput("Load " + CString((Info.IsSystem()) ? "system" : "local") + " module <\033[1m" + sName + "\033[22m>?", false)) {
-						vsLines.push_back("\tLoadModule = " + sName);
+					if (CUtils::StdoutIsTTY()) {
+						if (CUtils::GetBoolInput("Load module <\033[1m" + sName + "\033[22m>?", false))
+							vsLines.push_back("\tLoadModule = " + sName);
+					} else {
+						if (CUtils::GetBoolInput("Load module <" + sName + ">?", false))
+							vsLines.push_back("\tLoadModule = " + sName);
 					}
 				}
 			}
@@ -796,25 +844,25 @@ bool CZNC::WriteNewConfig(CString& sConfigFile) {
 	CFile File;
 	bool bFileOK, bFileOpen = false;
 	do {
-		CUtils::PrintAction("Writing config [" + sConfigFile + "]");
+		CUtils::PrintAction("Writing config [" + m_sConfigFile + "]");
 
 		bFileOK = true;
-		if (CFile::Exists(sConfigFile)) {
-			if (!m_LockFile.TryExLock(sConfigFile)) {
+		if (CFile::Exists(m_sConfigFile)) {
+			if (!m_LockFile.TryExLock(m_sConfigFile)) {
 				CUtils::PrintStatus(false, "ZNC is currently running on this config.");
 				bFileOK = false;
 			} else {
 				m_LockFile.Close();
 				CUtils::PrintStatus(false, "This config already exists.");
 				if (CUtils::GetBoolInput("Would you like to overwrite it?", false))
-					CUtils::PrintAction("Overwriting config [" + sConfigFile + "]");
+					CUtils::PrintAction("Overwriting config [" + m_sConfigFile + "]");
 				else
 					bFileOK = false;
 			}
 		}
 
 		if (bFileOK) {
-			File.SetFileName(sConfigFile);
+			File.SetFileName(m_sConfigFile);
 			if (File.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
 				bFileOpen = true;
 			} else {
@@ -823,11 +871,11 @@ bool CZNC::WriteNewConfig(CString& sConfigFile) {
 			}
 		}
 		if (!bFileOK) {
-			CUtils::GetInput("Please specify an alternate location (or \"stdout\" for displaying the config)", sConfigFile, sConfigFile);
-			if (sConfigFile.Equals("stdout"))
+			CUtils::GetInput("Please specify an alternate location (or \"stdout\" for displaying the config)", m_sConfigFile, m_sConfigFile);
+			if (m_sConfigFile.Equals("stdout"))
 				bFileOK = true;
 			else
-				sConfigFile = ExpandConfigPath(sConfigFile);
+				m_sConfigFile = ExpandConfigPath(m_sConfigFile);
 		}
 	} while (!bFileOK);
 
@@ -933,6 +981,10 @@ bool CZNC::DoRehash(CString& sError)
 		return false;
 	}
 
+	// (re)open the config file
+	if (m_LockFile.IsOpen())
+		m_LockFile.Close();
+
 	if (!m_LockFile.Open(m_sConfigFile)) {
 		sError = "Can not open config file";
 		CUtils::PrintStatus(false, sError);
@@ -945,7 +997,7 @@ bool CZNC::DoRehash(CString& sError)
 		return false;
 	}
 
-	CFile File(m_LockFile.GetFD(), m_sConfigFile);
+	CFile &File = m_LockFile;
 
 	// This fd is re-used for rehashing, so we must seek back to the beginning!
 	if (!File.Seek(0)) {
@@ -1039,7 +1091,7 @@ bool CZNC::DoRehash(CString& sError)
 							if (!pRealUser->Clone(*pUser, sErr)
 									|| !AddUser(pRealUser, sErr)) {
 								sError = "Invalid user [" + pUser->GetUserName() + "] " + sErr;
-								DEBUG_ONLY(cout << "CUser::Clone() failed in rehash" << endl);
+								DEBUG("CUser::Clone() failed in rehash");
 							}
 							pUser->SetBeingDeleted(true);
 							delete pUser;
@@ -1132,7 +1184,7 @@ bool CZNC::DoRehash(CString& sError)
 			if (pUser) {
 				if (pChan) {
 					if (sName.Equals("Buffer")) {
-						pChan->SetBufferCount(strtoul(sValue.c_str(), NULL, 10));
+						pChan->SetBufferCount(sValue.ToUInt());
 						continue;
 					} else if (sName.Equals("KeepBuffer")) {
 						pChan->SetKeepBuffer(sValue.Equals("true"));
@@ -1154,7 +1206,7 @@ bool CZNC::DoRehash(CString& sError)
 					}
 				} else {
 					if (sName.Equals("Buffer")) {
-						pUser->SetBufferCount(strtoul(sValue.c_str(), NULL, 10));
+						pUser->SetBufferCount(sValue.ToUInt());
 						continue;
 					} else if (sName.Equals("KeepBuffer")) {
 						pUser->SetKeepBuffer(sValue.Equals("true"));
@@ -1354,7 +1406,7 @@ bool CZNC::DoRehash(CString& sError)
 						sIPV6Comment = " using ipv6";
 					}
 
-					unsigned short uPort = strtol(sPort.c_str(), NULL, 10);
+					unsigned short uPort = sPort.ToUShort();
 					CUtils::PrintAction("Binding to port [" + CString((bSSL) ? "+" : "") + CString(uPort) + "]" + sHostComment + sIPV6Comment);
 
 #ifndef HAVE_IPV6
@@ -1399,7 +1451,7 @@ bool CZNC::DoRehash(CString& sError)
 					CListener* pListener = new CListener(uPort, sBindHost, bSSL, bIPV6);
 
 					if (!pListener->Listen()) {
-						sError = "Unable to bind";
+						sError = "Unable to bind [" + CString(strerror(errno)) + "]";
 						CUtils::PrintStatus(false, sError);
 						delete pListener;
 						return false;
@@ -1450,6 +1502,9 @@ bool CZNC::DoRehash(CString& sError)
 				} else if (sName.Equals("ConnectDelay")) {
 					m_uiConnectDelay = sValue.ToUInt();
 					continue;
+				} else if (sName.Equals("AnonIPLimit")) {
+					m_uiAnonIPLimit = sValue.ToUInt();
+					continue;
 				}
 			}
 
@@ -1553,8 +1608,6 @@ bool CZNC::DoRehash(CString& sError)
 		delete pUser;
 	}
 
-	File.Close();
-
 	if (m_msUsers.size() == 0) {
 		sError = "You must define at least one user in your config.";
 		CUtils::PrintError(sError);
@@ -1670,13 +1723,18 @@ bool CZNC::DeleteUser(const CString& sUsername) {
 }
 
 bool CZNC::AddUser(CUser* pUser, CString& sErrorRet) {
-	if (pUser->IsValid(sErrorRet)) {
-		m_msUsers[pUser->GetUserName()] = pUser;
-		return true;
+	if (FindUser(pUser->GetUserName()) != NULL) {
+		sErrorRet = "User already exists";
+		DEBUG("User [" << pUser->GetUserName() << "] - already exists");
+		return false;
 	}
-
-	DEBUG_ONLY(cout << "Invalid user [" << pUser->GetUserName() << "] - [" << sErrorRet << "]" << endl);
-	return false;
+	if (!pUser->IsValid(sErrorRet)) {
+		DEBUG("Invalid user [" << pUser->GetUserName() << "] - ["
+				<< sErrorRet << "]");
+		return false;
+	}
+	m_msUsers[pUser->GetUserName()] = pUser;
+	return true;
 }
 
 CZNC& CZNC::Get() {
@@ -1684,23 +1742,46 @@ CZNC& CZNC::Get() {
 	return *pZNC;
 }
 
-void CZNC::UpdateTrafficStats() {
-	CSockManager* p = &m_Manager;
-	for (unsigned int a = 0; a < p->size(); a++) {
-		if ((*p)[a]->GetSockName().Left(5) == "IRC::") {
-			CIRCSock *i = (CIRCSock *)(*p)[a];
-			i->GetUser()->AddBytesRead((*p)[a]->GetBytesRead());
-			(*p)[a]->ResetBytesRead();
-			i->GetUser()->AddBytesWritten((*p)[a]->GetBytesWritten());
-			(*p)[a]->ResetBytesWritten();
-		} else if ((*p)[a]->GetSockName().Left(5) == "USR::") {
-			CClient *c = (CClient *)(*p)[a];
-			c->GetUser()->AddBytesRead((*p)[a]->GetBytesRead());
-			(*p)[a]->ResetBytesRead();
-			c->GetUser()->AddBytesWritten((*p)[a]->GetBytesWritten());
-			(*p)[a]->ResetBytesWritten();
+CZNC::TrafficStatsMap CZNC::GetTrafficStats(TrafficStatsPair &Users,
+			TrafficStatsPair &ZNC, TrafficStatsPair &Total) {
+	TrafficStatsMap ret;
+	unsigned long long uiUsers_in, uiUsers_out, uiZNC_in, uiZNC_out;
+	const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
+
+	uiUsers_in = uiUsers_out = 0;
+	uiZNC_in  = BytesRead();
+	uiZNC_out = BytesWritten();
+
+	for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+		ret[it->first] = TrafficStatsPair(it->second->BytesRead(), it->second->BytesWritten());
+		uiUsers_in  += it->second->BytesRead();
+		uiUsers_out += it->second->BytesWritten();
+	}
+
+	for (CSockManager::const_iterator it = m_Manager.begin(); it != m_Manager.end(); it++) {
+		if ((*it)->GetSockName().Left(5) == "IRC::") {
+			CIRCSock *p = (CIRCSock *) *it;
+			ret[p->GetUser()->GetUserName()].first  += p->GetBytesRead();
+			ret[p->GetUser()->GetUserName()].second += p->GetBytesWritten();
+			uiUsers_in  += p->GetBytesRead();
+			uiUsers_out += p->GetBytesWritten();
+		} else if ((*it)->GetSockName().Left(5) == "USR::") {
+			CClient *p = (CClient *) *it;
+			ret[p->GetUser()->GetUserName()].first  += p->GetBytesRead();
+			ret[p->GetUser()->GetUserName()].second += p->GetBytesWritten();
+			uiUsers_in  += p->GetBytesRead();
+			uiUsers_out += p->GetBytesWritten();
+		} else {
+			uiZNC_in  += (*it)->GetBytesRead();
+			uiZNC_out += (*it)->GetBytesWritten();
 		}
 	}
+
+	Users = TrafficStatsPair(uiUsers_in, uiUsers_out);
+	ZNC   = TrafficStatsPair(uiZNC_in, uiZNC_out);
+	Total = TrafficStatsPair(uiUsers_in + uiZNC_in, uiUsers_out + uiZNC_out);
+
+	return ret;
 }
 
 void CZNC::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
@@ -1714,10 +1795,6 @@ void CZNC::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
 	CUser* pUser = GetUser(AuthClass->GetUsername());
 
 	if (!pUser || !pUser->CheckPass(AuthClass->GetPassword())) {
-		if (pUser) {
-			pUser->PutStatus("Another client attempted to login as you, with a bad password.");
-		}
-
 		AuthClass->RefuseLogin("Invalid Password");
 		return;
 	}
@@ -1784,8 +1861,7 @@ protected:
 			// The timer runs until it once didn't find any users to connect
 			bUsersLeft = true;
 
-			DEBUG_ONLY(cout << "Connecting user [" << pUser->GetUserName()
-					<< "]" << endl);
+			DEBUG("Connecting user [" << pUser->GetUserName() << "]");
 
 			if (CZNC::Get().ConnectUser(pUser))
 				// User connecting, wait until next time timer fires
@@ -1793,7 +1869,7 @@ protected:
 		}
 
 		if (bUsersLeft == false) {
-			DEBUG_ONLY(cout << "ConnectUserTimer done" << endl);
+			DEBUG("ConnectUserTimer done");
 			CZNC::Get().DisableConnectUser();
 		}
 	}
